# setupModel.py
#
# Simple script to demonstrate selecting and setting up of 
# the Current Model
#
# The script selects the Current Model, sets its Parameters,
# and links Functions to Results that are not provided by the Model.
# Some Results are requested after finilizing the configuration.

import modellib

AU = 1.49598e11    # AU to

# Select the Current Model:
modellib.setCurrentModel('Shu77')

# Set all parameters:
modellib.setParamDouble( 'time',  1.e5)
modellib.setParamDouble( 'T',     30.)

# Shu77 does not provide the results
# 'abundance', 'bmag', and 'doppler'. Hence these
# Results have to be provided by Functions

# Constant abundance:
modellib.setFunction('abundance', 'scalarConst')
modellib.setFunctionParamDouble('abundance', 'val', 1.e-9)

# Constant doppler (100 m/s):
modellib.setFunction('doppler', 'scalarConst')
modellib.setFunctionParamDouble('doppler', 'val', 100.)

# Zero bmag:
modellib.setFunction('bmag', 'vectorConstR')
modellib.setFunctionParamDouble('bmag', 'val', 0.)

# Done:
modellib.finalizeConfiguration()

# Request some Results:
x, y, z = 100.*AU, 0., 0
print
print "Some Results at r=100AU:"
print
print "density:     %.2f m^-3" % modellib.density(x, y, z)
print "temperature: %.2f K" % modellib.temperature(x, y, z)
print "abundance:   %.2e" % modellib.abundance(x, y, z)
print "bmag:        (%.2f, %.2f, %.2f) T" % modellib.bmag(x, y, z)
print "velocity:    (%.2f, %.2f, %.2f) m/s" % modellib.velocity(x, y, z)
print
