################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
from matplotlib.pylab import cm
from matplotlib import colors
import sys
def casa_colors():
    
    cdict = {'red': ((0.0, 0.0, 0.0),  
                     (0.2, 0.0888, 0.0888),
                     (0.4, 0.0888, 0.0888),
                     (0.5, 0., 0.),
                     (0.6, 0.9405, 0.9405),
                     (0.8, 1.0, 1.0),
                     (0.9, 1.0, 1.0),
                     (1.0, 1.0, 1.0)),
             'green': ((0.0, 0.0, 0.0),
                       (0.2, 0.3061, 0.3061),
                       (0.4, 0.7815, 0.7815),
                       (0.5, 1.0, 1.0),
                       (0.6, 0.9191, 0.9191),
                       (0.8, 0.0, 0.0),
                       (0.9, 0.3568, 0.3568),
                       (1.0, 1.0, 1.0)),
             'blue': ((0.0, 0.0, 0.0),
                      (0.2, 0.8259, 0.8259),
                       (0.4, 1.0, 1.0),
                       (0.5, 0., 0.),
                       (0.6, 0., 0.),
                       (0.8, 0., 0.),
                       (0.9, 1.0, 1.0),
                      (1.0, 1.0, 1.0))}
             
    colvel = colors.LinearSegmentedColormap('colvel',cdict,256)

    return colvel

def color_velocity():
    cdict = {'red': ((0.0, 1.0, 1.0),
                     (0.5, 0.0, 0.0),
                     (1.0, 0.0, 0.0)),
             'green': ((0.0, 0.0, 0.0),
                       (0.5, 1.0, 1.0),
                       (1.0, 0.0, 0.0)),
             'blue': ((0.0, 0.0, 0.0),
                      (0.5, 0.0, 0.0),
                      (1.0, 1.0, 1.0))}
             
    colvel = colors.LinearSegmentedColormap('colvel',cdict,256)

    return colvel


def blue1():
    cdict = {'red': ((0.0, 0.0, 0.0),
                     (0.6, 0.0, 0.0),
                     (1.0, 1.0, 1.0)),
             'green': ((0.0, 0.0, 0.0),
                       (0.6, 0.0, 0.0),
                       (1.0, 1.0, 1.0)),
             'blue': ((0.0, 0.0, 0.0),
                      (0.6, 1.0, 1.0),
                      (1.0, 1.0, 1.0))}
             
    mcm_blue1 = colors.LinearSegmentedColormap('mcm_blue1',cdict,256)

    return mcm_blue1

class GridColor():
    def __init__(self, parent=None, currentColor=None, currentColorMap=None, isCurrentSingleColor=False):

        self.colorMaps={'binary':cm.binary, 'Blue':blue1(), 'BrBG':cm.BrBG, 'RdBu':cm.RdBu, 'hot':cm.hot,\
                'hsv':cm.hsv, 'jet':cm.jet,'spectral':cm.spectral, 'color_velocity':color_velocity(),\
                'casa':casa_colors()}
        
        if currentColor:
            self.currentColor=currentColor
        else:
            self.currentColor='#FFFFFF'


        if currentColorMap:
            self.currentColorMap=currentColorMap
        else:
            self.currentColorMap='jet'

        self.currentColorMapID = self.colorMaps.keys().index(self.currentColorMap)

        if isCurrentSingleColor:
            self.isCurrentSingleColor=True
        else:
            self.isCurrentSingleColor=False

    def setCurrentColor(self, colorName=''):
        """
            Sets the current color to a single color with name 'colorName'
        """
        self.currentColor=colorName
        self.isCurrentSingleColor=True
    
    def setCurrentColorMap(self, colorMapName='', colorMapID=0):
        """
            Sets the current color to a color map with name 'colorMapName'
        """
        if colorMapName!='':
            self.currentColorMap=colorMapName
            self.isCurrentSigleColor=False
            self.currentColorMapID = self.colorMaps.keys().index(self.currentColorMap)
        else:
            self.currentColorMapID = colorMapID
            self.currentColorMap = self.colorMaps[self.colorMaps.keys()[colorMapID]]
            self.isCurrentSigleColor=False
        

class Grid():
    """
        Class for grid/mesh 
    """
    def __init__(self, parent=None, minval=0., maxval=0., nval=0., gridstyle='lin'):

        self.minval    = minval
        self.maxval    = maxval
        self.nval      = nval
        self.gridstyle = gridstyle
        self.val       = []
        if nval!=0:
            self.makegrid()

    def makegrid(self, minval=None, maxval=None, nval=None, gridstyle=None):
        """
            Creates a linear/logarithmic grid
        """
        if minval!=None:
            self.minval = minval

        if maxval!=None:
            self.maxval = maxval

        if nval!=None:
            self.nval = nval

        if gridstyle!=None:
            self.gridstyle = gridstyle

        if self.gridstyle=='lin':
            self.val = self.minval + (self.maxval - self.minval) * (np.arange(self.nval, dtype=np.float64) / (self.nval - 1e0))
        if self.gridstyle=='log':
            self.val = self.minval * (self.maxval / self.minval) ** (np.arange(self.nval, dtype=np.float64) / (self.nval - 1e0))

        

