################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################



import sys, os
import numpy as npy
import modellib
from natconst import *
class GridCart2D():
    def __init__(self,xmin=None,xmax=None,nx=None,xl=None,xpos=None,xval=0.0,
                 ymin=None,ymax=None,ny=None,yl=None,ypos=None,yval=0.0,
                 zmin=None,zmax=None,nz=None,zl=None,zpos=None,zval=0.0):

        

        xmin = 0.0
        ymin = 0.0
        zmin = 0.0

        if (ymin==None):
            ymin = xmin
            ymax = xmax
            ny   = nx
        if (zmin==None):
            zmin = xmin
            zmax = xmax
            nz   = nx
        if (yl==None):
            yl = xl
        if (zl==None):
            zl = xl

        self.nx = nx
        self.ny = ny
        self.nz = nz

        
        if (nx==1):
            self.x = npy.arange(nx)*0.0 + xval
        else:
            if (xl==None):
                if (xpos!=None):
                    #self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax
                    self.x = npy.arange(nx) / (nx-1.) * (xmax - xmin) + xmin
                else:
                    self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax
                self.xi       = npy.zeros(nx+1, dtype=npy.float64)
                self.xi[1:-1] = (self.x[1:]+self.x[:-1]) * 0.5
                self.xi[0]    = self.xi[1] + self.xi[1] - self.xi[2]
                self.xi[nx]   = self.xi[nx-1] + self.xi[nx-1] - self.xi[nx-2]
            else:
                self.x = 10.**(npy.arange(nx) / (nx-1.) * (npy.log10(xmax) - npy.log10(xmin)) +\
                                      npy.log10(xmin))
                self.xi       = npy.zeros(nx+1, dtype=npy.float64)
                self.xi[1:-1] = sqrt(self.x[1:]*self.x[:-1]) 
                self.xi[0]    = self.xi[1] * self.xi[1] / self.xi[2]
                self.xi[nx]   = self.xi[nx-1] * self.xi[nx-1] / self.xi[nx-2]

        if (ny==1):
            self.y = npy.arange(ny)*0.0 + yval
        else:
            if (yl==None):
                if (ypos!=None):
                    #self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
                    self.y = npy.arange(ny) / (ny-1.) * (ymax - ymin) + ymin
                else:
                    self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
                self.yi       = npy.zeros(ny+1, dtype=npy.float64)
                self.yi[1:-1] = (self.y[1:]+self.y[:-1]) * 0.5
                self.yi[0]    = self.yi[1] + self.yi[1] - self.yi[2]
                self.yi[ny]   = self.yi[ny-1] + self.yi[ny-1] - self.yi[ny-2]
            else:
                self.y = 10.**(npy.arange(ny) / (ny-1.) * (npy.log10(ymax) - npy.log10(ymin)) +\
                                  npy.log10(ymin))

                self.yi       = npy.zeros(ny+1, dtype=npy.float64)
                self.yi[1:-1] = sqrt(self.y[1:]*self.y[:-1]) 
                self.yi[0]    = self.yi[1] * self.yi[1] / self.yi[2]
                self.yi[ny]   = self.yi[ny-1] * self.yi[ny-1] / self.yi[ny-2]

            
        if (nz==1):
            self.z = npy.arange(nz)*0.0 + zval
        else:
            if (zl==None):
                if (zpos!=None):
                    #self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
                    self.z = npy.arange(nz) / (nz-1.) * (zmax - zmin) + zmin
                else:
                    self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
                self.zi       = npy.zeros(nz+1, dtype=npy.float64)
                self.zi[1:-1] = (self.z[1:]+self.z[:-1]) * 0.5
                self.zi[0]    = self.zi[1] + self.zi[1] - self.zi[2]
                self.zi[nz]   = self.zi[nz-1] + self.zi[nz-1] - self.zi[nz-2]

            else:
                self.z = 10.**(npy.arange(nz) / (nz-1.) * (npy.log10(zmax) - npy.log10(zmin)) +\
                                  npy.log10(zmin))  
                self.zi       = npy.zeros(nz+1, dtype=npy.float64)
                self.zi[1:-1] = sqrt(self.z[1:]*self.z[:-1]) 
                self.zi[0]    = self.zi[1] * self.zi[1] / self.zi[2]
                self.zi[nz]   = self.zi[nz-1] * self.zi[nz-1] / self.zi[nz-2]

class GridCart():
    def __init__(self,xmin=None,xmax=None,nx=None,xl=None,xpos=None,
                 ymin=None,ymax=None,ny=None,yl=None,ypos=None,
                 zmin=None,zmax=None,nz=None,zl=None,zpos=None):

        
        xmin = 0.0
        ymin = 0.0
        zmin = 0.0

        if (ymin==None):
            ymin = xmin
            ymax = xmax
            ny   = nx
        if (zmin==None):
            zmin = xmin
            zmax = xmax
            nz   = nx
        if (yl==None):
            yl = xl
        if (zl==None):
            zl = xl

        self.nx = nx
        self.ny = ny
        self.nz = nz

        
        if (xl==None):
            if (xpos!=None):
                self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax
            else:
                self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax

            self.xi       = npy.zeros(nx+1, dtype=npy.float64)
            self.xi[1:-1] = (self.x[1:]+self.x[:-1]) * 0.5
            self.xi[0]    = self.xi[1] + self.xi[1] - self.xi[2]
            self.xi[nx]   = self.xi[nx-1] + self.xi[nx-1] - self.xi[nx-2]
        else:
            self.x = 10.**(npy.arange(nx) / (nx-1.) * (npy.log10(xmax) - npy.log10(xmin)) +\
                                  npy.log10(xmin))
            self.xi       = npy.zeros(nx+1, dtype=npy.float64)
            self.xi[1:-1] = sqrt(self.x[1:]*self.x[:-1]) 
            self.xi[0]    = self.xi[1] * self.xi[1] / self.xi[2]
            self.xi[nx]   = self.xi[nx-1] * self.xi[nx-1] / self.xi[nx-2]

        if (yl==None):
            if (ypos!=None):
                self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
            else:
                self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
            
            self.yi       = npy.zeros(ny+1, dtype=npy.float64)
            self.yi[1:-1] = (self.y[1:]+self.y[:-1]) * 0.5
            self.yi[0]    = self.yi[1] + self.yi[1] - self.yi[2]
            self.yi[ny]   = self.yi[ny-1] + self.yi[ny-1] - self.yi[ny-2]
        else:
            self.y = 10.**(npy.arange(ny) / (ny-1.) * (npy.log10(ymax) - npy.log10(ymin)) +\
                                  npy.log10(ymin))
            self.yi       = npy.zeros(ny+1, dtype=npy.float64)
            self.yi[1:-1] = sqrt(self.y[1:]*self.y[:-1]) 
            self.yi[0]    = self.yi[1] * self.yi[1] / self.yi[2]
            self.yi[ny]   = self.yi[ny-1] * self.yi[ny-1] / self.yi[ny-2]

        if (zl==None):
            if (zpos!=None):
                self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
            else:
                self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
            self.zi       = npy.zeros(nz+1, dtype=npy.float64)
            self.zi[1:-1] = (self.z[1:]+self.z[:-1]) * 0.5
            self.zi[0]    = self.zi[1] + self.zi[1] - self.zi[2]
            self.zi[nz]   = self.zi[nz-1] + self.zi[nz-1] - self.zi[nz-2]
        else:
            self.z = 10.**(npy.arange(nz) / (nz-1.) * (npy.log10(zmax) - npy.log10(zmin)) +\
                                  npy.log10(zmin))  
            
            self.zi       = npy.zeros(nz+1, dtype=npy.float64)
            self.zi[1:-1] = sqrt(self.z[1:]*self.z[:-1]) 
            self.zi[0]    = self.zi[1] * self.zi[1] / self.zi[2]
            self.zi[nz]   = self.zi[nz-1] * self.zi[nz-1] / self.zi[nz-2]


class GridSph():
    def __init__(self,rmin=None,rmax=None,nr=None,rl=None,
                 tmin=None,tmax=None,nt=None,
                 pmin=None,pmax=None,np=None):
        
        self.nr  = nr
        self.nt  = nt
        self.np  = np
        if (rl==True):
            self.r    = 10.**(npy.arange(nr*1.0) / (nr-1.) * (npy.log10(rmax) - npy.log10(rmin)) +\
                                  npy.log10(rmin))
        else:
            self.r    = npy.arange(nr*1.0) / (nr-1.) * (rmax-rmin) + rmin

        self.theta  = npy.arange(nt*1.0) / (nt-1.) * (tmax-tmin) + tmin
        self.phi  = npy.arange(np*1.0) / (np-1.) * (pmax-pmin) + pmin
    

def getPlotModelVar( GridPar, model_id=0,  varname=' ', defaultPar=False, UnitSystem='SI'):
    nc      = natconst()

    grid_type = 0        
    if (grid_type==0):
        grid = GridCart2D(xmin=GridPar.xmin,xmax=GridPar.xmax,nx=GridPar.nx,xpos=GridPar.xpos,xval=GridPar.xval,
                                ymin=GridPar.ymin,ymax=GridPar.ymax,ny=GridPar.ny,ypos=GridPar.ypos,yval=GridPar.yval,
                                zmin=GridPar.zmin,zmax=GridPar.zmax,nz=GridPar.nz,zpos=GridPar.zpos,zval=GridPar.zval)

        res = {}
        res['var'] = npy.zeros((grid.nx,grid.ny,grid.nz),dtype=npy.float64)        
        res['x'] = grid.x
        res['y'] = grid.y
        res['z'] = grid.z
        res['nx'] = GridPar.nx
        res['ny'] = GridPar.ny
        res['nz'] = GridPar.nz
    else:
        print 'Not supported grid style'
        sys.exit()

    
    if UnitSystem=='CGS':
        unitconv = {'density':1e-6, 'temperature':1.0, 'velocity':1e2, 'bmag':1e-4, 'column_density':1e-4}
    else:
        unitconv = {'density':1.0, 'temperature':1.0, 'velocity':1.0, 'bmag':1.0, 'column_density':1.0}

            
    if (grid_type==0):
        if (varname=='density'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        res['var'][ix,iy,iz]  = modellib.density(grid.x[ix],grid.y[iy],grid.z[iz]) * unitconv['density']
        elif (varname=='temperature'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        res['var'][ix,iy,iz]  = modellib.temperature(grid.x[ix],grid.y[iy],grid.z[iz]) * unitconv['temperature']
        elif (varname=='velocity_x'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        dum  = modellib.velocity(grid.x[ix],grid.y[iy],grid.z[iz])
                        res['var'][ix,iy,iz] = dum[0] * unitconv['velocity']
        elif (varname=='velocity_y'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        dum  = modellib.velocity(grid.x[ix],grid.y[iy],grid.z[iz]) 
                        res['var'][ix,iy,iz] = dum[1] * unitconv['velocity']
        elif (varname=='velocity_z'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        dum  = modellib.velocity(grid.x[ix],grid.y[iy],grid.z[iz])
                        res['var'][ix,iy,iz] = dum[2] * unitconv['velocity']
        elif (varname=='abundance'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        res['var'][ix,iy,iz] = modellib.abundance(grid.x[ix],grid.y[iy],grid.z[iz])

        elif (varname=='tdust'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        res['var'][ix,iy,iz] = modellib.t_dust(grid.x[ix],grid.y[iy],grid.z[iz]) * unitconv['temperature']

        elif (varname=='doppler'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        res['var'][ix,iy,iz] = modellib.doppler(grid.x[ix],grid.y[iy],grid.z[iz]) * unitconv['velocity']

        elif (varname=='bmag_x'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        dum = modellib.bmag(grid.x[ix],grid.y[iy],grid.z[iz])
                        res['var'][ix,iy,iz] = dum[0] * unitconv['bmag']

        elif (varname=='bmag_y'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        dum = modellib.bmag(grid.x[ix],grid.y[iy],grid.z[iz])
                        res['var'][ix,iy,iz] = dum[1] * unitconv['bmag']

        elif (varname=='bmag_z'):
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        dum = modellib.bmag(grid.x[ix],grid.y[iy],grid.z[iz])
                        res['var'][ix,iy,iz] = dum[2] * unitconv['bmag']

        elif (varname=='column_density_x'):

            if grid.nx==1:
                nxi   = 300
                xmin  = -GridPar.xmax
                xi    = npy.arange(nxi) / (float(nxi)-1.) * (GridPar.xval - xmin) + xmin
                x     = 0.5 * (xi[:1] + xi[:-1])
                dx    = xi[1:] - xi[:-1]

                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        dummy = 0.0
                        for ix in range(nxi-1):
                            dummy = dummy + modellib.density(x[ix], grid.y[iy], grid.z[iz]) * unitconv['column_density'] * dx[ix]
                        res['var'][0,iy,iz] = dummy

            else:
                nxi   = grid.nx+1
                xi    = grid.xi
                x     = grid.x
                dx    = xi[1:] - xi[:-1]
                
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        res['var'][0,iy,iz] = modellib.density(x[0], grid.y[iy], grid.z[iz]) * unitconv['column_density'] * dx[0]
                        for ix in range(1,nxi-1):
                            res['var'][ix,iy,iz] = res['var'][ix-1,iy,iz] + modellib.density(x[ix], grid.y[iy], grid.z[iz]) * unitconv['column_density'] * dx[ix]


        elif (varname=='column_density_y'):

            if grid.ny==1:
                nyi   = 300
                ymin  = -GridPar.ymax
                yi    = npy.arange(nyi) / (float(nyi)-1.) * (GridPar.yval - ymin) + ymin
                y     = 0.5 * (yi[:1] + yi[:-1]) 
                dy    = yi[1:] - yi[:-1]
                
                for ix in range(grid.nx):
                    for iz in range(grid.nz):
                        dummy = 0.0
                        for iy in range(nyi-1):
                            dummy = dummy + modellib.density(grid.x[ix], y[iy], grid.z[iz]) * unitconv['column_density'] * dy[iy]
                        res['var'][ix,0,iz] = dummy


            else:
                nyi   = grid.ny+1
                yi    = grid.yi
                y     = grid.y
                dy    = yi[1:] - yi[:-1]

                for ix in range(grid.nx):
                    for iz in range(grid.nz):
                        res['var'][ix,0,iz] = modellib.density(grid.x[ix], y[0], grid.z[iz]) * unitconv['column_density'] * dy[0]
                        for iy in range(1,nyi-1):
                            res['var'][ix,iy,iz] = res['var'][ix,iy-1,iz] + \
                                    modellib.density(grid.x[ix], y[iy], grid.z[iz]) * unitconv['column_density'] * dy[iy]
                
        elif (varname=='column_density_z'):

            if grid.nz==1:
                nzi   = 300
                zmin  = -GridPar.zmax
                zi    = npy.arange(nzi) / (float(nzi)-1.) * (GridPar.zval - zmin) + zmin
                z     = 0.5 * (zi[1:] + zi[:-1]) 
                dz    = zi[1:] - zi[:-1]

                
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        dummy = 0.0
                        for iz in range(nzi-1):
                            dummy = dummy + modellib.density(grid.x[ix], grid.y[iy], z[iz]) * unitconv['column_density'] * dz[iz]
                        res['var'][ix,iy,0] = dummy

            else:
                nzi   = grid.nz+1
                zi    = grid.zi
                z     = grid.z
                dz    = zi[1:] - zi[:-1]

                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        res['var'][ix,iy,0] = modellib.density(grid.x[ix], grid.y[iy], z[0]) * unitconv['column_density'] * dz[0]
                        for iz in range(1,nzi-1):
                            res['var'][ix,iy,iz] = res['var'][ix,iy,iz-1] + \
                                    modellib.density(grid.x[ix], grid.y[iy], z[iz]) * unitconv['column_density'] * dz[iz]
        else:
            for ix in range(grid.nx):
                for iy in range(grid.ny):
                    for iz in range(grid.nz):
                        res['var'][ix][iy][iz] = 0.0

    return res


def setDefaultParamValues(ModelID):
        
    paramIDs = modellib.getParamIDs(ModelID)

    for i in range(len(paramIDs)):
        paramType = modellib.getParamType(ModelID, paramIDs[i])
        
        if paramType=='int':
            modellib.setParamInt(paramIDs[i], modellib.getParamDefValInt(ModelID, paramIDs[i]))

        if paramType=='double':
            modellib.setParamDouble(paramIDs[i], modellib.getParamDefValDouble(ModelID, paramIDs[i]))

        if paramType=='enum':
            modellib.setParamEnumIndex(paramIDs[i], modellib.getParamEnumDefIndex(ModelID, paramIDs[i]))

        if paramType=='string':
            if paramIDs[i]=='dustopac_fname':
                modellib.setParamString(paramIDs[i], os.environ['ARTIST_ROOT']+'/data/jena_thin_e6.tab')
            else:
                modellib.setParamString(paramIDs[i], modellib.getParamDefValString(ModelID, paramIDs[i]))

    return


