################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
from PyQt4 import QtCore, QtGui
from artist_gui.GridDialogWindowBase import Ui_GridDialog
from artist_gui.helper_functions import *

class GridDialog(QtGui.QDialog, Ui_GridDialog):
    """
    Grid Dialog class :
        Provides a dialog to set up a 1D grid 
    """

    ApplyGrid = QtCore.pyqtSignal(object,object,str,int,name='ApplyGrid')
    ApplyGridOnlyGrid = QtCore.pyqtSignal(object,name='ApplyGrid')

    def __init__(self, parent=None, grid=None, varName=None, databounds=None, gridcolor=None, windowstyle=None):
        super(GridDialog, self).__init__(parent)
        self.setupUi(self)
        
        if varName!=None:
            self.setWindowTitle(varName+' Contour Grid')
        else:
            self.setWindowTitle('Grid')
        
        self.grid = grid
        self.varName = varName
        self.databounds=databounds
        self.gridcolor=gridcolor
        self.windowstyle=windowstyle
        self.valList = QtCore.QStringList() 
        self.valListModel = QtGui.QStringListModel()

        self.setGridParams()
       
        if self.windowstyle<2:
            self.GridDialogCMapBrowseCBox.addItems(self.gridcolor.colorMaps.keys())
            color = QtGui.QColor(0,0,0)
            color.setNamedColor(self.gridcolor.currentColor)
            self.GridDialogColorDisplayEdit.setPalette(QtGui.QPalette(color))
            self.GridDialogColorDisplayEdit.setAutoFillBackground(True)
        
        if self.windowstyle==0:
            self.GridDialogFitToDataButton.setVisible(True)
            self.GridDialogColorMapRButton.setVisible(True)
            self.GridDialogCMapBrowseCBox.setVisible(True) 

            index = self.GridDialogCMapBrowseCBox.findText(self.gridcolor.currentColorMap)
            self.GridDialogCMapBrowseCBox.setCurrentIndex(index)

            if self.gridcolor.isCurrentSingleColor==False:
                self.GridDialogColorMapRButton.setChecked(True)
            if self.gridcolor.isCurrentSingleColor==True:
                self.GridDialogSingleColorRButton.setChecked(True)

        elif self.windowstyle==1:
            self.GridDialogFitToDataButton.setVisible(True)
            self.GridDialogSingleColorRButton.setVisible(False)
            self.GridDialogColorDisplayEdit.setVisible(False)
            self.GridDialogColorBrowseButton.setVisible(False)
            self.GridDialogColorMapRButton.setChecked(True)
            self.GridDialogColorMapRButton.setVisible(True)
            self.GridDialogCMapBrowseCBox.setVisible(True) 
           
            index = self.GridDialogCMapBrowseCBox.findText(self.gridcolor.currentColorMap)
            self.GridDialogCMapBrowseCBox.setCurrentIndex(index)
        
        elif self.windowstyle==2:
            self.GridDialogFitToDataButton.setVisible(False)
            self.GridDialogSingleColorRButton.setVisible(False)
            self.GridDialogColorDisplayEdit.setVisible(False)
            self.GridDialogColorBrowseButton.setVisible(False)
            self.GridDialogColorMapRButton.setVisible(False)
            self.GridDialogCMapBrowseCBox.setVisible(False) 

        self.connect(self.GridDialogGenGridEdit,QtCore.SIGNAL("clicked()"), self.generateGrid)
        self.connect(self.GridDialogDoneButton,QtCore.SIGNAL("clicked()"), self.acceptAndExit)
        self.connect(self.GridDialogCancelButton,QtCore.SIGNAL("clicked()"), self.reject)
            
        self.connect(self.GridDialogRemoveItemButton,QtCore.SIGNAL("clicked()"), self.removeitems)
        self.connect(self.GridDialogAddItemButton,QtCore.SIGNAL("clicked()"), self.additems)
        self.connect(self.GridDialogFitToDataButton,QtCore.SIGNAL("clicked()"), self.fitBoundsToData)
        self.connect(self.GridDialogColorBrowseButton, QtCore.SIGNAL("clicked()"), self.browsecolors)
        self.connect(self.GridDialogCMapBrowseCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.changecolormap)
        self.connect(self.GridDialogColorMapRButton, QtCore.SIGNAL("clicked()"), self.changecolorstyle)
        self.connect(self.GridDialogSingleColorRButton, QtCore.SIGNAL("clicked()"), self.changecolorstyle)

        self.connect(self.GridDialogApplyButton,QtCore.SIGNAL("clicked()"), self.emitApplyGridSignal)
        
    def acceptAndExit(self):
        self.emitApplyGridSignal()
        self.accept()

    def emitApplyGridSignal(self):
        
        self.readOutGridParams()
        if self.windowstyle<2:
            self.ApplyGrid.emit(self.grid, self.gridcolor, str(self.varName).strip(), self.windowstyle) 
        else:
            self.ApplyGridOnlyGrid.emit(self.grid)


    
    def changecolorstyle(self):

        if self.windowstyle==0:
            if self.GridDialogColorMapRButton.isChecked():
                self.gridcolor.isCurrentSingleColor = False
            if self.GridDialogSingleColorRButton.isChecked():
                self.gridcolor.isCurrentSingleColor = True

    def changecolormap(self, cmap_ID=0):

        cmapName = str(self.GridDialogCMapBrowseCBox.currentText())
        self.gridcolor.currentColorMapID = self.gridcolor.colorMaps.keys().index(cmapName)
        self.gridcolor.currentColorMap = cmapName
    def browsecolors(self):
        color = QtGui.QColorDialog.getColor(QtCore.Qt.green, self)

        if color.isValid(): 
            self.GridDialogColorDisplayEdit.setPalette(QtGui.QPalette(color))
            self.GridDialogColorDisplayEdit.setAutoFillBackground(True)
            self.gridcolor.currentColor = str(color.name())
    def fitBoundsToData(self):
        self.grid.minval = self.databounds[0]
        self.grid.maxval = self.databounds[1]
        
        self.GridDialogMinvalEdit.setText(str(self.grid.minval)) 
        self.GridDialogMaxvalEdit.setText(str(self.grid.maxval)) 
        self.generateGrid()
    
        if self.windowstyle<2:
            color = QtGui.QColor(0,0,0)
            color.setNamedColor(self.gridcolor.currentColor)
            self.GridDialogColorDisplayEdit.setPalette(QtGui.QPalette(color))
            self.GridDialogColorDisplayEdit.setAutoFillBackground(True)

    def additems(self):
        self.valList.append(QtCore.QString('1.0'))
        self.valListModel.setStringList(self.valList)
        self.GridDialogValListView.setModel(self.valListModel)

    def removeitems(self):
        val_tobe_removed = []
        for index in self.GridDialogValListView.selectedIndexes():
            val_tobe_removed.append(self.valList[index.row()])
        for value in val_tobe_removed:
            index = self.valList.indexOf(value)
            self.valList.removeAt(index)
        self.valListModel.setStringList(self.valList)
        self.GridDialogValListView.setModel(self.valListModel)

    def generateGrid(self):
        self.readOutGridParams()
        self.grid.makegrid()
        self.valList.clear()
        self.valListModel=QtGui.QStringListModel()
        self.setGridParams()

    def setGridParams(self):         
        self.GridDialogMinvalEdit.setText(str(self.grid.minval)) 
        self.GridDialogMaxvalEdit.setText(str(self.grid.maxval)) 
        self.GridDialogNvalEdit.setText(str(self.grid.nval)) 
        if self.grid.gridstyle=='lin':
            self.GridDialogLinGridRButton.setChecked(True)
        if self.grid.gridstyle=='log':
            self.GridDialogLogGridRButton.setChecked(True)

        if self.windowstyle<2:
            color = QtGui.QColor(0,0,0)
            color.setNamedColor(self.gridcolor.currentColor)
            self.GridDialogColorDisplayEdit.setPalette(QtGui.QPalette(color))
            self.GridDialogColorDisplayEdit.setAutoFillBackground(True)

        for item in self.grid.val:
            self.valList.append(str(item))
        self.valListModel.setStringList(self.valList)
        self.GridDialogValListView.setModel(self.valListModel)
        
    def readOutGridParams(self):         
        self.grid.minval = str2float(str(self.GridDialogMinvalEdit.text()), 'Min. value (Contour Grid parameters)')
        self.grid.maxval = str2float(str(self.GridDialogMaxvalEdit.text()), 'Max. value (Contour Grid parameters)')
        self.grid.nval = str2float(str(self.GridDialogNvalEdit.text()), 'Number of values (Contour Grid parameters)')
        if self.GridDialogLinGridRButton.isChecked():
            self.grid.gridstyle='lin'
        elif self.GridDialogLogGridRButton.isChecked():
            self.grid.gridstyle='log'

        if self.windowstyle<2:
            if self.GridDialogColorMapRButton.isChecked():
                self.gridcolor.isCurrentSingleColor=False
            if self.GridDialogSingleColorRButton.isChecked():
                self.gridcolor.isCurrentSingleColor=True

        items = self.valListModel.stringList()
        val   = [str2float(str(i), 'Contour grid values') for i in items]
        self.grid.val   = np.array(val)

