################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
from PyQt4 import QtCore, QtGui
import os 

class ConfigurationParams():

    def __init__(self):

        self.artist_version = "1.1Beta"
        self.ivan_version = "1.1Beta"
        self.unitSystem = 'SI'
        self.autoupdateFigure = False
        self.paraViewCommand = 'paraview'
        self.mImagePolicy = 'Series'
        self.copyDataFilesToModelDir = True
        self.windowParams = {'font_family':'Helvetica', 'font_size':'11'}
        self.figureParams = {'bgcolor':'black', 'fgcolor':'white', 'selection_color':'white', 'distance_color':'white',\
                'font_size':'10', 'colorbar_fontsize':'10', 'distance_fontsize':'10'}

    def writeConfigurationParamsToFile(self, fName='', ARTIST=True):

        if fName!='':
            try:
                fName = os.environ['HOME']+'/'+fName
                wfile = open(fName, 'w')
            except Exception as e:
                print e
                return e

            wfile.write("%s\n"%"# CONFIGURATION FILE FOR ARTIST/IVAN")

            if ARTIST:
                wfile.write("%s\n"%"<ARTIST>")
                wfile.write("%s\n"%("version = "+self.artist_version))
                wfile.write("%s\n"%("unitSystem = "+self.unitSystem))
                if self.autoupdateFigure: 
                    wfile.write("%s\n"%("autoupdateFigure = True"))
                else:
                    wfile.write("%s\n"%("autoupdateFigure = False"))
                wfile.write("%s\n"%("paraViewCommand = "+self.paraViewCommand))
                wfile.write("%s\n"%("mImagePolicy = "+self.mImagePolicy))
                wfile.write("%s\n"%("copyDataFilesToModelDir = "+str(self.copyDataFilesToModelDir)))
            
            wfile.write("%s\n"%"<IVAN>")
            wfile.write("%s\n"%("version = "+self.ivan_version))
            wfile.write("%s\n"%"<windowParams>")
            for ikey in self.windowParams.keys():
                wfile.write("%s\n"%(ikey+'='+self.windowParams[ikey]))
            
            wfile.write("%s\n"%"<figureParams>")
            for ikey in self.figureParams.keys():
                wfile.write("%s\n"%(ikey+'='+self.figureParams[ikey]))
           
            wfile.close()
           
            return 0 

    def readConfigurationParamsFromFile(self,fName=''):

        if fName!='':
            try:
                fName = os.environ['HOME']+'/'+fName
                rfile = open(fName, 'r')
            except Exception as e:
                return e

            dum = rfile.readline()
            dum = rfile.readline().split('<')[1].split('>')[0]
            if dum=='ARTIST':
                dum = rfile.readline().split('=')[1]
                wrongVersion=True
                if dum.strip()=='1.0':
                    wrongVersion=False
                if dum.strip()=='1.1Beta':
                    wrongVersion=False
                if wrongVersion:
                    rfile.close()
                    return 1
                self.artist_version = dum.strip()
                self.unitSystem = rfile.readline().split('=')[1].strip()
                dum = rfile.readline().split('=')
                if dum[1].strip()=='True':
                    self.autoupdateFigure = True
                elif dum[1].strip()=='False':
                    self.autoupdateFigure = False

                self.paraViewCommand = rfile.readline().split('=')[1].strip()
                self.mImagePolicy = rfile.readline().split('=')[1].strip()
                dum = rfile.readline().split('=')[1].strip()

                if dum.strip()=='True':
                    self.copyDataFilesToModelDir = True
                elif dum.strip()=='False':
                    self.copyDataFilesToModelDir = False

                dum = rfile.readline().split('<')[1].split('>')[0].strip()

           
            if dum!='IVAN':
                rfile.close()
                return 1

            self.ivanVersion = rfile.readline().split('=')[1].strip()
            dum = rfile.readline().split('<')[1].split('>')[0].strip()

            if dum=='windowParams':
                for i in range(len(self.windowParams.keys())):
                    dum = rfile.readline().split('=')
                    self.windowParams[dum[0]] = dum[1].strip()


                dum = rfile.readline().split('<')[1].split('>')[0]
                if dum=='figureParams':
                    for i in range(len(self.figureParams.keys())):
                        dum = rfile.readline().split('=')
                        self.figureParams[dum[0]] = dum[1].strip()

                else:
                    rfile.close()
                    return 1


            elif dum=='figureParams':
                for i in range(len(self.figureParams.keys())):
                    dum = rfile.readline().split('=')
                    self.figureParams[dum[0]] = dum[1].strip()
            
                dum = rfile.readline().split('<')[1].split('>')[0]
                if dum=='windowParams':
                    for i in range(len(self.windowParams.keys())):
                        dum = rfile.readline().split('=')
                        self.windowParams[dum[0]] = dum[1].strip()
                else:
                    rfile.close()
                    return 1
            else:   
                rfile.close()
                return 1
            
            rfile.close()

            return 0 


def mynanmax(x=None):
    return x[np.isnan(x)==False].max()
def mynanmin(x=None):
    return x[np.isnan(x)==False].min()
def str2int(txt, parName=''):
    try: 
        value = int(txt)
    except:
        reply = QtGui.QMessageBox.critical(None,"Value error", 'Invalid value in parameter '+parName+\
                '. Parameter is not a valid integer.', QtGui.QMessageBox.Ok)
        if reply == QtGui.QMessageBox.Ok:
            value = None

    return value
def str2float(txt, parName=''):
    try: 
        value = np.float64(txt)
    except:
        reply = QtGui.QMessageBox.critical(None,"Value error", 'Invalid value in parameter '+parName+\
                '. Parameter is not a valid float', QtGui.QMessageBox.Ok)
        if reply == QtGui.QMessageBox.Ok:
            value = None

    return value

def str2ToolTipString(txt, str_len=50, nobreak=False):


    tt_str = ''
    if len(txt)<str_len:
        tt_str = '<p style="white-space:pre; font-style:normal; font-weight:300;"> '+txt + '</p>'
    else:
        if nobreak:
            tt_str = '<p style="white-space:pre; font-style:normal; font-weight:300 "> '+txt + '</p>'
        else:

            
            #res_txt = ''.join(txt.strip().split('\n'))
            res_txt = ' '.join(txt.strip().split(' '))
            ind1 = 0
            ind2 = str_len
            while len(res_txt)>str_len:
                txt1    = res_txt[:ind2]

                stxt1   = txt1.split(' ')
                nword   = len(stxt1)
                line    = ' '.join(res_txt.split(' ')[:nword])
                
                tt_str  = tt_str + line +'\n '
                ind1    = len(line)+1
                res_txt = res_txt[ind1:].strip()

            tt_str = '<p style="white-space:pre; font-style:normal; font-weight:300"> '+tt_str + res_txt + '</p>'

    return tt_str







