################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
from PyQt4 import QtGui, QtCore
import matplotlib
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.pylab import colorbar, contourf
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from artist_gui.getplotmodel import *
from artist_gui.helper_functions import ConfigurationParams

import modellib

import sys

import time

class MplCanvas1D(FigureCanvas):
    __pyqtSignals__ = ("update2DContours()")

    def __init__(self):

        ConfigParams = ConfigurationParams()
        self.figureConfigParams = ConfigParams.figureParams

        fsizex = 5.0
        fsizey = 5.0
        self.fig = Figure(figsize=(fsizex,fsizey), dpi=100)
        self.axes = self.fig.add_axes([0.12*5./fsizex, 0.1*5./fsizex, 1.-0.16*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True)

        self.axes.hold(True)
        self.axes.set_adjustable('box')
       
        
        FigureCanvas.__init__(self, self.fig)
        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)


        self.PlottedVarName       = 'density'
        #self.AllVarName           = ['density','temperature','velocity1', 'velocity2', 'velocity3', 'abundance', 'tdust',\
                    #'bmag1', 'bmag2', 'bmag3', 'doppler']
        self.AllVarName           = ['density','temperature','velocity_x', 'velocity_y', 'velocity_z', 'abundance', 'tdust', \
                'doppler', 'bmag_x', 'bmag_y', 'bmag_z']
        self.AllVarLabel          = {'density':'Gas density', 'temperature':'Gas temperature', 'velocity_x':'Velocity (x)',\
                'velocity_y':'Velocity (y)','velocity_z':'Velocity (z)', 'abundance':'Abundance', 'tdust':'Dust Temperature', 'doppler':'Doppler b', \
                'bmag_x':'B (x)', 'bmag_y':'B (y)', 'bmag_z':'B (z)'}
        self.PlottedModelID       = 0
        self.px                   = -1.
        self.py                   = -1.
        self.pxrange              = [1, 2000.]
        self.pyrange              = [0., 1.]
        self.npoints              = 30
        self.xlog                 = False
        self.ylog                 = False
        self.crdvalc              = [1e14, 0., 0.]
        self.crdvals              = [1e14, 90., 0.]
        self.crd1valc             = 1e14
        self.crd2valc             = 0.0
        self.crd3valc             = 0.0
        self.crd1vals             = 1e14
        self.crd2vals             = 90.0
        self.crd3vals             = 0.0
        self.varRange             = [0., 1.]
        self.crdsys               = 'cartesian'
        self.crd                  = 1
        self.plotXUnitName        = ['cm', 'm', 'AU', 'pc']
        self.plotXUnit            = (1e-2, 1.0, 1.496e11, 3.0857200e+16)
        self.currentXUnitID       = 2


    def getVar1D(self, UnitSystem='SI'):
        """Calculate the variable to be plotted in a 1D plot"""


        if UnitSystem=='CGS':
            unitconv = {'density':1e-6, 'temperature':1.0, 'velocity':1e2, 'bmag':1e-4, 'column_density':1e-4}
        else:
            unitconv = {'density':1.0, 'temperature':1.0, 'velocity':1.0, 'bmag':1.0, 'dolumn_density':1.0}


        if (self.crdsys=='cartesian'):
            if self.xlog:
                self.px = self.pxrange[0] * (self.pxrange[1]/self.pxrange[0])**(np.arange(self.npoints, dtype=np.float64)/(float(self.npoints)-1.))
            else:
                self.px = np.arange(self.npoints, dtype=np.float64) / (float(self.npoints)-1.) * (self.pxrange[1] - self.pxrange[0]) + self.pxrange[0]


            self.py = np.zeros(self.npoints, dtype=np.float64)

            dummy  = np.zeros([self.npoints, 3], dtype=np.float64)

            dummy[:,0] = self.crdvalc[0] * self.plotXUnit[self.currentXUnitID]
            dummy[:,1] = self.crdvalc[1] * self.plotXUnit[self.currentXUnitID]
            dummy[:,2] = self.crdvalc[2] * self.plotXUnit[self.currentXUnitID]
            dummy[:,self.crd-1] = self.px * self.plotXUnit[self.currentXUnitID]
 



            if (modellib.getCurrentModelID()!=''):
                if (self.PlottedVarName=='density'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.density(dummy[i,0], dummy[i,1], dummy[i,2]) * unitconv['density']
                if (self.PlottedVarName=='temperature'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.temperature(dummy[i,0], dummy[i,1], dummy[i,2])* unitconv['temperature'] 
                if (self.PlottedVarName=='velocity_x'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.velocity(dummy[i,0], dummy[i,1], dummy[i,2])[0]* unitconv['velocity']
                if (self.PlottedVarName=='velocity_y'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.velocity(dummy[i,0], dummy[i,1], dummy[i,2])[1]* unitconv['velocity']
                if (self.PlottedVarName=='velocity_z'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.velocity(dummy[i,0], dummy[i,1], dummy[i,2])[2]* unitconv['velocity']
                if (self.PlottedVarName=='abundance'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.abundance(dummy[i,0], dummy[i,1], dummy[i,2])
                if (self.PlottedVarName=='tdust'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.tdust(dummy[i,0], dummy[i,1], dummy[i,2])* unitconv['temperature'] 
                if (self.PlottedVarName=='bmag_x'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.bmag(dummy[i,0], dummy[i,1], dummy[i,2])[0]* unitconv['bmag']
                if (self.PlottedVarName=='bmag_y'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.bmag(dummy[i,0], dummy[i,1], dummy[i,2])[1]* unitconv['bmag']
                if (self.PlottedVarName=='bmag_z'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.bmag(dummy[i,0], dummy[i,1], dummy[i,2])[2]* unitconv['bmag']
                if (self.PlottedVarName=='doppler'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.doppler(dummy[i,0], dummy[i,1], dummy[i,2])[2]* unitconv['velocity']


            self.varRange = [self.py.min(), self.py.max()]
            if (self.py.min()==self.py.max()):
                self.varRange = [self.py.min()*0.9, self.py.max()*1.1]

        if (self.crdsys=='spherical'):
            if self.xlog:
                self.px = self.pxrange[0] * (self.pxrange[1]/self.pxrange[0])**(np.arange(self.npoints, dtype=np.float64)/(float(self.npoints)-1.))
            else:
                self.px = np.arange(self.npoints, dtype=np.float64) / (float(self.npoints)-1.) * (self.pxrange[1] - self.pxrange[0]) + self.pxrange[0]

            
            self.py = np.zeros(self.npoints, dtype=np.float64)
            dummy   = np.zeros([self.npoints, 3], dtype=np.float64)
            deg2rad = np.pi/180.

            if (self.crd==1):
                dummy[:,0] = self.px * np.sin(self.crdvals[2]*deg2rad) * np.cos(self.crdvals[1]*deg2rad) 
                dummy[:,1] = self.px * np.sin(self.crdvals[2]*deg2rad) * np.sin(self.crdvals[1]*deg2rad)
                dummy[:,2] = self.px * np.cos(self.crdvals[2]*deg2rad)
            if (self.crd==2):
                dummy[:,0] = self.crdvals[0] * np.sin(self.crdvals[2]*deg2rad) * np.cos(self.px*deg2rad) 
                dummy[:,1] = self.crdvals[0] * np.sin(self.crdvals[2]*deg2rad) * np.sin(self.px*deg2rad)
                dummy[:,2] = self.crdvals[0] * np.cos(self.crdvals[2]*deg2rad)
            if (self.crd==3):
                dummy[:,0] = self.crdvals[0] * np.sin(self.px*deg2rad) * np.cos(self.crdvals[1]*deg2rad) 
                dummy[:,1] = self.crdvals[0] * np.sin(self.px*deg2rad) * np.sin(self.crdvals[1]*deg2rad)
                dummy[:,2] = self.crdvals[0] * np.cos(self.px*deg2rad)

            dummy[:,0] = dummy[:,0] * self.plotXUnit[self.currentXUnitID]
            dummy[:,1] = dummy[:,1] * self.plotXUnit[self.currentXUnitID]
            dummy[:,2] = dummy[:,2] * self.plotXUnit[self.currentXUnitID]
 


            if (modellib.getCurrentModelID()!=''):
                if (self.PlottedVarName=='density'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.density(dummy[i,0], dummy[i,1], dummy[i,2])* unitconv['density']
                if (self.PlottedVarName=='temperature'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.temperature(dummy[i,0], dummy[i,1], dummy[i,2])* unitconv['temperature']
                if (self.PlottedVarName=='velocity_x'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.velocity(dummy[i,0], dummy[i,1], dummy[i,2])[0]* unitconv['velocity']
                if (self.PlottedVarName=='velocity_y'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.velocity(dummy[i,0], dummy[i,1], dummy[i,2])[1]* unitconv['velocity']
                if (self.PlottedVarName=='velocity_z'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.velocity(dummy[i,0], dummy[i,1], dummy[i,2])[2]* unitconv['velocity']
                if (self.PlottedVarName=='abundance'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.abundance(dummy[i,0], dummy[i,1], dummy[i,2])
                if (self.PlottedVarName=='tdust'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.tdust(dummy[i,0], dummy[i,1], dummy[i,2])* unitconv['temperature']
                if (self.PlottedVarName=='bmag_x'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.bmag(dummy[i,0], dummy[i,1], dummy[i,2])[0]* unitconv['bmag']
                if (self.PlottedVarName=='bmag_y'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.bmag(dummy[i,0], dummy[i,1], dummy[i,2])[1]* unitconv['bmag']
                if (self.PlottedVarName=='bmag_z'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.bmag(dummy[i,0], dummy[i,1], dummy[i,2])[2]* unitconv['bmag']
                if (self.PlottedVarName=='doppler'):
                    for i in range(self.npoints):
                        self.py[i] = modellib.doppler(dummy[i,0], dummy[i,1], dummy[i,2])[2]* unitconv['velocity']

            self.varRange = [self.py.min(), self.py.max()]
            if (self.py.min()==self.py.max()):
                self.varRange = [self.py.min()*0.9, self.py.max()*1.1]

    def plotVar1D(self, UnitSystem='SI', Rmin=0., Rmax=0.): 
        """ Draws a 1D plot (variable vs distance) """

        xt_cart = ['x ', 'y ', 'z ']
        xt_sph  = ['r ', r'$\phi$ [deg]', r'$\theta$ [deg]']
        yt_si   = [r'number Gas density [m$^{-3}$]', 'Gas temperature [K]', 'Velocity (x)  [m/s]', 'Velocity (y) [m/s]', 'Velocity (z) [m/s]', 'Abundance',\
                'Dust temperature [K]', 'B (x) [T]', 'B (y) [T]', 'B (z) [T]', 'Doppler b [m/s]']
        yt_cgs  = [r'number Gas density [cm$^{-3}$]', 'Gas temperature [K]', 'Velocity (x)  [cm/s]', 'Velocity (y) [cm/s]', 'Velocity (z) [cm/s]', 'Abundance',\
                'Dust temperature [K]', 'B (x) [G]', 'B (y) [G]', 'B (z) [G]', 'Doppler b [cm/s]']

        if UnitSystem=='SI':
            yt = yt_si
        else:
            yt = yt_cgs

        if (self.crdsys=='cartesian'):
            xtitle = xt_cart[self.crd-1] + '['+self.plotXUnitName[self.currentXUnitID]+']'
            ivar   = self.AllVarName.index(self.PlottedVarName)
            ytitle = yt[ivar]
            plotX = self.px / self.plotXUnit[self.currentXUnitID]
        
        if (self.crdsys=='spherical'):
            if self.crd==1:
                xtitle = xt_sph[self.crd-1] + '['+self.plotXUnitName[self.currentXUnitID]+']'
                plotX = self.px / self.plotXUnit[self.currentXUnitID]
            else:
                xtitle = xt_sph[self.crd-1] 
                plotX = self.px
            
        ivar   = self.AllVarName.index(self.PlottedVarName)
        ytitle = yt[ivar]

         	
        self.axes.clear()
        self.axes.plot(self.px, self.py, 'bo-', markersize=4)
        self.axes.set_xlim(self.pxrange)
        self.axes.set_ylim(self.pyrange)
        self.axes.set_xlabel(xtitle)
        self.axes.set_ylabel(ytitle)
        if self.xlog:
            self.axes.set_xscale('log')
        if self.ylog:
            self.axes.set_yscale('log')
         
        self.axes.tick_params(axis='both', labelsize=self.figureConfigParams['font_size'])
        self.axes.xaxis.label.set_size(self.figureConfigParams['font_size'])
        self.axes.yaxis.label.set_size(self.figureConfigParams['font_size'])

        # Plot the minscale and radius parameters
        if self.crdsys=='cartesian':
            yrange = self.axes.axis()[2:]
            self.axes.plot(np.array([Rmin, Rmin])/self.plotXUnit[self.currentXUnitID], yrange, 'r')
            self.axes.plot(np.array([Rmax, Rmax])/self.plotXUnit[self.currentXUnitID], yrange, 'r')
            if self.xlog:
                anx_min = Rmin * 0.8
                anx_max = Rmax * 1.2
            else:
                dx = (self.pxrange[1] - self.pxrange[0])*0.05
                anx_min = Rmin - dx
                anx_max = Rmax + dx

            if self.ylog:
                any_minmax = np.sqrt(self.pyrange[0]*self.pyrange[1])
            else:
                any_minmax = 0.5*(self.pyrange[0]+self.pyrange[1])

            self.axes.annotate('minScale', xy=(anx_min/self.plotXUnit[self.currentXUnitID], any_minmax),
                    xytext=(anx_min/self.plotXUnit[self.currentXUnitID], any_minmax), rotation=90, color='r') 
            self.axes.annotate('Radius', xy=(anx_max/self.plotXUnit[self.currentXUnitID], any_minmax),
                    xytext=(anx_max/self.plotXUnit[self.currentXUnitID], any_minmax), rotation=90, color='r') 


        elif self.crdsys=='spherical':
            yrange = self.axes.axis()[2:]
            if self.crd==1:
                self.axes.plot(np.array([Rmin, Rmin])/self.plotXUnit[self.currentXUnitID], yrange, 'r')
                self.axes.plot(np.array([Rmax, Rmax])/self.plotXUnit[self.currentXUnitID], yrange, 'r')
            if self.xlog:
                anx_min = Rmin * 0.8
                anx_max = Rmax * 1.2
            else:
                dx = (self.pxrange[1] - self.pxrange[0])*0.05
                anx_min = Rmin - dx
                anx_max = Rmax + dx

            if self.ylog:
                any_minmax = np.sqrt(self.pyrange[0]*self.pyrange[1])
            else:
                any_minmax = 0.5*(self.pyrange[0]+self.pyrange[1])

            self.axes.annotate('minScale', xy=(anx_min/self.plotXUnit[self.currentXUnitID], any_minmax),
                    xytext=(anx_min/self.plotXUnit[self.currentXUnitID], any_minmax), rotation=90, color='r') 
            self.axes.annotate('Radius', xy=(anx_max/self.plotXUnit[self.currentXUnitID], any_minmax),
                    xytext=(anx_max/self.plotXUnit[self.currentXUnitID], any_minmax), rotation=90, color='r') 

        self.draw()


    def updateAxisParams(self, figureConfigParams=None):

        self.figureConfigParams = figureConfigParams
      


        self.axes.set_xlabel(self.axes.xaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])
        self.axes.set_ylabel(self.axes.yaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])
        self.axes.tick_params(axis='both', labelsize=self.figureConfigParams['font_size'])

        self.draw()

class mplFigure1D(QtGui.QWidget):
    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)


        self.canvas = MplCanvas1D()
        self.toolbar = NavigationToolbar(self.canvas, self)
        self.vbl = QtGui.QVBoxLayout()
        self.vbl.addWidget(self.canvas)
        self.vbl.addWidget(self.toolbar)
        self.setLayout(self.vbl)
        
        self.connect(self.canvas, QtCore.SIGNAL("mouseDoubleClick(QMouseEvent)"), self.canvas.mouseDoubleClickEvent)


