################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


from PyQt4 import QtGui, QtCore
from matplotlib.pylab import *
from matplotlib import cm, ticker
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar

import sys
import time

class MplCanvas(FigureCanvas):
    def __init__(self):
        
        fsizex = 4.0
        fsizey = 5.0
        self.fig = Figure(figsize=(fsizex,fsizey), dpi=100)
        self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True)

        #x = np.arange(12.)
        #self.axes.plot(x, x*x, 'or')
        self.axes.hold(True)
        self.axes.set_adjustable('box')
       
        
        FigureCanvas.__init__(self, self.fig)
        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        
        
    def plotData(self, dataX=None, dataY=None, xtitle='', ytitle='', logscale=False): 

        self.axes.clear()
        fontsize = 10

        if logscale:
            self.axes.loglog(dataX, dataY, 'bo-', markersize=4)
        else:
            self.axes.plot(dataX, dataY, 'b', drawstyle='steps-mid')
        for tick in self.axes.get_xaxis().get_major_ticks():
            tick.label1.set_fontsize(fontsize)
        for tick in self.axes.get_yaxis().get_major_ticks():
            tick.label1.set_fontsize(fontsize)

        y_formatter = ticker.ScalarFormatter()
        y_formatter.set_powerlimits((-2,2))

        self.axes.yaxis.set_major_formatter(y_formatter)
        self.axes.set_xlim(dataX.min(), dataX.max())
        self.axes.set_ylim(dataY.min()-(dataY.max()-dataY.min())*0.07, dataY.max()+(dataY.max()-dataY.min())*0.07 )
        self.axes.set_xlabel(xtitle, fontsize=fontsize)
        self.axes.set_ylabel(ytitle, fontsize=fontsize)

        self.draw()

class mplFigurePixSpec(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.canvas = MplCanvas()
        self.toolbar = NavigationToolbar(self.canvas, self)
        self.vbl = QtGui.QVBoxLayout()
        self.vbl.addWidget(self.canvas)
        self.vbl.addWidget(self.toolbar)
        self.setLayout(self.vbl)

