################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import sys
from PyQt4 import QtCore, QtGui, uic
from artist_gui.StatusDisplayWindowBase import Ui_StatusDialog

import time
import lime, modellib
import threading

class StatusDisplay(QtGui.QMainWindow, Ui_StatusDialog):
    """
    A window widget to display progress bars and other status updates for the Lime run
    """
  
    class LimeThread (threading.Thread):
    
        def __init__(self, par, images):
            self._par = par
            self._images = images
            self._stop = threading.Event()

            threading.Thread.__init__(self) 

        def run(self):
            try:
                lime.runLime(self._par, self._images, False)
            except Exception, e:
                print e
                reply = QtGui.QMessageBox.critical(None,"Lime Error", e, QtGui.QMessageBox.Ok)
                if reply == QtGui.QMessageBox.Ok:
                    return

        def stop(self):
            self._stop.set()

        def stopped(self):
            return self._stop.isSet()


    def __init__(self,parent=None):
        super(StatusDisplay, self).__init__(parent)
        self.setupUi(self)
        self._initalize()


    def _runLime(self, par, images, app=None):
    
        
        self.limeThread = self.LimeThread(par, images)
        self.limeThread.start()
   
        self.app = app

        while self.limeThread.isAlive():
            self._updateGui()
            self.app.processEvents()
    

    def _initalize(self):
        self.GridBuildingPBar.setValue(0)
        self.GridSmoothingPBar.setValue(0)
        self.PhotonPropagationPBar.setValue(0)
        self.RayTracingPBar.setValue(0)

        self.GridBuildingDoneLabel.setText('')
        self.GridSmoothingDoneLabel.setText('')
        self.PhotonPropagationDoneLabel.setText('')
        self.RayTracingDoneLabel.setText('')

        self.IterNrLabel.setText("0")
        self.MinSNRLabel.setText("0")
        self.MedSNRLabel.setText("0")

        self.MessageTBrowser.setPlainText("")

    def _updateGui(self):

        GridBuildprogressVal = 100*lime.getStatus().progressGridBuilding
        GridSmoothprogressVal = 100*lime.getStatus().progressGridSmoothing
        PhotonPropagationprogressVal = 100*lime.getStatus().progressPhotonPropagation
        RayTracingprogressVal = 100*lime.getStatus().progressRayTracing

        self.GridBuildingPBar.setValue(GridBuildprogressVal)
        self.GridSmoothingPBar.setValue(GridSmoothprogressVal)
        self.PhotonPropagationPBar.setValue(PhotonPropagationprogressVal)
        self.RayTracingPBar.setValue(RayTracingprogressVal)

        if GridBuildprogressVal==100.: 
            self.GridBuildingDoneLabel.setText('Done')
        if GridSmoothprogressVal==100.:
            self.GridSmoothingDoneLabel.setText('Done')
        if PhotonPropagationprogressVal==100.:
            self.PhotonProppagationDoneLabel.setText('Done')
        if RayTracingprogressVal==100.:
            self.RayTracingDoneLabel.setText('Done') 

        self.IterNrLabel.setText("%d" % lime.getStatus().numberIterations)
        self.MinSNRLabel.setText("%f" % lime.getStatus().minsnr)
        self.MedSNRLabel.setText("%f" % lime.getStatus().median)
        self.MessageText.setText(str(lime.getStatus().message))

        
        


