###############################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


from __future__ import with_statement
import numpy as np
import sys
from PyQt4 import QtCore, QtGui
import modellib, lime, os
from artist_gui.status_display import *
import subprocess as sp

from artist_gui.imageviewer import *
from artist_gui.griddialog import GridDialog
from artist_gui.helper_functions import *
from artist_gui.getplotmodel import *
from artist_gui.AboutDialogBase import Ui_AboutDialog
from artist_gui.MainControlWindowBase import Ui_MainWindow
from artist_gui.VisualizeMainWindowBase import Ui_VisualizeMainWindow
from artist_gui.StartupDialogBase import Ui_StartupDialog
from artist_gui.ModellibBrowserDialogBase import Ui_ModellibBrowserDialog
from artist_gui.AbunRegionCustomizationDialogBase import Ui_AbunRegionCusmizationDialog
from artist_gui.MoldatfileBrowserDialogBase import Ui_MoldatfileBrowserDialog
from artist_gui.PixSpecMainWindowBase import Ui_mplFigurePixSpecMainWindow
from artist_gui.grid import *
import pyfits, copy
import ArtistTable 
print 'Setting up logging...'
import logging

log = logging.getLogger('artist')
log.setLevel(logging.DEBUG)
handler = logging.FileHandler('artist.log')
handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(module)s:%(lineno)d:  %(message)s'))
log.addHandler(handler)


def readmoldatfile(fname=None):

    log.debug('Opening moldata file '+fname)
    try: 
        rfile = open(fname, 'r')
    except:
        log.exception('Moldata file cannot be opened - file : '+fname)
        return 
    dum = rfile.readline()
    if dum.__contains__('MOLECULE')==False:
        print 'The file is not in the LAMBDA file format'
        return
    dum = rfile.readline()

    dum = rfile.readline() 
    dum = rfile.readline() 
    
    dum = rfile.readline() 
    dum = rfile.readline() 
    nr_lev = int(dum)

    dum = rfile.readline() 
    for i in range(nr_lev):
        dum = rfile.readline() 

    dum = rfile.readline() 
    dum = rfile.readline() 
    nr_rtrans = int(dum)

    dum = rfile.readline() 

    res = {}
    res['ID']   = []
    res['A']    = []
    res['nu0']  = []
    res['eupk'] = []
    for i in range(nr_rtrans):
        dum = rfile.readline() 
        txt = dum.split()

        res['ID'].append(int(txt[0])-1)
        res['A'].append(float(txt[3]))
        res['nu0'].append(float(txt[4]))
        res['eupk'].append(float(txt[5]))

    rfile.close()

    return res

class MainWindowCustomizeDialog(QtGui.QDialog, Ui_CustomizeDialog):

    
    updateCustomization = QtCore.pyqtSignal(object,int,name='updateCustomization')
    writeCustomization = QtCore.pyqtSignal(object,name='writeCustomization')


    def __init__(self, parent=None, configParams=None, showToolTipHelp=False):
        super(MainWindowCustomizeDialog, self).__init__(parent)
        self.setupUi(self)

        self.configParams = cp.deepcopy(configParams)
        self.setStyleSheet('font-family: '+self.configParams.windowParams['font_family']+\
                '; font-size: '+self.configParams.windowParams['font_size']+'pt;')
        
        self.ARTISTFrame.setEnabled(True)
        self.ARTISTFrame.setVisible(True)
        self.setParamValues()
        if showToolTipHelp:
            self.setAllToolTips()
        else:
            self.hideAllToolTips()
            

        self.connect(self.PrefDoneButton, QtCore.SIGNAL("clicked()"), self.readOutAllPrefsAndExit)
        self.connect(self.PrefSetAsDefaultButton, QtCore.SIGNAL("clicked()"), self.writeDefaultsToFile)
        self.connect(self.PrefCancelButton, QtCore.SIGNAL("clicked()"), self.exitWithoutReadOut)
        self.connect(self.PrefParaViewBrowseButton, QtCore.SIGNAL("clicked()"), self.openParaViewBrowser)
        self.connect(self.PrefWindowFontSelectorButton, QtCore.SIGNAL("clicked()"), self.openFontDialog)
        self.connect(self.PrefCanvasBgColorSelectorButton, QtCore.SIGNAL("clicked()"), self.browseCanvasBgColor)
        self.connect(self.PrefCanvasFgColorSelectorButton, QtCore.SIGNAL("clicked()"), self.browseCanvasFgColor)
    
    def hideAllToolTips(self):
        
        self.PrefUnitSystemSelectorCBox.setToolTip('')
        self.PrefMImagePolicyCBox.setToolTip('')
        self.PrefCopyDatafilesCheckBox.setToolTip('')
        self.PrefParaViewComLedit.setToolTip('')
        self.PrefParaViewBrowseButton.setToolTip('')
        self.PrefWindowFontFamilyLedit.setToolTip('')
        self.PrefWindowFontSelectorButton.setToolTip('')

        self.PrefWindowFontSizeLedit.setToolTip('')
        self.PrefCanvasFontSizeLedit.setToolTip('')
        self.PrefCanvasAutoUpdateCBox.setToolTip('')
        self.PrefCanvasBgColorSelectorButton.setToolTip('')
        self.PrefCanvasFgColorSelectorButton.setToolTip('')

    def setAllToolTips(self):

        self.PrefUnitSystemSelectorCBox.setToolTip(str2ToolTipString('Select the unit system in the figures (visualization window)'))
        self.PrefMImagePolicyCBox.setToolTip(str2ToolTipString('Select the mode how multiple images are calculated if more than one parameter changes from image to image. In series model all variables change together while in matrix mode variables change independently. If two variables change from image to image in series mode both variable should have a list of N values and N images will be calculated. In matrix mode the two variables change independently of each other and NxN images will be calculated with all possible combinations of the two parameters.'))
        self.PrefCopyDatafilesCheckBox.setToolTip(str2ToolTipString('Copy all data files (e.g., dust opacity, molecular data file) to the model directory to keep all files at one place'))
        self.PrefParaViewComLedit.setToolTip(str2ToolTipString('Command to launch Paraview'))
        self.PrefParaViewBrowseButton.setToolTip(str2ToolTipString('File browser to select the Paraview command file. (On Macs the .app directory should be selected and the executable will be found within the subdirectories automatically)'))
        self.PrefWindowFontFamilyLedit.setToolTip(str2ToolTipString('Name of the font family used in the windows'))
        self.PrefWindowFontSelectorButton.setToolTip(str2ToolTipString('Open a font dialog to select the font family and the size'))

        self.PrefWindowFontSizeLedit.setToolTip(str2ToolTipString('Size of the font used in the windows'))
        self.PrefCanvasFontSizeLedit.setToolTip(str2ToolTipString('Name of the font family used in the figures'))
        self.PrefCanvasAutoUpdateCBox.setToolTip(str2ToolTipString('Automatically update 2D contour plots when any parameter of the model / contour changes (Note, this can slow down the GUI)'))
        self.PrefCanvasBgColorSelectorButton.setToolTip(str2ToolTipString('Select the background color of the figures in the image viewer window'))
        self.PrefCanvasFgColorSelectorButton.setToolTip(str2ToolTipString('Select the foreground color of the figures in the image viewer window'))

    def setParamValues(self):

        if self.configParams.unitSystem=='SI':
            self.PrefUnitSystemSelectorCBox.setCurrentIndex(0)
        if self.configParams.unitSystem=='CGS':
            self.PrefUnitSystemSelectorCBox.setCurrentIndex(1)

        if self.configParams.autoupdateFigure:
            self.PrefCanvasAutoUpdateCBox.setChecked(True)
        else:
            self.PrefCanvasAutoUpdateCBox.setChecked(False)

        if self.configParams.mImagePolicy=='Series':
            self.PrefMImagePolicyCBox.setCurrentIndex(0)
        if self.configParams.mImagePolicy=='Matrix':
            self.PrefMImagePolicyCBox.setCurrentIndex(1)

        if self.configParams.copyDataFilesToModelDir:
            self.PrefCopyDatafilesCheckBox.setChecked(True)
        else:
            self.PrefCopyDatafilesCheckBox.setChecked(False)

        self.PrefParaViewComLedit.setText(self.configParams.paraViewCommand)

        self.PrefWindowFontFamilyLedit.setText(self.configParams.windowParams['font_family'])
        self.PrefWindowFontSizeLedit.setText(str(self.configParams.windowParams['font_size']).strip())

        self.PrefCanvasFontSizeLedit.setText(str(self.configParams.figureParams['font_size']).strip())

        color = QtGui.QColor(0,0,0)
        color.setNamedColor(self.configParams.figureParams['bgcolor'])
        self.PrefCanvasBgColorLedit.setPalette(QtGui.QPalette(color))
        self.PrefCanvasBgColorLedit.setAutoFillBackground(True)

        color = QtGui.QColor(0,0,0)
        color.setNamedColor(self.configParams.figureParams['fgcolor'])
        self.PrefCanvasFgColorLedit.setPalette(QtGui.QPalette(color))
        self.PrefCanvasFgColorLedit.setAutoFillBackground(True)


    def openFontDialog(self):
        
        oldFont = self.parent().font()
        dum = QtGui.QFontDialog()
        dum.setCurrentFont(oldFont)
        dum.exec_()
        if dum.result()==1:
            newFont = dum.selectedFont()
            family = str(newFont.family())
            psize = str(newFont.pointSize()).strip()
           
            self.setStyleSheet('font-family: '+family+'; font-size: '+psize+'pt;')
            self.configParams.windowParams['font_family'] = family
            self.configParams.windowParams['font_size'] = psize
            self.PrefWindowFontFamilyLedit.setText(family) 
            self.PrefWindowFontSizeLedit.setText(psize) 

    def browseCanvasFgColor(self):
        color = QtGui.QColorDialog.getColor(QtCore.Qt.green, self)

        if color.isValid(): 
            self.PrefCanvasFgColorLedit.setPalette(QtGui.QPalette(color))
            self.PrefCanvasFgColorLedit.setAutoFillBackground(True)
            self.configParams.figureParams['fgcolor']=str(color.name())


    def browseCanvasBgColor(self):
        color = QtGui.QColorDialog.getColor(QtCore.Qt.green, self)

        if color.isValid(): 
            self.PrefCanvasBgColorLedit.setPalette(QtGui.QPalette(color))
            self.PrefCanvasBgColorLedit.setAutoFillBackground(True)
            self.configParams.figureParams['bgcolor']=str(color.name())

    def openParaViewBrowser(self):
        
        if sys.platform=='darwin':
            dum = str(QtGui.QFileDialog.getExistingDirectory(self, 'Open Paraview.app',  options=QtGui.QFileDialog.DontUseNativeDialog))
           
            if dum=='': return
            
            sdum = dum.split()

            ParaviewDir = sdum[0]
            for i in range(1,len(sdum)):
                ParaviewDir = ParaviewDir + '\ ' + sdum[i]

            command = sp.Popen(['find '+ParaviewDir+' -name paraview'], shell=True, stdout=sp.PIPE).communicate()[0].split('\n')
         
            for i in range(len(command)):
                if os.path.isfile(command[i]):
                    com = command[i]
                    break
                else:
                    ParaviewDir = ''
                    com = ''
                    dum = ''

            if dum!='':
                self.configParams.paraViewCommand = com
                self.PrefParaViewComLedit.setText(com)
                python_ver = str(sys.version_info[0])+'.'+str(sys.version_info[1])
                dirname = os.environ['ARTIST_ROOT']+'/lib/python'+python_ver+'/site-packages/artist_gui/'
                wfile = open(dirname+'/paraview.com','w')
                wfile.write(com)
                wfile.close
            else:
                reply = QtGui.QMessageBox.critical(self, "Error",
                        'The paraview command was not found in '+dum,
                        QtGui.QMessageBox.Ok)
        else:
            com = str(QtGui.QFileDialog.getOpenFileName(self, 'Open Paraview',  options=QtGui.QFileDialog.DontUseNativeDialog))

            if com!='':
                self.configParams.paraViewCommand = com
                self.PrefParaViewComLedit.setText(com)
                python_ver = str(sys.version_info[0])+'.'+str(sys.version_info[1])
                dirname = os.environ['ARTIST_ROOT']+'/lib/python'+python_ver+'/site-packages/artist_gui/'
                wfile = open(dirname+'/paraview.com','w')
                wfile.write(com)
                wfile.close
            else:
                reply = QtGui.QMessageBox.critical(self, "Error",
                        'The paraview command was not found in '+dum,
                        QtGui.QMessageBox.Ok)


    def changeUnitSystem(self):

        self.parent().configParams.unitSystem = str(self.PrefUnitSystemSelectorCBox.currentText())
        self.configParams.unitSystem = self.parent().configParams.unitSystem
        self.unitSystemChanged.emit()
       
    def exitWithoutReadOut(self):   

        self.reject()

    def writeDefaultsToFile(self):
        self.configParams.windowParams['font_family'] = self.PrefWindowFontFamilyLedit.text()
        self.configParams.windowParams['font_size'] = str(str2int(self.PrefWindowFontSizeLedit.text(), 'Font Size (Window)'))
        self.configParams.figureParams['font_size'] = str(str2int(self.PrefCanvasFontSizeLedit.text(), 'Font Size (Image Canvas)'))
        self.configParams.unitSystem = str(self.PrefUnitSystemSelectorCBox.currentText())
        self.configParams.mImagePolicy = str(self.PrefMImagePolicyCBox.currentText())
        self.configParams.copyDataFilesToModelDir = self.PrefCopyDatafilesCheckBox.isChecked()
        txt = self.PrefParaViewComLedit.text()
        self.configParams.paraViewCommand = str(txt) 
        
        self.writeCustomization.emit(self.configParams)

    def readOutAllPrefsAndExit(self):


        self.configParams.windowParams['font_family'] = self.PrefWindowFontFamilyLedit.text()
        self.configParams.windowParams['font_size'] = str(str2int(self.PrefWindowFontSizeLedit.text(), 'Font Size (Window)'))
        self.configParams.figureParams['font_size'] = str(str2int(self.PrefCanvasFontSizeLedit.text(), 'Font Size (Image Canvas)'))
        self.configParams.unitSystem = str(self.PrefUnitSystemSelectorCBox.currentText())
        self.configParams.autoupdateFigure = self.PrefCanvasAutoUpdateCBox.isChecked() 
        self.configParams.mImagePolicy = str(self.PrefMImagePolicyCBox.currentText())
        self.configParams.copyDataFilesToModelDir = self.PrefCopyDatafilesCheckBox.isChecked()
        txt = self.PrefParaViewComLedit.text()
        self.configParams.paraViewCommand = str(txt) 

        self.updateCustomization.emit(self.configParams, 1)
        self.accept()

class modelSetupPar():

    def __init__(self,parent=None, par=None, img=None):
       

        self.modelID       = []
        self.modelParams   = []
        self.par           = {}
        self.img           = []
        self.abunRegion    = False
        self.RegionAbunParams = AbundanceModel()
        self.variable      = {'density':{'isFunction':False},
                              'temperature':{'isFunction':False}, 
                              'abundance':{'isFunction':False}, 
                              'velocity':{'isFunction':False}, 
                              'doppler':{'isFunction':False},  
                              'bmag':{'isFunction':False}, 
                              'tdust':{'isFunction':False}}

        ModelID = modellib.getCurrentModelID()
        self.modelID.append(modellib.getCurrentModelID())
        m = {}

        if ModelID!='Tabdata':
            for parname in modellib.getParamIDs(ModelID):
                ptype = modellib.getParamType(ModelID,parname)
                if ptype=='int':
                    m[parname] = modellib.getParamInt(parname)
                if ptype=='double':
                    m[parname] = modellib.getParamDouble(parname)
                if ptype=='string':
                    m[parname] = modellib.getParamString(parname)
                if ptype=='enum':
                    m[parname] = modellib.getParamEnumIndex(parname)

        self.modelParams.append(m)

        if (par!=None):
            self.par['radius']     = par.radius
            self.par['minScale']   = par.minScale
            self.par['pIntensity'] = par.pIntensity
            self.par['sinkPoints'] = par.sinkPoints
            self.par['dust']       = par.dust
            self.par['outputfile'] = par.outputfile
            self.par['gridfile']   = par.gridfile
            self.par['moldatfile'] = par.moldatfile
            self.par['lte_only']   = par.lte_only
            self.par['tcmb']       = par.tcmb
            self.par['sampling']   = par.sampling
            self.par['antialias']  = par.antialias
            self.par['blend']      = par.blend

        if (img!=None):
            im = {}
            im['pxls']       = img.pxls
            im['imgres']     = img.imgres
            im['theta']      = img.theta
            im['distance']   = img.distance
            im['unit']       = img.unit
            im['filename']   = img.filename
            im['phi']        = img.phi
            im['source_vel'] = img.source_vel
            im['nchan']      = img.nchan
            im['velres']     = img.velres
            im['trans']      = img.trans
            im['bandwidth']  = img.bandwidth
            im['freq']       = img.freq

            self.img.append(im)
        
        for var in self.variable.keys():
            if modellib.isCurrentResultFunction(var):
                self.variable[var]['isFunction'] = True
                self.variable[var]['functionID'] = modellib.getFunctionID(var)
                self.variable[var]['functionParamName']=modellib.getFunctionParamIDs(self.variable[var]['functionID'])
                dum = []
                for pname in self.variable[var]['functionParamName']:
                    ptype = modellib.getFunctionParamType(self.variable[var]['functionID'], pname)

                    if ptype=='double':
                        dum.append(modellib.getFunctionParamDouble(var, pname))
                    if ptype=='int':
                        dum.append(modellib.getFunctionParamInt(var, pname))
                    if ptype=='string':
                        dum.append(modellib.getFunctionParamString(var, pname))
                    
                self.variable[var]['functionParamValue']=dum
            
            if var=='temperature':
                if modellib.isTempIdentTdust():
                    self.variable[var]['isFunction'] = True
                    self.variable[var]['functionID'] = 'setTempIdentTdust'
                    self.variable[var]['functionParamName'] = []
                    self.variable[var]['functionParamValue'] = []
            
            if var=='tdust':
                if modellib.isTdustIdentTemp():
                    self.variable[var]['isFunction'] = True
                    self.variable[var]['functionID'] = 'setTdustIdentTemp'
                    self.variable[var]['functionParamName'] = []
                    self.variable[var]['functionParamValue'] = []


    def writeModelSetupFile(self, fileName=None, abunModel=None):
        """ Writes out a setup file storing all important parameters"""

        if (fileName==None): fileName='model'

        wfile = open(fileName+'.stp', 'w')
        wfile.write("%s\n"%('# ARTIST model setup v1.1'))

        wfile.write("%s\n"%('<'+self.modelID[0]+'>'))
        for parname in self.modelParams[0].keys():
            ptype = modellib.getParamType(self.modelID[0],parname)
            if ptype=='int':
                wfile.write("%15s %d\n"%(parname+' = ',self.modelParams[0][parname])) 
            if ptype=='double':
                wfile.write("%15s %.9e\n"%(parname+' = ',self.modelParams[0][parname])) 
            if ptype=='string':
                wfile.write("%15s %s\n"%(parname+' = ',self.modelParams[0][parname])) 
            if ptype=='enum':
                wfile.write("%15s %d\n"%(parname+' = ',self.modelParams[0][parname])) 


        wfile.write("%s\n"%('<LimeInpPar>'))
        for parname in self.par.keys():
            wfile.write("%15s %s\n"%(parname+' = ',str(self.par[parname])))
        wfile.write("%s\n"%('<LimeImgPar>'))
        for parname in self.img[0].keys():
            txt = str(self.img[0][parname])
            wfile.write("%15s %s\n"%(parname+' = ',txt))
        wfile.write("%s\n"%('<CustomFunctions>'))
        for var in self.variable.keys():
            if self.variable[var]['isFunction']:
                wfile.write("%15s %s\n"%(var+' = ','True'))
                wfile.write("%15s %s\n"%('functionID = ',self.variable[var]['functionID']))
                for i in range(len(self.variable[var]['functionParamName'])):
                    parName = self.variable[var]['functionParamName'][i]
                    parVal = self.variable[var]['functionParamValue'][i]
                    if parName=='filename':
                        wfile.write("%15s %s\n"%(parName+' = ',parVal))
                    else:
                        wfile.write("%15s %.9e\n"%(parName+' = ',parVal))
                if var=='abundance':        
                    if self.abunRegion:
                        sep = abunModel.regionSeparators[abunModel.regionSeparatorsCurrentIndex]
                        sepunit = abunModel.regionSeparatorUnitSelection[sep]
                        wfile.write("%15s %s\n"%('abunRegions'+' = ','True'))
                        wfile.write("%15s %s\n"%('Separator'+' = ',sep))
                        wfile.write("%15s %s\n"%('Separ.Unit'+' = ',sepunit))
                        wfile.write("%15s %d\n"%('nRegion'+' = ',abunModel.nRegion))
                        for ireg in range(abunModel.nRegion-1):
                            label = 'Boundary '+('%d'%ireg)+' = '
                            wfile.write("%15s %.9e\n"%(label,  abunModel.regionBoundaries[ireg]))
                        for ireg in range(abunModel.nRegion):
                            label = 'Abundance '+('%d'%ireg)+' = '
                            wfile.write("%15s %.9e\n"%(label,  abunModel.regionAbundances[ireg]))
                    else:
                        wfile.write("%15s %s\n"%('abunRegions'+' = ','False'))

            else:
                wfile.write("%15s %s\n"%(var+' = ','False'))

        wfile.close()

    def readModelSetupFile(self, fileName=None):
        """ Writes out a setup file storing all important parameters"""


        supported_versions = ['v0.1', 'v1.0', 'v1.1']
        if (fileName==None): fileName='model'
        
        log.debug('Opening model setup file '+os.getcwd()+'/'+fileName+'.stp')

        try:
            rfile = open(fileName+'.stp')
            txt   = rfile.readline().strip()
            if (txt[0:20]!='# ARTIST model setup'):

                log.exception('Model setup is not in ARTIST format')
                reply = QtGui.QMessageBox.critical(None,"Error",fileName+".stp is not an ARTIST setup file", QtGui.QMessageBox.Ok)
                return False
            else:
                version = txt[21:].strip() 
                isVersionSupported = False
                for i in supported_versions:
                    if i==version : isVersionSupported=True

                if not isVersionSupported:
                    reply = QtGui.QMessageBox.critical(None,"Error","The version of the model setup file ("+version+\
                            ")is unsupported", QtGui.QMessageBox.Ok)
                    log.exception('Model setup file version is not supported ('+version+')')

                    return False
                    
                txt   = rfile.readline().strip()
                self.modelID[0] = txt[txt.index('<')+1:txt.index('>')]
                self.modelParams = []
                m = {}
                txt = rfile.readline().strip().split('=')
                while(txt[0][0]!='<'):
                    txt[0], txt[1]= txt[0].strip(), txt[1].strip()
                    if txt[0].strip()=='dustopac_fname':
                        m[txt[0]] = txt[1]
                    else:
                        m[txt[0]] = float(txt[1])
                    txt = rfile.readline().strip().split('=')


                self.modelParams.append(m)
                if txt[0].rstrip('\n')!='<LimeInpPar>':
                    print 'ERROR'
                    print 'File is not the proper ARTIST stp format'
                    print '<ModelID> block should be followed by <LimeInpPar>'
                    print txt
                else:
                    txt = rfile.readline().strip().split('=')

                    while(txt[0][0]!='<'):
                        txt[0], txt[1]= txt[0].strip(), txt[1].strip()
                        
                        if txt[0]=='dust':
                            self.par[txt[0]] = txt[1]
                        elif txt[0]=='outputfile':
                            self.par[txt[0]] = txt[1]
                        elif txt[0]=='gridfile':
                            self.par[txt[0]] = txt[1]
                        elif txt[0]=='moldatfile':
                            self.par[txt[0]] = txt[1]
                        else:
                            self.par[txt[0]] = float(txt[1])

                        txt = rfile.readline().strip().split('=')

                    if txt[0].rstrip('\n')!='<LimeImgPar>':
                        print 'ERROR'
                        print 'File is not the proper ARTIST stp format'
                        print '<LimeInpPar> block should be followed by <LimeImgPar>'
                        print txt
                    else:
                        im = {}
                        txt = rfile.readline().strip().split('=')
                        while(txt[0][0]!='<'):
                            txt[0], txt[1]= txt[0].strip(), txt[1].strip()
                            if txt[0]=='filename':
                                im[txt[0]] = txt[1]
                            else:
                                im[txt[0]] = float(txt[1])

                            
                            txt = rfile.readline().split('=')
                        self.img.append(im) 
                        if txt[0].rstrip('\n')!='<CustomFunctions>':
                            print 'ERROR'
                            print 'File is not the proper ARTIST stp format'
                            print '<LimeImgPar> block should be followed by <CustomFunctions>'
                            print txt
                        else:
                            for var in self.variable.keys():
                                txt = rfile.readline().strip().split('=')
                                if txt[1].strip()=='True': 
                                    self.variable[var]['isFunction'] = True
                                if txt[1].strip()=='False': 
                                    self.variable[var]['isFunction'] = False
                                if self.variable[var]['isFunction']:
                                    txt = rfile.readline().strip().split('=')
                                    self.variable[var]['functionID'] = txt[1].strip()

                                    if self.variable[var]['functionID']=='setTempIdentTdust':
                                        self.variable[var]['functionParamName']=[]
                                        self.variable[var]['functionParamValue']=[]
                                    elif self.variable[var]['functionID']=='setTdustIdentTemp':
                                        self.variable[var]['functionParamName']=[]
                                        self.variable[var]['functionParamValue']=[]
                                    else:
                                        self.variable[var]['functionParamName']=[]
                                        self.variable[var]['functionParamValue']=[]
                                        for i in range(len(modellib.getFunctionParamIDs(txt[1].strip()))):
                                            txt = rfile.readline().strip().split('=')
                                            self.variable[var]['functionParamName'].append(txt[0].strip()) 
                                            if txt[0].strip()=='filename':
                                                self.variable[var]['functionParamValue'].append(txt[1]) 
                                            else:
                                                self.variable[var]['functionParamValue'].append(float(txt[1])) 

                                    if var=='abundance':
                                        if float(version[1:])>=1.1:
                                            txt = rfile.readline().strip().split('=')

                                            if txt[1].strip()=='True':
                                                
                                                self.abunRegion    = True

                                                txt = rfile.readline().strip().split('=') 
                                                sep = txt[1].strip() 
                                                self.RegionAbunParams.regionSeparatorsCurrentIndex = self.RegionAbunParams.regionSeparators.index(sep)
                                                
                                                txt = rfile.readline().strip().split('=')
                                                self.RegionAbunParams.regionSeparatorUnitSelection[sep] = txt[1].strip()

                                                txt = rfile.readline().strip().split('=')
                                                self.RegionAbunParams.nRegion = int(txt[1].strip())

                                                self.RegionAbunParams.regionBoundaries = []
                                                for ireg in range(self.RegionAbunParams.nRegion-1):
                                                    txt = rfile.readline().strip().split('=')
                                                    self.RegionAbunParams.regionBoundaries.append(float(txt[1].strip()))

                                                self.RegionAbunParams.regionAbundances = []
                                                for ireg in range(self.RegionAbunParams.nRegion):
                                                    txt = rfile.readline().strip().split('=')
                                                    self.RegionAbunParams.regionAbundances.append(float(txt[1].strip()))
                                        else:
                                            self.abunRegion    = False

                                             
            rfile.close()
           

            return True
        except Exception, e:
            log.exception('Problem during reading the parmeterfile')
            log.exception(e)
            reply = QtGui.QMessageBox.critical(None,"Error","Problem with reading "+fileName+"\n The file either "+\
                    " does not exist or is corrupted.", QtGui.QMessageBox.Ok)
            return False


class MoldatfileBrowserDialog(QtGui.QDialog, Ui_MoldatfileBrowserDialog):
    """
    Dialog class to select transitions in LAMBDA data files
    """
    
    returnSelectedTrans= QtCore.pyqtSignal(list,name='returnSelectedTrans')

    def __init__(self, parent=None, data=None, selectedTrans=None, SingleSelectMode=False, configParams=None):
        super(MoldatfileBrowserDialog, self).__init__(parent)
        self.setupUi(self)

        if configParams!=None:
            self.setStyleSheet('font-family: '+configParams.windowParams['font_family']+\
                    '; font-size: '+configParams.windowParams['font_size']+'pt;')

        self.avDataModel = QtGui.QStandardItemModel(0, 4, self)
        self.avDataModel.setHeaderData(0, QtCore.Qt.Horizontal, "ID")
        self.avDataModel.setHeaderData(1, QtCore.Qt.Horizontal, "Frequency [GHz]")
        self.avDataModel.setHeaderData(2, QtCore.Qt.Horizontal, "Einstein A [1/s]")
        self.avDataModel.setHeaderData(3, QtCore.Qt.Horizontal, "Eup/k [K]")

        
        self.SingleSelectMode = SingleSelectMode
        self.data = data
        for i in range(len(data['ID'])):
            self.addNewRowtoDataModel(self.avDataModel, data['ID'][i], data['nu0'][i], data['A'][i], data['eupk'][i])
        
        self.avProxyModel = QtGui.QSortFilterProxyModel()
        self.avProxyModel.setDynamicSortFilter(True)

        self.avProxyModel.setSourceModel(self.avDataModel)
        self.AvailableTransitionsTreeView.setModel(self.avDataModel)


        if SingleSelectMode:
            self.AvailableTransitionsTreeView.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
            
            self.UsedTransitionsTreeView.setVisible(False)
            self.RemoveTransitionContFrame.setVisible(False)
            self.AddTransitionToUsedButton.setVisible(False)
        else:
            self.AvailableTransitionsTreeView.setSelectionMode(QtGui.QAbstractItemView.MultiSelection)
            self.UsedTransitionsTreeView.setVisible(True)
            self.RemoveTransitionContFrame.setVisible(True)
            self.AddTransitionToUsedButton.setVisible(True)

        self.usDataModel = QtGui.QStandardItemModel(0, 4, self)
        self.usDataModel.setHeaderData(0, QtCore.Qt.Horizontal, "ID")
        self.usDataModel.setHeaderData(1, QtCore.Qt.Horizontal, "Frequency [GHz]")
        self.usDataModel.setHeaderData(2, QtCore.Qt.Horizontal, "Einstein A [1/s]")
        self.usDataModel.setHeaderData(3, QtCore.Qt.Horizontal, "Eup/k [K]")

        if selectedTrans!=None:
            for i in selectedTrans:
                ii = data['ID'].index(i)
                self.addNewRowtoDataModel(self.usDataModel, data['ID'][ii], data['nu0'][ii], data['A'][ii], data['eupk'][ii])
        
        self.usProxyModel = QtGui.QSortFilterProxyModel()
        self.usProxyModel.setDynamicSortFilter(True)

        self.usProxyModel.setSourceModel(self.usDataModel)
        self.UsedTransitionsTreeView.setModel(self.usDataModel)

        dum = ["ID", "Frequency", "Einstein A", "Eup/k"]
        self.FilterParSelectorCBox.addItems(dum)
        self.FilterParSelectorCBox.setCurrentIndex(1)

        self.connect(self.FilterParSelectorCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.filterParChanged)
        self.connect(self.FilterMinLedit, QtCore.SIGNAL("returnPressed()"), self.applyFilter)
        self.connect(self.FilterMaxLedit, QtCore.SIGNAL("returnPressed()"), self.applyFilter)
        self.connect(self.FilterGroupBox, QtCore.SIGNAL("toggled(bool)"), self.applyFilter)

        self.connect(self.AddTransitionToUsedButton, QtCore.SIGNAL("clicked()"), self.addAvTransition)
        self.connect(self.RemoveTransitionFromUsedButton, QtCore.SIGNAL("clicked()"), self.removeUsTransition)
        self.connect(self.DoneButton,QtCore.SIGNAL("clicked()"), self.emitReturnSelectedTransAndExit)
        self.connect(self.CancelButton,QtCore.SIGNAL("clicked()"), self.reject)

    def filterParChanged(self):
        """
        Updates the label widgets if the filtered parameter has chnaged in the combobox widget
        """

        parID = self.FilterParSelectorCBox.currentIndex()
        if parID==0:
            self.FilterMinLabel.setText('Min ')
            self.FilterMaxLabel.setText('Max ')
        if parID==1:
            self.FilterMinLabel.setText('Min [GHz]')
            self.FilterMaxLabel.setText('Max [GHz]')
        if parID==2:
            self.FilterMinLabel.setText('Min [1/s]')
            self.FilterMaxLabel.setText('Max [1/s]')
        if parID==3:
            self.FilterMinLabel.setText('Min [K]')
            self.FilterMaxLabel.setText('Max [K]')
    def emitReturnSelectedTransAndExit(self):
        """
        Emits the returnSelectedTransAndExit signal 
        """
        selectedIDs = []

        if self.SingleSelectMode:
            self.usDataModel.clear()
            
            ids = self.AvailableTransitionsTreeView.selectedIndexes()
            
            if len(ids)<1: 
                reply = QtGui.QMessageBox.critical(None,"Error", " You must select a transition for raytracing", QtGui.QMessageBox.Ok)
                return

            data = []
            index = ids[0]
            item = self.avDataModel.item(index.row(), index.column())
            selectedIDs.append(str(item.text()))
               
        else:
            for i in range(self.usDataModel.rowCount()):
                selectedIDs.append(str(self.usDataModel.item(i,0).text()))

            if (len(selectedIDs)==0):
                reply = QtGui.QMessageBox.critical(None,"Error", " You must select at least one transition for raytracing", QtGui.QMessageBox.Ok)
                return
            elif (selectedIDs[0].strip()==''):
                reply = QtGui.QMessageBox.critical(None,"Error", " You must select at least one transition for raytracing", QtGui.QMessageBox.Ok)
                return

        self.returnSelectedTrans.emit(selectedIDs)
        self.accept()
    def applyFilter(self):
        """
        Apply filters to the data model
        """
        if self.FilterGroupBox.isChecked():
            parID = self.FilterParSelectorCBox.currentIndex()

            dum1 = str(self.FilterMinLedit.text())
            dum2 = str(self.FilterMaxLedit.text())

            if (dum1.strip()=='')&(dum2.strip()==''):
                return


            if parID == 0:
                dum = self.data['ID']
                parMin = str2int(self.FilterMinLedit.text(), 'Min')
                parMax = str2int(self.FilterMaxLedit.text(), 'Max')
            elif parID == 1:
                dum = self.data['nu0']
                parMin = str2float(self.FilterMinLedit.text(), 'Min')
                parMax = str2float(self.FilterMaxLedit.text(), 'Max')

            elif parID == 2:
                dum = self.data['A']
                parMin = str2float(self.FilterMinLedit.text(), 'Min')
                parMax = str2float(self.FilterMaxLedit.text(), 'Max')

            elif parID == 3:
                dum = self.data['eupk']
                parMin = str2float(self.FilterMinLedit.text(), 'Min')
                parMax = str2float(self.FilterMaxLedit.text(), 'Max')
          

            ii = [i for i in range(len(dum)) if (dum[i]>=parMin)&(dum[i]<=parMax)]

            data2 = {}
            data2['ID']   = [self.data['ID'][i] for i in ii]
            data2['nu0']  = [self.data['nu0'][i] for i in ii]
            data2['A']    = [self.data['A'][i] for i in ii]
            data2['eupk'] = [self.data['eupk'][i] for i in ii]

            self.avDataModel = QtGui.QStandardItemModel(0, 4, self)
            self.avDataModel.setHeaderData(0, QtCore.Qt.Horizontal, "ID")
            self.avDataModel.setHeaderData(1, QtCore.Qt.Horizontal, "Frequency [GHz]")
            self.avDataModel.setHeaderData(2, QtCore.Qt.Horizontal, "Einstein A [1/s]")
            self.avDataModel.setHeaderData(3, QtCore.Qt.Horizontal, "Eup/k [K]")

            for i in range(len(data2['ID'])):
                self.addNewRowtoDataModel(self.avDataModel, data2['ID'][i], data2['nu0'][i], data2['A'][i], data2['eupk'][i])

            self.avProxyModel = QtGui.QSortFilterProxyModel()
            self.avProxyModel.setDynamicSortFilter(True)

            self.avProxyModel.setSourceModel(self.avDataModel)
            self.AvailableTransitionsTreeView.setModel(self.avDataModel)
        else:
            self.avDataModel = QtGui.QStandardItemModel(0, 4, self)
            self.avDataModel.setHeaderData(0, QtCore.Qt.Horizontal, "ID")
            self.avDataModel.setHeaderData(1, QtCore.Qt.Horizontal, "Frequency [GHz]")
            self.avDataModel.setHeaderData(2, QtCore.Qt.Horizontal, "Einstein A [1/s]")
            self.avDataModel.setHeaderData(3, QtCore.Qt.Horizontal, "Eup/k [K]")


            for i in range(len(self.data['ID'])):
                self.addNewRowtoDataModel(self.avDataModel, self.data['ID'][i], self.data['nu0'][i], self.data['A'][i], self.data['eupk'][i])
            
            self.avProxyModel = QtGui.QSortFilterProxyModel()
            self.avProxyModel.setDynamicSortFilter(True)

            self.avProxyModel.setSourceModel(self.avDataModel)
            self.AvailableTransitionsTreeView.setModel(self.avDataModel)


    def removeUsTransition(self):
        """
        Removes used transition(s) 
        """

        ids = self.UsedTransitionsTreeView.selectedIndexes()
        nrows = len(ids)/4
        for irow in range(nrows):
            index = ids[irow * 4]
            rid = index.row()
            self.usDataModel.removeRow(rid)
        
        self.usProxyModel = QtGui.QSortFilterProxyModel()
        self.usProxyModel.setDynamicSortFilter(True)

        self.usProxyModel.setSourceModel(self.usDataModel)
        self.UsedTransitionsTreeView.setModel(self.usDataModel)

    def addAvTransition(self):
        """
        Add new transition to the used set of transitions
        """


        nrows = self.usDataModel.rowCount()
        usedTransIDs = []
        for irow in range(nrows):
            item = self.usDataModel.item(irow, 0)
            usedTransIDs.append(str(item.text()))

        ids = self.AvailableTransitionsTreeView.selectedIndexes()
        nrows = len(ids)/4
        for irow in range(nrows):
            data = []
            if str(self.avDataModel.item(ids[irow*4].row(), 0).text())  not in usedTransIDs:
                for icol in range(4):
                    index = ids[irow*4 + icol]
                    item = self.avDataModel.item(index.row(), index.column())
                    data.append(str(item.text()))
               
                self.addNewRowtoDataModel(self.usDataModel, data[0], data[1], data[2], data[3])

        self.usProxyModel = QtGui.QSortFilterProxyModel()
        self.usProxyModel.setDynamicSortFilter(True)

        self.usProxyModel.setSourceModel(self.usDataModel)
        self.UsedTransitionsTreeView.setModel(self.usDataModel)


    def addNewRowtoDataModel(self, datamodel, trans_id, trans_nu0, trans_a, trans_eupk):
        """
        Add new row to the data model
        """
        datamodel.insertRow(0)
        datamodel.setData(datamodel.index(0, 0), trans_id) 
        datamodel.setData(datamodel.index(0, 1), trans_nu0) 
        datamodel.setData(datamodel.index(0, 2), trans_a) 
        datamodel.setData(datamodel.index(0, 3), trans_eupk) 

class AboutDialog(QtGui.QDialog, Ui_AboutDialog):

    def __init__(self, parent=None):
        super(AboutDialog, self).__init__(parent)
        self.setupUi(self)

        self.connect(self.CloseButton,QtCore.SIGNAL("clicked()"), self.accept)

        
class AbundanceCustomDialog(QtGui.QDialog, Ui_AbunRegionCusmizationDialog):
    """
    Modellib Browser dialog
    """
    applyCustomAbundance = QtCore.pyqtSignal(object,name='applyCustomAbundance')


    def __init__(self, parent=None, minScale=None, radius=None, abunModel=None, configParams=None):
        super(AbundanceCustomDialog, self).__init__(parent)
        self.setupUi(self)
       
        if configParams!=None:
            self.setStyleSheet('font-family: '+configParams.windowParams['font_family']+\
                    '; font-size: '+configParams.windowParams['font_size']+'pt;')

        self.AbunReg0LEdit.setText('0.0')
        self.AbunReg0LEdit.setReadOnly(True)


        self.setTabOrder(self.BoundSeparatorCBox, self.BoundNumberCBox)
        self.setTabOrder(self.BoundNumberCBox, self.BoundRegUnitCBox)
        self.setTabOrder(self.BoundRegUnitCBox, self.AbunReg1LEdit)
        self.setTabOrder(self.AbunReg1LEdit, self.BoundReg12LEdit)
        self.setTabOrder(self.BoundReg12LEdit, self.AbunReg2LEdit)
        self.setTabOrder(self.AbunReg2LEdit, self.BoundReg23LEdit)
        self.setTabOrder(self.BoundReg23LEdit, self.AbunReg3LEdit)
        self.setTabOrder(self.AbunReg3LEdit, self.BoundReg34LEdit)
        self.setTabOrder(self.BoundReg34LEdit, self.AbunReg4LEdit)
        self.setTabOrder(self.AbunReg4LEdit, self.BoundReg45LEdit)
        self.setTabOrder(self.BoundReg45LEdit, self.AbunReg5LEdit)
        self.setTabOrder(self.AbunReg5LEdit, self.DoneButton)
        self.setTabOrder(self.DoneButton, self.CancelButton)

        self.connect(self.CancelButton,QtCore.SIGNAL("clicked()"), self.reject)
        self.connect(self.DoneButton,QtCore.SIGNAL("clicked()"), self.readOutandWriteTable)
        self.connect(self.BoundSeparatorCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.changeRegionSeparator)
        self.connect(self.BoundNumberCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.changeNumberOfRegions)

        self.abunRegLEdit = [self.AbunReg1LEdit, self.AbunReg2LEdit, self.AbunReg3LEdit, self.AbunReg4LEdit, self.AbunReg5LEdit]
        self.regBoundLEdit = [self.BoundReg12LEdit, self.BoundReg23LEdit, self.BoundReg34LEdit, self.BoundReg45LEdit]
        self.abunRegLabel = [self.AbunReg1Label, self.AbunReg2Label, self.AbunReg3Label, self.AbunReg4Label, self.AbunReg5Label]
        self.regBoundLabel = [self.BoundReg12Label, self.BoundReg23Label, self.BoundReg34Label, self.BoundReg45Label]
        self.regLabel = [self.Reg1Label, self.Reg2Label, self.Reg3Label, self.Reg4Label, self.Reg5Label]
        self.abunModel = abunModel

        self.BoundSeparatorCBox.clear()
        self.BoundSeparatorCBox.addItems(self.abunModel.regionSeparators)
        self.BoundSeparatorCBox.setCurrentIndex(0)
        self.changeRegionSeparator(0)
        self.changeNumberOfRegions(0)


        self.minScale = minScale
        self.radius   = radius

        self.setAbundData()
    def changeRegionSeparator(self, index=None):

        dum = str(self.BoundSeparatorCBox.currentText()).strip()

        self.BoundRegUnitCBox.clear()
        self.BoundRegUnitCBox.addItems(self.abunModel.regionSeparatorUnits[dum])
        index = self.abunModel.regionSeparatorUnits[dum].index(self.abunModel.regionSeparatorUnitSelection[dum])
        self.BoundRegUnitCBox.setCurrentIndex(index)

    def changeNumberOfRegions(self, index=None):

        nreg = int(str(self.BoundNumberCBox.currentText()).strip()) - 1
        nreg_max = 5

        for i in range(nreg):
            self.regLabel[i+1].setVisible(True)
            self.abunRegLEdit[i+1].setVisible(True)
            self.abunRegLabel[i+1].setVisible(True)
            self.regBoundLEdit[i].setVisible(True)
            self.regBoundLabel[i].setVisible(True)
        for i in range(nreg, nreg_max-1):
            self.regLabel[i+1].setVisible(False)
            self.abunRegLEdit[i+1].setVisible(False)
            self.abunRegLabel[i+1].setVisible(False)
            self.regBoundLEdit[i].setVisible(False)
            self.regBoundLabel[i].setVisible(False)

    def setAbundData(self):
            
        self.BoundSeparatorCBox.setCurrentIndex(self.abunModel.regionSeparatorsCurrentIndex)
        separator = self.abunModel.regionSeparators[self.abunModel.regionSeparatorsCurrentIndex]
        self.BoundNumberCBox.setCurrentIndex(self.abunModel.nRegion-2)
        index = self.BoundRegUnitCBox.findText(self.abunModel.regionSeparatorUnitSelection[separator])
        self.BoundRegUnitCBox.setCurrentIndex(index)

        for i in range(self.abunModel.nRegion-1):
            self.regBoundLEdit[i].setText(str(self.abunModel.regionBoundaries[i]))    
            self.abunRegLEdit[i].setText(str(self.abunModel.regionAbundances[i]))    
            
        self.abunRegLEdit[self.abunModel.nRegion-1].setText(str(self.abunModel.regionAbundances[self.abunModel.nRegion-1]))    


    def readOutAbundData(self):
        
        self.abunModel.regionSeparatorsCurrentIndex = self.BoundSeparatorCBox.currentIndex()
        separator = self.abunModel.regionSeparators[self.abunModel.regionSeparatorsCurrentIndex]
        log.debug('Separator : '+separator)
        self.abunModel.regionSeparatorUnitSelection[separator] = str(self.BoundRegUnitCBox.currentText()).strip()
        log.debug('Unit : '+self.abunModel.regionSeparatorUnitSelection[separator])
        self.abunModel.nRegion = int(self.BoundNumberCBox.currentText())
        log.debug('Number of Regions : '+str(self.abunModel.nRegion))

        boundaries = []
        regionAbundances = []

        for i in range(self.abunModel.nRegion-1):
            boundaries.append(str2float(self.regBoundLEdit[i].text(), 'Region boundaries (Abundances)'))
            regionAbundances.append(str2float(self.abunRegLEdit[i].text(), 'Region abundances (Abundances)'))
            
            log.debug('Region #'+str(i)+' abundance : '+str(regionAbundances[i]))

            log.debug('Region #'+str(i)+' boundary : '+str(boundaries[i]))


        if len(boundaries)>1:
            for i in range(len(boundaries)-1):
                if boundaries[i]>=boundaries[i+1]:
                    reply = QtGui.QMessageBox.critical(None,"Error", 'Region boundaries must be in monotonic ascending order '+ \
                            ' from Region #1 to Region #N.', QtGui.QMessageBox.Ok)

                    log.debug('ERROR : abundance region boundaries are not in ascending order')
                    return False
                
        regionAbundances.append(str2float(self.abunRegLEdit[self.abunModel.nRegion-1].text(), 'Region abundances (Abundances)'))
        log.debug('Region #'+str(i)+' abundance : '+str(regionAbundances[i]))


        self.abunModel.regionAbundances = regionAbundances
        self.abunModel.regionBoundaries = boundaries

        return True

    def readOutandWriteTable(self):

        log.debug('Region Abundances')
        dum = self.readOutAbundData()
        if dum :
            log.debug('Writing Abundance table...')
            self.abunModel.makeAbundanceTable(self.minScale, self.radius, writeTable=True)
           
            
            self.applyCustomAbundance.emit(self.abunModel)
            self.accept()

class ModellibBrowser(QtGui.QDialog, Ui_ModellibBrowserDialog):
    """
    Modellib Browser dialog
    """
    def __init__(self, parent=None, configParams=None):
        super(ModellibBrowser, self).__init__(parent)
        self.setupUi(self)
        
        if configParams!=None:
            self.setStyleSheet('font-family: '+configParams.windowParams['font_family']+\
                    '; font-size: '+configParams.windowParams['font_size']+'pt;')


        self.connect(self.ModellibListWidget,QtCore.SIGNAL("currentRowChanged(int)"), self.modelNameChanged)
        self.connect(self.CancelButton,QtCore.SIGNAL("clicked()"), self.reject)
        self.connect(self.SelectButton,QtCore.SIGNAL("clicked()"), self.selectAndExit)

        self.modelNames = [i for i in modellib.getModelIDs()]
        self.currentModel = modellib.getCurrentModelID()
        if self.currentModel.strip()=='':
            self.currentModel = modelNames[0]
        if self.modelNames.__contains__('user'):
            self.modelNames.remove('user')

        self.valList = QtCore.QStringList()

        for item in self.modelNames:
            self.valList.append(item)
        
        self.ModellibListWidget.addItems(self.valList)

        item = self.ModellibListWidget.findItems(QtCore.QString(self.currentModel),QtCore.Qt.MatchFlags()) 
        self.ModellibListWidget.setCurrentItem(item[0])
        self.setModelDescLabels(self.currentModel)


    def selectAndExit(self):

        modelName = str(self.ModellibListWidget.selectedItems()[0].text())
        modellib.setCurrentModel(modelName)
        self.accept()

    def modelNameChanged(self, index=None):

        modelName = str(self.ModellibListWidget.item(index).text())
        self.setModelDescLabels(modelName)


    def setModelDescLabels(self, modelID=None):
        """
        Function to set all the descriptor labels
        """
        
        self.ModelNameLabel.setText(modelID)
        self.ModelDescLabel.setText(modellib.getModelDesc(modelID))
        self.ModelBibrefLabel.setText(modellib.getModelBibref(modelID))

        parNameLabel = [self.Par1NameLabel,self.Par2NameLabel,self.Par3NameLabel,self.Par4NameLabel,self.Par5NameLabel,\
                        self.Par6NameLabel,self.Par7NameLabel,self.Par8NameLabel,self.Par9NameLabel,self.Par10NameLabel]

        parDescLabel = [self.Par1DescLabel,self.Par2DescLabel,self.Par3DescLabel,self.Par4DescLabel,self.Par5DescLabel,\
                        self.Par6DescLabel,self.Par7DescLabel,self.Par8DescLabel,self.Par9DescLabel,self.Par10DescLabel]


        modPars = modellib.getParamIDs(modelID)

        for i in range(len(modPars)):
            if modellib.getParamType(modelID, modPars[i])=='string':
                txt = modPars[i]
            else:
                txt = modPars[i]+' ['+modellib.getParamUnit(modelID, modPars[i])+']'
            parNameLabel[i].setText(txt)
            parNameLabel[i].setVisible(True)
        
            parDescLabel[i].setText(modellib.getParamDesc(modelID, modPars[i]))
            parDescLabel[i].setVisible(True)

        for i in range(len(modPars), 10):
            parNameLabel[i].setVisible(False)
            parDescLabel[i].setVisible(False)


class VisualizeMainWindow(QtGui.QMainWindow, Ui_VisualizeMainWindow):
    def __init__(self, parent = None):
        super(VisualizeMainWindow, self).__init__(parent)
        self.setupUi(self)

        self.showToolTipHelp = False

        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+right"), self, self.moveToNextTab)
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+left"), self, self.moveToPreviousTab)

        self.Plot2DTabWidget.setCurrentIndex(0)
        self.setTabOrder(self.LCont1Button, self.LCont1CBox)
        self.setTabOrder(self.LCont1CBox, self.LCont1MoreButton)
        self.setTabOrder(self.LCont1MoreButton, self.LCont2Button)

        self.setTabOrder(self.LCont2Button, self.LCont2CBox)
        self.setTabOrder(self.LCont2CBox, self.LCont2MoreButton)
        self.setTabOrder(self.LCont2MoreButton, self.LCont3Button)
        
        self.setTabOrder(self.LCont3Button, self.LCont3CBox)
        self.setTabOrder(self.LCont3CBox, self.LCont3MoreButton)
        self.setTabOrder(self.LCont3MoreButton, self.LCont4Button)
        
        self.setTabOrder(self.LCont4Button, self.LCont4CBox)
        self.setTabOrder(self.LCont4CBox, self.LCont4MoreButton)
        self.setTabOrder(self.LCont4MoreButton, self.LCont5Button)
        
        self.setTabOrder(self.LCont5Button, self.LCont5CBox)
        self.setTabOrder(self.LCont5CBox, self.LCont5MoreButton)
        self.setTabOrder(self.LCont5MoreButton, self.LCont6Button)
        
        self.setTabOrder(self.LCont6Button, self.LCont6CBox)
        self.setTabOrder(self.LCont6CBox, self.LCont6MoreButton)
        self.setTabOrder(self.LCont6MoreButton, self.FContComboBox)
        self.setTabOrder(self.FContComboBox, self.FContMoreButton)
        
        self.setTabOrder(self.FContMoreButton, self.CSectionXZRButton)
        self.setTabOrder(self.CSectionXZRButton, self.CSectionYZRButton)
        self.setTabOrder(self.CSectionYZRButton, self.CSectionXYRButton)
        self.setTabOrder(self.CSectionXYRButton, self.CSection3rdAxisValueLedit)
        self.setTabOrder(self.CSection3rdAxisValueLedit, self.Plot2DXrange)

        self.setTabOrder(self.Plot2DXrange, self.Plot2DYrange)
        self.setTabOrder(self.Plot2DYrange, self.Plot2DGrabBoundFigureButton)
        self.setTabOrder(self.Plot2DGrabBoundFigureButton, self.Plot2DUnitSelectorCBox)

        self.setTabOrder(self.PGridNXLedit, self.PGridNYLedit)
        self.setTabOrder(self.PGridNYLedit, self.PGridNZLedit)
        self.setTabOrder(self.PGridNZLedit, self.PGridXminLedit)
        self.setTabOrder(self.PGridXminLedit, self.PGridYminLedit)
        self.setTabOrder(self.PGridYminLedit, self.PGridZminLedit)
        self.setTabOrder(self.PGridZminLedit, self.PGridXmaxLedit)
        self.setTabOrder(self.PGridXmaxLedit, self.PGridYmaxLedit)
        self.setTabOrder(self.PGridYmaxLedit, self.PGridZmaxLedit)
        self.setTabOrder(self.PGridZmaxLedit, self.PGridXlogCheckBox)
        self.setTabOrder(self.PGridXlogCheckBox, self.PGridYlogCheckBox)
        self.setTabOrder(self.PGridYlogCheckBox, self.PGridYCoupleToXCheckBox)
        self.setTabOrder(self.PGridYCoupleToXCheckBox, self.PGridZlogCheckBox)
        self.setTabOrder(self.PGridZlogCheckBox, self.PGridZCoupleToXCheckBox)
        self.setTabOrder(self.PGridZCoupleToXCheckBox, self.PGridUpdateButton)


        self.setTabOrder(self.Plot1DcrdCartesianRButton, self.Plot1DcrdSphericalRButton)
        self.setTabOrder(self.Plot1DcrdSphericalRButton, self.Plot1Dcrd1RButton)
        self.setTabOrder(self.Plot1Dcrd1RButton, self.Plot1Dcrd2RButton)
        self.setTabOrder(self.Plot1Dcrd2RButton, self.Plot1Dcrd3RButton)
        self.setTabOrder(self.Plot1Dcrd3RButton, self.Plot1Dcrd2Ledit)
        self.setTabOrder(self.Plot1Dcrd2Ledit, self.Plot1Dcrd3Ledit)
        self.setTabOrder(self.Plot1Dcrd3Ledit, self.Plot1DVarSelectorCBox)
        self.setTabOrder(self.Plot1DVarSelectorCBox, self.Plot1DXrangeLedit)
        self.setTabOrder(self.Plot1DXrangeLedit, self.Plot1DXrangeResetButton)
        self.setTabOrder(self.Plot1DXrangeResetButton, self.Plot1DXlogCheckBox)
        self.setTabOrder(self.Plot1DXlogCheckBox, self.Plot1DYrangeLedit)
        self.setTabOrder(self.Plot1DYrangeLedit, self.Plot1DYrangeResetButton)
        self.setTabOrder(self.Plot1DYrangeResetButton, self.Plot1DYlogCheckBox)
        self.setTabOrder(self.Plot1DYlogCheckBox, self.Plot1DNrofPointsLedit)



        #self.Plot1DVarSelectorCBox.addItems(self.mplFigureWidget1D.canvas.AllVarName)
        varNames = []
        keys = self.mplFigureWidget1D.canvas.AllVarLabel.keys()
        for ikey in keys:
            varNames.append(self.mplFigureWidget1D.canvas.AllVarLabel[ikey])
        self.Plot1DVarSelectorCBox.addItems(varNames[::-1])
        self.Plot1DVarSelectorCBox.setCurrentIndex(0)
        

        self.connect(self.Plot1DcrdCartesianRButton, QtCore.SIGNAL("clicked(bool)"), self.updateFigure1D)
        self.connect(self.Plot1DcrdSphericalRButton, QtCore.SIGNAL("clicked(bool)"), self.updateFigure1D)

        self.connect(self.Plot1Dcrd1RButton, QtCore.SIGNAL("clicked(bool)"), self.updateFigure1DNewYRange)
        self.connect(self.Plot1Dcrd2RButton, QtCore.SIGNAL("clicked(bool)"), self.updateFigure1DNewYRange)
        self.connect(self.Plot1Dcrd3RButton, QtCore.SIGNAL("clicked(bool)"), self.updateFigure1DNewYRange)
        
        self.connect(self.Plot1Dcrd2Ledit, QtCore.SIGNAL("returnPressed()"), self.updateFigure1DNewYRange)
        self.connect(self.Plot1Dcrd3Ledit, QtCore.SIGNAL("returnPressed()"), self.updateFigure1DNewYRange)
        
        self.connect(self.Plot1DXlogCheckBox, QtCore.SIGNAL("clicked(bool)"), self.replotScaleFigure1D)
        self.connect(self.Plot1DXrangeResetButton, QtCore.SIGNAL("clicked(bool)"), self.replotXRangeFigure1D)

        self.connect(self.Plot1DYrangeResetButton, QtCore.SIGNAL("clicked(bool)"), self.replotYRangeFigure1D)
        self.connect(self.Plot1DYlogCheckBox, QtCore.SIGNAL("clicked(bool)"), self.replotScaleFigure1D)

       
        self.connect(self.Plot1DXrangeLedit, QtCore.SIGNAL("returnPressed()"), self.updateFigure1D)
        self.connect(self.Plot1DYrangeLedit, QtCore.SIGNAL("returnPressed()"), self.updateFigure1D)
        self.connect(self.Plot1DNrofPointsLedit, QtCore.SIGNAL("returnPressed()"), self.updateFigure1D)
        
        self.connect(self.Plot1DVarSelectorCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.updateFigure1DNewYRange)

        self.CurrentModelID = 0
        modellib.setCurrentModel(modellib.getModelIDs()[self.CurrentModelID])        
        
        self.connect(self.LCont1MoreButton,QtCore.SIGNAL("clicked()"), self.openLGridCont1Dialog)
        self.connect(self.LCont2MoreButton,QtCore.SIGNAL("clicked()"), self.openLGridCont2Dialog)
        self.connect(self.LCont3MoreButton,QtCore.SIGNAL("clicked()"), self.openLGridCont3Dialog)
        self.connect(self.LCont4MoreButton,QtCore.SIGNAL("clicked()"), self.openLGridCont4Dialog)
        self.connect(self.LCont5MoreButton,QtCore.SIGNAL("clicked()"), self.openLGridCont5Dialog)
        self.connect(self.LCont6MoreButton,QtCore.SIGNAL("clicked()"), self.openLGridCont6Dialog)

        self.connect(self.FContMoreButton,QtCore.SIGNAL("clicked()"), self.openFGridDialog)
        self.connect(self.FContComboBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.drawFilledContours)
        self.connect(self.Plot2DUnitSelectorCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.plot2DChangeAxisUnit)

        self.connect(self.LCont1Button, QtCore.SIGNAL("stateChanged(int)"), self.drawContourLines)
        self.connect(self.LCont2Button, QtCore.SIGNAL("stateChanged(int)"), self.drawContourLines)
        self.connect(self.LCont3Button, QtCore.SIGNAL("stateChanged(int)"), self.drawContourLines)
        self.connect(self.LCont4Button, QtCore.SIGNAL("stateChanged(int)"), self.drawContourLines)
        self.connect(self.LCont5Button, QtCore.SIGNAL("stateChanged(int)"), self.drawContourLines)
        self.connect(self.LCont6Button, QtCore.SIGNAL("stateChanged(int)"), self.drawContourLines)
       
        self.connect(self.LCont1CBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.drawContourLines)
        self.connect(self.LCont2CBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.drawContourLines)
        self.connect(self.LCont3CBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.drawContourLines)
        self.connect(self.LCont4CBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.drawContourLines)
        self.connect(self.LCont5CBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.drawContourLines)
        self.connect(self.LCont6CBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.drawContourLines)

        varNames = []
        keys = self.mplFigureWidget2D.canvas.PlottedLContVarName
        for ikey in keys:
            varNames.append(self.mplFigureWidget2D.canvas.PlottedFContVarLabel[ikey])

        self.LCont1CBox.addItems(varNames)
        self.LCont2CBox.addItems(varNames)
        self.LCont3CBox.addItems(varNames)
        self.LCont4CBox.addItems(varNames)
        self.LCont5CBox.addItems(varNames)
        self.LCont6CBox.addItems(varNames)

        self.FContComboBox.addItems(varNames)
        
        self.connect(self.LCont1CBox, QtCore.SIGNAL("currentIndexChanged()"), self.reDrawContours)        


        self.connect(self.PGridUpdateButton, QtCore.SIGNAL("clicked(bool)"), self.updatePGrid)        
        self.connect(self.PGridYCoupleToXCheckBox, QtCore.SIGNAL("clicked(bool)"), self.updatePGrid)
        self.connect(self.PGridZCoupleToXCheckBox, QtCore.SIGNAL("clicked(bool)"), self.updatePGrid)

        self.connect(self.CSectionXZRButton, QtCore.SIGNAL("clicked(bool)"), self.changeCrossSection)
        self.connect(self.CSectionYZRButton, QtCore.SIGNAL("clicked(bool)"), self.changeCrossSection)
        self.connect(self.CSectionXYRButton, QtCore.SIGNAL("clicked(bool)"), self.changeCrossSection)
        self.connect(self.CSection3rdAxisValueLedit, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)

        self.connect(self.PGridXmaxLedit, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        
        self.connect(self.PGridYmaxLedit, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        
        self.connect(self.PGridZmaxLedit, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        
        self.connect(self.PGridNXLedit, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        
        self.connect(self.PGridNYLedit, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        
        self.connect(self.PGridNZLedit, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        

        self.connect(self.Plot2DXrange, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        
        self.connect(self.Plot2DYrange, QtCore.SIGNAL("returnPressed()"), self.reDrawContours)        


        self.connect(self.mplFigureWidget2D.canvas, QtCore.SIGNAL("update2DContours()"), self.plot2DChangeXYLimMouseClick)
        self.connect(self.PControlUpdateButton, QtCore.SIGNAL("clicked()"), self.forceUpdateAllPlots) 

        self.setPGridWidgetValues()
        self.setCrossSectionValues()

        self.nc = natconst()

        self.Plot2DXrange.setText('-100.0, 100.0')
        self.Plot2DYrange.setText('-100.0, 100.0')
        self.PGridXminLedit.setText('-100.0')
        self.PGridYminLedit.setText('-100.0')
        self.PGridZminLedit.setText('-100.0')
        self.PGridXmaxLedit.setText('100.0')
        self.PGridYmaxLedit.setText('100.0')
        self.PGridZmaxLedit.setText('100.0')
        self.mplFigureWidget2D.canvas.PlottedFContVarName = 'density'
        self.enableDisableAxisCoupling()

        self.setXrange1D()
        self.setYrange1D()
        self.setNpoints1D()
        self.set2nd3rdCoordinate1D()
    
    def hideAllToolTips(self):

        self.showToolTipHelp = False

        self.Plot1DcrdsysLabel.setToolTip('')
        self.Plot1DcrdCartesianRButton.setToolTip('')
        self.Plot1DcrdSphericalRButton.setToolTip('')
        self.Plot1Dcrd1RButton.setToolTip('')
        self.Plot1Dcrd2RButton.setToolTip('')
        self.Plot1Dcrd3RButton.setToolTip('')

        self.Plot1Dcrd2Ledit.setToolTip('')
        self.Plot1Dcrd3Ledit.setToolTip('')
        self.Plot1DVarSelectorCBox.setToolTip('')
        self.Plot1DXrangeLedit.setToolTip('')
        self.Plot1DXrangeResetButton.setToolTip('')
        self.Plot1DXrangeResetButton.setToolTip('')

        self.Plot1DYrangeLedit.setToolTip('')
        self.Plot1DYrangeResetButton.setToolTip('')
        self.Plot1DYrangeResetButton.setToolTip('')
        self.Plot1DNrofPointsLedit.setToolTip('')
        
   

        self.LCont1Button.setToolTip('')
        self.LCont1CBox.setToolTip('')
        self.LCont1MoreButton.setToolTip('')

        self.LCont2Button.setToolTip('')
        self.LCont2CBox.setToolTip('')
        self.LCont2MoreButton.setToolTip('')
        
        self.LCont3Button.setToolTip('')
        self.LCont3CBox.setToolTip('')
        self.LCont3MoreButton.setToolTip('')

        self.LCont4Button.setToolTip('')
        self.LCont4CBox.setToolTip('')
        self.LCont4MoreButton.setToolTip('')
        
        self.LCont5Button.setToolTip('')
        self.LCont5MoreButton.setToolTip('')
        
        self.LCont6Button.setToolTip('')
        self.LCont6CBox.setToolTip('')
        self.LCont6MoreButton.setToolTip('')
        
        self.FContComboBox.setToolTip('')
        self.FContMoreButton.setToolTip('')


        self.CSectionXZRButton.setToolTip('')
        self.CSectionYZRButton.setToolTip('')
        self.CSectionXYRButton.setToolTip('')
        self.CSection3rdAxisValueLedit.setToolTip('')

        self.Plot2DXrange.setToolTip('')
        self.Plot2DYrange.setToolTip('')
        self.Plot2DGrabBoundFigureButton.setToolTip('')
        self.Plot2DUnitSelectorCBox.setToolTip('')
        self.PControlUpdateButton.setToolTip('')
       

        self.PGridNXLedit.setToolTip('')
        self.PGridXminLedit.setToolTip('')
        self.PGridXmaxLedit.setToolTip('')
        self.PGridXlogCheckBox.setToolTip('')
        
        self.PGridNYLedit.setToolTip('')
        self.PGridYminLedit.setToolTip('')
        self.PGridYmaxLedit.setToolTip('')
        self.PGridYlogCheckBox.setToolTip('')
        self.PGridYCoupleToXCheckBox.setToolTip('')
        
        self.PGridNZLedit.setToolTip('')
        self.PGridZminLedit.setToolTip('')
        self.PGridZmaxLedit.setToolTip('')
        self.PGridZlogCheckBox.setToolTip('')
        self.PGridZCoupleToXCheckBox.setToolTip('')
        self.PGridUpdateButton.setToolTip('')


    def setAllToolTips(self):
        
        self.showToolTipHelp = True

        self.Plot1DcrdsysLabel.setToolTip(str2ToolTipString('Select the coordinate system whose coordinates should be used for plotting'))
        self.Plot1DcrdCartesianRButton.setToolTip(str2ToolTipString('Plot variables along one of the Cartesian coordinates'))
        self.Plot1DcrdSphericalRButton.setToolTip(str2ToolTipString('Plot variables along one of the spherical coordinates'))
        self.Plot1Dcrd1RButton.setToolTip(str2ToolTipString('Coordinate to be plotted on the horizontal axis'))
        self.Plot1Dcrd2RButton.setToolTip(str2ToolTipString('Coordinate to be plotted on the horizontal axis'))
        self.Plot1Dcrd3RButton.setToolTip(str2ToolTipString('Coordinate to be plotted on the horizontal axis'))

        self.Plot1Dcrd2Ledit.setToolTip(str2ToolTipString('Set a fixed value for this coordinate. (All physical variables are defined in three dimensions. To make a 1D plot two of the coordinates need to be fixed.)'))
        self.Plot1Dcrd3Ledit.setToolTip(str2ToolTipString('Set a fixed value for this coordinate. (All physical variables are defined in three dimensions. To make a 1D plot two of the coordinates need to be fixed.)'))
        self.Plot1DVarSelectorCBox.setToolTip(str2ToolTipString('Select the variable to be plotted'))
        self.Plot1DXrangeLedit.setToolTip(str2ToolTipString('Select the plotted range of the horizontal axis'))
        self.Plot1DXrangeResetButton.setToolTip(str2ToolTipString('Reset the plotted range of the horizontal axis to [minScale,Radius] (i.e. to the inner and outer radius of the model)'))
        self.Plot1DXlogCheckBox.setToolTip(str2ToolTipString('Use logarithmic scale in the horizontal axis instead of linear scale.'))

        self.Plot1DYrangeLedit.setToolTip(str2ToolTipString('Select the plotted range of the vertical axis'))
        self.Plot1DYrangeResetButton.setToolTip(str2ToolTipString('Reset the plotted range of the vertical axis to the data range [min(variable), max(variable)] in the selected xrange interval'))
        self.Plot1DYlogCheckBox.setToolTip(str2ToolTipString('Use logarithmic scale in the vertical axis instead of linear scale.'))
        self.Plot1DNrofPointsLedit.setToolTip(str2ToolTipString('Number of points to be used in the plot'))
        
   

        self.LCont1Button.setToolTip(str2ToolTipString('Enable/disable the contour lines of this variable'))
        self.LCont1CBox.setToolTip(str2ToolTipString('Select the physical variable to be plotted'))
        self.LCont1MoreButton.setToolTip(str2ToolTipString('Set the parameters of the contours (e.g., lower- and upper bounds, stepsize, lin-/log-scale, colors)'))

        self.LCont2Button.setToolTip(str2ToolTipString('Enable/disable the contour lines of this variable'))
        self.LCont2CBox.setToolTip(str2ToolTipString('Select the physical variable to be plotted'))
        self.LCont2MoreButton.setToolTip(str2ToolTipString('Set the parameters of the contours (e.g., lower- and upper bounds, stepsize, lin-/log-scale, colors)'))
        
        self.LCont3Button.setToolTip(str2ToolTipString('Enable/disable the contour lines of this variable'))
        self.LCont3CBox.setToolTip(str2ToolTipString('Select the physical variable to be plotted'))
        self.LCont3MoreButton.setToolTip(str2ToolTipString('Set the parameters of the contours (e.g., lower- and upper bounds, stepsize, lin-/log-scale, colors)'))

        self.LCont4Button.setToolTip(str2ToolTipString('Enable/disable the contour lines of this variable'))
        self.LCont4CBox.setToolTip(str2ToolTipString('Select the physical variable to be plotted'))
        self.LCont4MoreButton.setToolTip(str2ToolTipString('Set the parameters of the contours (e.g., lower- and upper bounds, stepsize, lin-/log-scale, colors)'))
        
        self.LCont5Button.setToolTip(str2ToolTipString('Enable/disable the contour lines of this variable'))
        self.LCont5CBox.setToolTip(str2ToolTipString('Select the physical variable to be plotted'))
        self.LCont5MoreButton.setToolTip(str2ToolTipString('Set the parameters of the contours (e.g., lower- and upper bounds, stepsize, lin-/log-scale, colors)'))
        
        self.LCont6Button.setToolTip(str2ToolTipString('Enable/disable the contour lines of this variable'))
        self.LCont6CBox.setToolTip(str2ToolTipString('Select the physical variable to be plotted'))
        self.LCont6MoreButton.setToolTip(str2ToolTipString('Set the parameters of the contours (e.g., lower- and upper bounds, stepsize, lin-/log-scale, colors)'))
        
        self.FContComboBox.setToolTip(str2ToolTipString('Select the physical variable to be plotted in gray-/colorscale'))
        self.FContMoreButton.setToolTip(str2ToolTipString('Set the parameters of the contours (e.g., lower- and upper bounds, stepsize, lin-/log-scale, colors)'))


        self.CSectionXZRButton.setToolTip(str2ToolTipString('Plot contours in the XZ plane'))
        self.CSectionYZRButton.setToolTip(str2ToolTipString('Plot contours in the YZ plane'))
        self.CSectionXYRButton.setToolTip(str2ToolTipString('Plot contours in the XY plane'))
       
        if self.CSectionXZRButton.isChecked():
            self.CSection3rdAxisValueLedit.setToolTip(str2ToolTipString('Set the value of the Y coordinate'))
        if self.CSectionYZRButton.isChecked():
            self.CSection3rdAxisValueLedit.setToolTip(str2ToolTipString('Set the value of the X coordinate'))
        if self.CSectionXYRButton.isChecked():
            self.CSection3rdAxisValueLedit.setToolTip(str2ToolTipString('Set the value of the Z coordinate'))

        self.Plot2DXrange.setToolTip(str2ToolTipString('Select the plotted range of the horizontal axis (comma-separated list of minValue, maxValue)'))
        self.Plot2DYrange.setToolTip(str2ToolTipString('Select the plotted range of the vertical axis (comma-separated list of minValue, maxValue)'))
        self.Plot2DGrabBoundFigureButton.setToolTip(str2ToolTipString('Select the boundaries of the figure with the mouse. Check this box then click on the four-way arrow icon below the figure. Then right-click-and-drag in the figure to zoom in or out. Finally to adjust the resolution of the figure double click on the figure.'))
        self.Plot2DUnitSelectorCBox.setToolTip(str2ToolTipString('Select the unit of the figure axes'))
        self.PControlUpdateButton.setToolTip(str2ToolTipString('Update the figure'))
       

        self.PGridNXLedit.setToolTip(str2ToolTipString('Number of grid points used in the X-direction to calculate the plots (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridXminLedit.setToolTip(str2ToolTipString('Lower boundary of the grid, used to calculate the contour plot, in the X-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridXmaxLedit.setToolTip(str2ToolTipString('Upper boundary of the grid, used to calculate the contour plot, in the X-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridXlogCheckBox.setToolTip(str2ToolTipString('Use logarithmic scale in the grid, used to calculate the contour plot, in the X-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        
        self.PGridNYLedit.setToolTip(str2ToolTipString('Number of grid points used in the Y-direction to calculate the plots (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridYminLedit.setToolTip(str2ToolTipString('Lower boundary of the grid, used to calculate the contour plot, in the Y-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridYmaxLedit.setToolTip(str2ToolTipString('Upper boundary of the grid, used to calculate the contour plot, in the Y-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridYlogCheckBox.setToolTip(str2ToolTipString('Use logarithmic scale in the grid, used to calculate the contour plot, in the Y-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridYCoupleToXCheckBox.setToolTip(str2ToolTipString('Use the same grid parameters as for the X-axis (Nx, Xmin, Xmax)'))
        
        self.PGridNZLedit.setToolTip(str2ToolTipString('Number of grid points used in the Z-direction to calculate the plots (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridZminLedit.setToolTip(str2ToolTipString('Lower boundary of the grid, used to calculate the contour plot, in the Z-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridZmaxLedit.setToolTip(str2ToolTipString('Upper boundary of the grid, used to calculate the contour plot, in the Z-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridZlogCheckBox.setToolTip(str2ToolTipString('Use logarithmic scale in the grid, used to calculate the contour plot, in the Z-direction  (NOTE: this cartesian grid is used only for visualization! For the real calculation a 3D Delaunay grid will be used)'))
        self.PGridZCoupleToXCheckBox.setToolTip(str2ToolTipString('Use the same grid parameters as for the X-axis (Nx, Xmin, Xmax)'))
        self.PGridUpdateButton.setToolTip(str2ToolTipString('Update the figure'))

    def moveToPreviousTab(self):
        ind = self.Plot2DTabWidget.currentIndex()
        if ind==0: ind = 2
        self.Plot2DTabWidget.setCurrentIndex((ind-1)%2)

    def moveToNextTab(self):

        ind = self.Plot2DTabWidget.currentIndex()
        self.Plot2DTabWidget.setCurrentIndex((ind+1)%2)



    def updateFigure2DWithNewGrid(self, grid=None, gridcolor=None, varName=None, filledContours=0):


        varName = str(varName).strip()
        if filledContours==0:
            ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index(varName)
            self.mplFigureWidget2D.canvas.GridPar.LContGrid[varName] = copy.deepcopy(grid)
            self.mplFigureWidget2D.canvas.GridPar.LContGridColor[varName] = copy.deepcopy(gridcolor)
            self.reDrawContours()

        if filledContours==1:
            self.mplFigureWidget2D.canvas.GridPar.FContGrid[varName] = copy.deepcopy(grid)
            self.mplFigureWidget2D.canvas.GridPar.FContGridColor[varName] = copy.deepcopy(gridcolor)
            self.reDrawContours()
    
    def getGrid(self, varName=None, windowstyle=0):

        ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index(varName)
        if self.mplFigureWidget2D.canvas.GridPar.CSectionPlane==0:
            databounds = [self.mplFigureWidget2D.canvas.var[:,0,:,ivar].min(), self.mplFigureWidget2D.canvas.var[:,0,:,ivar].max()]
        if self.mplFigureWidget2D.canvas.GridPar.CSectionPlane==1:
            databounds = [self.mplFigureWidget2D.canvas.var[0,:,:,ivar].min(), self.mplFigureWidget2D.canvas.var[0,:,:,ivar].max()]
        if self.mplFigureWidget2D.canvas.GridPar.CSectionPlane==2:
            databounds = [self.mplFigureWidget2D.canvas.var[:,:,0,ivar].min(), self.mplFigureWidget2D.canvas.var[:,:,0,ivar].max()]
       
        dum = GridDialog(self, grid=self.mplFigureWidget2D.canvas.GridPar.LContGrid[varName], varName=varName, \
                databounds=databounds, gridcolor=self.mplFigureWidget2D.canvas.GridPar.LContGridColor[varName], windowstyle=windowstyle)
        dum.ApplyGrid.connect(self.updateFigure2DWithNewGrid)
        dum.exec_()
        dum.ApplyGrid.disconnect(self.updateFigure2DWithNewGrid)

    def openLGridCont1Dialog(self):
        txt = str(self.LCont1CBox.currentText()).strip()
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        for ikey in labels.keys():
            if labels[ikey]==txt:
                self.getGrid(ikey)

    def openLGridCont2Dialog(self):
        txt = str(self.LCont2CBox.currentText()).strip()
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        for ikey in labels.keys():
            if labels[ikey]==txt:
                self.getGrid(ikey)

    def openLGridCont3Dialog(self):
        txt = str(self.LCont3CBox.currentText()).strip()
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        for ikey in labels.keys():
            if labels[ikey]==txt:
                self.getGrid(ikey)
    
    def openLGridCont4Dialog(self):
        txt = str(self.LCont4CBox.currentText()).strip()
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        for ikey in labels.keys():
            if labels[ikey]==txt:
                self.getGrid(ikey)
    
    def openLGridCont5Dialog(self):
        txt = str(self.LCont5CBox.currentText()).strip()
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        for ikey in labels.keys():
            if labels[ikey]==txt:
                self.getGrid(ikey)

    def openLGridCont6Dialog(self):
        txt = str(self.LCont6CBox.currentText()).strip()
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        for ikey in labels.keys():
            if labels[ikey]==txt:
                self.getGrid(ikey)

    #def openDensLGridDialog(self):
        #varName = 'density'
        #self.getGrid(varName)

    #def openTempLGridDialog(self):
        #varName = 'temperature'
        #self.getGrid(varName)
    
    #def openVxLGridDialog(self):
        #varName = 'velocity_x'
        #self.getGrid(varName)
    #def openVyLGridDialog(self):
        #varName = 'velocity_y'
        #self.getGrid(varName)
    
    #def openVzLGridDialog(self):
        #varName = 'velocity_z'
        #self.getGrid(varName)
    
    #def openAbunLGridDialog(self):
        #varName = 'abundance'
        #self.getGrid(varName)
    
    def openFGridDialog(self):

        varName = self.mplFigureWidget2D.canvas.PlottedFContVarName 
        self.getGrid(varName, windowstyle=1)

    def plot2DChangeXYLimMouseClick(self):

        if self.Plot2DGrabBoundFigureButton.isChecked():
            unitIndex = self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID
            unit      = self.mplFigureWidget2D.canvas.CPlotPar.plotLimUnit[unitIndex]
        
            dum = self.mplFigureWidget2D.canvas.axes.get_xlim()
            self.Plot2DXrange.setText(str(dum[0])+','+str(dum[1]))
            self.mplFigureWidget2D.canvas.CPlotPar.plotXlim = dum
            self.PGridXmaxLedit.setText(str(np.abs(dum).max()))


            dum = self.mplFigureWidget2D.canvas.axes.get_ylim()
            self.Plot2DYrange.setText(str(dum[0])+','+str(dum[1]))
            self.mplFigureWidget2D.canvas.CPlotPar.plotYlim = dum
            self.PGridYmaxLedit.setText(str(np.abs(dum).max()))


            self.reDrawContours()

    def plot2DChangeAxisUnit(self):
        unitIndex     = self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID
        unit_old      = self.mplFigureWidget2D.canvas.CPlotPar.plotLimUnit[unitIndex]
        self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID = self.Plot2DUnitSelectorCBox.currentIndex()
        unitIndex     = self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID
        unit_new      = self.mplFigureWidget2D.canvas.CPlotPar.plotLimUnit[unitIndex]
    
        self.mplFigureWidget1D.canvas.currentXUnitID = self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID
    
        conv = unit_old / unit_new

        dum = self.mplFigureWidget2D.canvas.axes.get_xlim()  
        dum = [dum[0]*conv, dum[1]*conv]
        self.Plot2DXrange.setText(str(dum[0])+','+str(dum[1]))
        self.mplFigureWidget2D.canvas.CPlotPar.plotXlim = dum
        self.PGridXmaxLedit.setText(str(np.abs(dum).max()))

        dum = self.mplFigureWidget2D.canvas.axes.get_ylim() 
        dum = [dum[0]*conv, dum[1]*conv]
        self.Plot2DYrange.setText(str(dum[0])+','+str(dum[1]))
        self.mplFigureWidget2D.canvas.CPlotPar.plotYlim = dum
        self.PGridYmaxLedit.setText(str(np.abs(dum).max()))

        dum = str2float(self.CSection3rdAxisValueLedit.text(), '3rd Axis Value (Plot 2D)')
        dum = dum * conv
        self.CSection3rdAxisValueLedit.setText(str(dum))

        self.reDrawContours()

        dum = self.mplFigureWidget1D.canvas.axes.get_xlim() 
        dum = [dum[0]*conv, dum[1]*conv]
        self.mplFigureWidget1D.canvas.pxrange = dum
        self.Plot1DXrangeLedit.setText(str(dum[0])+','+str(dum[1]))

        if (self.mplFigureWidget1D.canvas.crdsys=='cartesian'):
            dum = str2float(self.Plot1Dcrd2Ledit.text(), '2nd coordinate (1D Visualize)') 
            dum = dum*conv
            self.Plot1Dcrd2Ledit.setText(str(dum))
            
            dum = str2float(self.Plot1Dcrd3Ledit.text(), '3rd coordinate (1D Visualize)') 
            dum = dum*conv
            self.Plot1Dcrd3Ledit.setText(str(dum))
        else:
            if (self.mplFigureWidget1D.canvas.crd!=1):
                dum = str2float(self.Plot1Dcrd2Ledit.text(), '2nd coordinate (1D Visualize)') 
                dum = dum*conv
                self.Plot1Dcrd2Ledit.setText(str(dum))
            

        self.readOutPgrid1D()
        self.mplFigureWidget1D.canvas.getVar1D(self.parent().configParams.unitSystem)
        self.replotYRangeFigure1D()

 
    def updatePGrid(self):
       
        self.enableDisableAxisCoupling()
        
        readPGridValsOK = self.readOutPGridWidgetValues()
        if not readPGridValsOK: return

        self.mplFigureWidget2D.canvas.updateModelVars(self )
        self.reDrawContours()
    def set2nd3rdCoordinate1D(self):


        ids  = [[1,2], [0,2], [0,1]] 
        crd = self.mplFigureWidget1D.canvas.crd
        if (self.mplFigureWidget1D.canvas.crdsys=='cartesian'):
            txt2 = str(self.mplFigureWidget1D.canvas.crdvalc[ids[crd-1][0]])
            txt3 = str(self.mplFigureWidget1D.canvas.crdvalc[ids[crd-1][1]])
            self.Plot1Dcrd2Ledit.setText(txt2)
            self.Plot1Dcrd3Ledit.setText(txt3)
        else:
            txt2 = str(self.mplFigureWidget1D.canvas.crdvals[ids[crd-1][0]])
            txt3 = str(self.mplFigureWidget1D.canvas.crdvals[ids[crd-1][1]])
            self.Plot1Dcrd2Ledit.setText(txt2)
            self.Plot1Dcrd3Ledit.setText(txt3)

    def setXrange1D(self):

        txt  = str(self.mplFigureWidget1D.canvas.pxrange[0])+','+ str(self.mplFigureWidget1D.canvas.pxrange[1])
        self.Plot1DXrangeLedit.setText(txt)
    
    def setYrange1D(self):

        txt  = str(self.mplFigureWidget1D.canvas.pyrange[0])+','+ str(self.mplFigureWidget1D.canvas.pyrange[1])
        self.Plot1DYrangeLedit.setText(txt)

    def setNpoints1D(self):

        txt  = str(self.mplFigureWidget1D.canvas.npoints)
        self.Plot1DNrofPointsLedit.setText(txt)

    def readOutPgrid1D(self):

        txt = self.Plot1DXrangeLedit.text().split(',')
        val = [str2float(txt[0], 'X range (1D Visualize)'), str2float(txt[1], 'X range (1D Visualize)')]
        self.mplFigureWidget1D.canvas.pxrange = val
        
        txt = self.Plot1DYrangeLedit.text().split(',')
        val = [str2float(txt[0], 'Y range (1D Visualize)'), str2float(txt[1], 'Y range (1D Visualize)')]
        self.mplFigureWidget1D.canvas.pyrange = val
        
        txt = self.Plot1DNrofPointsLedit.text()
        val = str2int(txt, 'Nr of points (1D Visualize)')
        self.mplFigureWidget1D.canvas.npoints = val
        self.mplFigureWidget1D.canvas.xlog    = self.Plot1DXlogCheckBox.isChecked()
        self.mplFigureWidget1D.canvas.ylog    = self.Plot1DYlogCheckBox.isChecked()

        val2 = str2float(self.Plot1Dcrd2Ledit.text(), '2nd coordinate (1D Visualize)')
        val3 = str2float(self.Plot1Dcrd3Ledit.text(), '3rd coordinate (1D Visualize)')

        ids  = [[1,2], [0,2], [0,1]] 

        crd = self.mplFigureWidget1D.canvas.crd
        if (self.mplFigureWidget1D.canvas.crdsys=='cartesian'):
            self.mplFigureWidget1D.canvas.crdvalc[ids[crd-1][0]] = val2
            self.mplFigureWidget1D.canvas.crdvalc[ids[crd-1][1]] = val3
        else:
            self.mplFigureWidget1D.canvas.crdvals[ids[crd-1][0]] = val2
            self.mplFigureWidget1D.canvas.crdvals[ids[crd-1][1]] = val3


    def replotScaleFigure1D(self):
        self.mplFigureWidget1D.canvas.xlog    = self.Plot1DXlogCheckBox.isChecked()
        self.mplFigureWidget1D.canvas.ylog    = self.Plot1DYlogCheckBox.isChecked()

        Rmin    = str2float(self.parent().LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
        Rmax    = str2float(self.parent().LRTRadiusLedit.text(), 'radius (Code Parameters / Grid)') * 1.496e11
        self.mplFigureWidget1D.canvas.getVar1D(self.parent().configParams.unitSystem)
        self.mplFigureWidget1D.canvas.plotVar1D(self.parent().configParams.unitSystem, Rmin=Rmin, Rmax=Rmax)

    def replotXRangeFigure1D(self):
        
        self.readOutPgrid1D()
        conv = 1.496e11 / self.mplFigureWidget1D.canvas.plotXUnit[self.mplFigureWidget1D.canvas.currentXUnitID]
        minScale = str2float(self.parent().LRTminScaleLedit.text(), 'minScale (Code parameters / Grid)') * conv
        radius   = str2float(self.parent().LRTRadiusLedit.text(), 'Radius (Code parameters / Grid)') * conv
        self.mplFigureWidget1D.canvas.pxrange = [minScale, radius]
        self.setXrange1D()
        Rmin    = str2float(self.parent().LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
        Rmax    = str2float(self.parent().LRTRadiusLedit.text(), 'radius (Code Parameters / Grid)') * 1.496e11
        self.mplFigureWidget1D.canvas.plotVar1D(self.parent().configParams.unitSystem, Rmin=Rmin, Rmax=Rmax)

    def replotYRangeFigure1D(self):
        
        self.readOutPgrid1D()
        self.mplFigureWidget1D.canvas.pyrange = self.mplFigureWidget1D.canvas.varRange
        self.setYrange1D()
        Rmin    = str2float(self.parent().LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
        Rmax    = str2float(self.parent().LRTRadiusLedit.text(), 'radius (Code Parameters / Grid)') * 1.496e11
        self.mplFigureWidget1D.canvas.plotVar1D(self.parent().configParams.unitSystem, Rmin=Rmin, Rmax=Rmax)

    def updateFigure1DNewYRange(self):
    

        if (self.Plot1Dcrd1RButton.isChecked()): self.mplFigureWidget1D.canvas.crd = 1
        if (self.Plot1Dcrd2RButton.isChecked()): self.mplFigureWidget1D.canvas.crd = 2
        if (self.Plot1Dcrd3RButton.isChecked()): self.mplFigureWidget1D.canvas.crd = 3
        self.readOutPgrid1D()
        if (self.Plot1DcrdCartesianRButton.isChecked()):
            self.Plot1Dcrd1RButton.setText('x')
            self.Plot1Dcrd2RButton.setText('y')
            self.Plot1Dcrd3RButton.setText('z')
            self.mplFigureWidget1D.canvas.crdsys = 'cartesian'

            if (self.Plot1Dcrd1RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('y crd:')
                self.Plot1Dcrd3Label.setText('z crd:')
                

            if (self.Plot1Dcrd2RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('x crd:')
                self.Plot1Dcrd3Label.setText('z crd:')
           
            if (self.Plot1Dcrd3RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('x crd:')
                self.Plot1Dcrd3Label.setText('y crd:')


        if (self.Plot1DcrdSphericalRButton.isChecked()):
            self.Plot1Dcrd1RButton.setText('r')
            self.Plot1Dcrd2RButton.setText('phi')
            self.Plot1Dcrd3RButton.setText('theta')
            self.mplFigureWidget1D.canvas.crdsys = 'spherical'

            if (self.Plot1Dcrd1RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('phi crd:')
                self.Plot1Dcrd3Label.setText('theta crd:')

            if (self.Plot1Dcrd2RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('r crd:')
                self.Plot1Dcrd3Label.setText('theta crd:')
           
            if (self.Plot1Dcrd3RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('r crd:')
                self.Plot1Dcrd3Label.setText('phi crd:')
        


        txt = str(self.Plot1DVarSelectorCBox.currentText())
        for ikey in self.mplFigureWidget1D.canvas.AllVarLabel.keys(): 
            if txt==self.mplFigureWidget1D.canvas.AllVarLabel[ikey]:
                self.mplFigureWidget1D.canvas.PlottedVarName = ikey
        
        self.mplFigureWidget1D.canvas.getVar1D(self.parent().configParams.unitSystem)
        self.mplFigureWidget1D.canvas.pyrange = self.mplFigureWidget1D.canvas.varRange
        Rmin    = str2float(self.parent().LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
        Rmax    = str2float(self.parent().LRTRadiusLedit.text(), 'radius (Code Parameters / Grid)') * 1.496e11
        self.mplFigureWidget1D.canvas.plotVar1D(self.parent().configParams.unitSystem, Rmin=Rmin, Rmax=Rmax)
        self.setYrange1D()
    def updateFigure1D(self):
    
        self.readOutPgrid1D()
        txt = str(self.Plot1DVarSelectorCBox.currentText())
        for ikey in self.mplFigureWidget1D.canvas.AllVarLabel.keys():
            if txt==self.mplFigureWidget1D.canvas.AllVarLabel[ikey]:
                self.mplFigureWidget1D.canvas.PlottedVarName = ikey
                
        crdsys = ''
        crd    = 0


        if (self.Plot1DcrdCartesianRButton.isChecked()):
            self.Plot1Dcrd1RButton.setText('x')
            self.Plot1Dcrd2RButton.setText('y')
            self.Plot1Dcrd3RButton.setText('z')
            self.mplFigureWidget1D.canvas.crdsys = 'cartesian'

            if (self.Plot1Dcrd1RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('y crd:')
                self.Plot1Dcrd3Label.setText('z crd:')
                self.mplFigureWidget1D.canvas.crd = 1
                

            if (self.Plot1Dcrd2RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('x crd:')
                self.Plot1Dcrd3Label.setText('z crd:')
                self.mplFigureWidget1D.canvas.crd = 2
           
            if (self.Plot1Dcrd3RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('x crd:')
                self.Plot1Dcrd3Label.setText('y crd:')
                self.mplFigureWidget1D.canvas.crd = 3


        if (self.Plot1DcrdSphericalRButton.isChecked()):
            self.Plot1Dcrd1RButton.setText('r')
            self.Plot1Dcrd2RButton.setText('phi')
            self.Plot1Dcrd3RButton.setText('theta')
            self.mplFigureWidget1D.canvas.crdsys = 'spherical'

            if (self.Plot1Dcrd1RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('phi crd:')
                self.Plot1Dcrd3Label.setText('theta crd:')
                self.mplFigureWidget1D.canvas.crd = 1

            if (self.Plot1Dcrd2RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('r crd:')
                self.Plot1Dcrd3Label.setText('theta crd:')
                self.mplFigureWidget1D.canvas.crd = 2
           
            if (self.Plot1Dcrd3RButton.isChecked()):
                self.Plot1Dcrd2Label.setText('r crd:')
                self.Plot1Dcrd3Label.setText('phi crd:')
                self.mplFigureWidget1D.canvas.crd = 3
        
        
        self.mplFigureWidget1D.canvas.getVar1D(self.parent().configParams.unitSystem)
        Rmin    = str2float(self.parent().LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
        Rmax    = str2float(self.parent().LRTRadiusLedit.text(), 'radius (Code Parameters / Grid)') * 1.496e11
        self.mplFigureWidget1D.canvas.plotVar1D(self.parent().configParams.unitSystem, Rmin=Rmin, Rmax=Rmax)

    
    def changeCrossSection(self):

        readCrossSectionVals = self.readOutCrossSectionValues()
        if not readCrossSectionVals: return

        readPGridValsOK = self.readOutPGridWidgetValues()
        if not readPGridValsOK: return

        self.mplFigureWidget2D.canvas.updateModelVars(self,defaultPar=False)
        self.reDrawContours()

    def enableDisableAxisCoupling(self):
        
        if self.PGridYCoupleToXCheckBox.isChecked():
            self.mplFigureWidget2D.canvas.GridPar.ycouple2x=True
            self.PGridNYLedit.setVisible(False)
            self.PGridYminLedit.setVisible(False)
            self.PGridYmaxLedit.setVisible(False)
            self.PGridNYLabel.setVisible(False)
            self.PGridYminLabel.setVisible(False)
            self.PGridYmaxLabel.setVisible(False)
        else:
            self.mplFigureWidget2D.canvas.GridPar.ycouple2x=False
            self.PGridNYLedit.setVisible(True)
            self.PGridYminLedit.setVisible(True)
            self.PGridYmaxLedit.setVisible(True)
            self.PGridNYLabel.setVisible(True)
            self.PGridYminLabel.setVisible(True)
            self.PGridYmaxLabel.setVisible(True)

        if self.PGridZCoupleToXCheckBox.isChecked():
            self.mplFigureWidget2D.canvas.GridPar.zcouple2x=True
            self.PGridNZLedit.setVisible(False)
            self.PGridZminLedit.setVisible(False)
            self.PGridZmaxLedit.setVisible(False)
            self.PGridNZLabel.setVisible(False)
            self.PGridZminLabel.setVisible(False)
            self.PGridZmaxLabel.setVisible(False)
        else:
            self.mplFigureWidget2D.canvas.GridPar.zcouple2x=False
            self.PGridNZLedit.setVisible(True)
            self.PGridZminLedit.setVisible(True)
            self.PGridZmaxLedit.setVisible(True)
            self.PGridNZLabel.setVisible(True)
            self.PGridZminLabel.setVisible(True)
            self.PGridZmaxLabel.setVisible(True)



    def readOutPGridWidgetValues(self):

        unitIndex = self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID
        unit      = self.mplFigureWidget2D.canvas.CPlotPar.plotLimUnit[unitIndex]
        
        txt = self.PGridNXLedit.text()
        dum = str2int(txt, 'Nx')
        if dum==None: return False

        if (dum*2!=self.mplFigureWidget2D.canvas.GridPar.nx):
            self.mplFigureWidget2D.canvas.GridPar.nx = dum*2
            self.mplFigureWidget2D.canvas.GridPar.gridChanged = True
        if (self.mplFigureWidget2D.canvas.GridPar.CSectionPlane==1):
            self.mplFigureWidget2D.canvas.GridPar.nx = 1

        txt = self.PGridXminLedit.text()
        dum = str2float(txt, 'Xmin')
        if (dum!=self.mplFigureWidget2D.canvas.GridPar.xmin/unit):
            dum = str2float(txt, 'Xmin')
            if dum==None: return False

            #print '*************'
            #print dum, self.mplFigureWidget2D.canvas.GridPar.xmin/unit
            self.mplFigureWidget2D.canvas.GridPar.xmin = dum*unit
            self.mplFigureWidget2D.canvas.GridPar.gridChanged = True
        
        txt = self.PGridXmaxLedit.text()
        dum = str2float(txt, 'Xmax')
        if (dum!=self.mplFigureWidget2D.canvas.GridPar.xmax/unit):
            dum = str2float(txt, 'Xmax')
            if dum==None: return False
            self.mplFigureWidget2D.canvas.GridPar.xmax = dum*unit
            self.mplFigureWidget2D.canvas.GridPar.gridChanged = True



        if (self.mplFigureWidget2D.canvas.GridPar.ycouple2x==True):
            txt = self.PGridNXLedit.text()
            dum = str2int(txt, 'Nx')
            if dum==None: return False
            self.mplFigureWidget2D.canvas.GridPar.ny = dum*2
            
            txt = self.PGridXminLedit.text()
            dum = str2float(txt, 'Xmin')
            if dum==None: return False
            self.mplFigureWidget2D.canvas.GridPar.ymin = dum*unit
            
            txt = self.PGridXmaxLedit.text()
            dum = str2float(txt, 'Xmax')
            if dum==None: return False
            self.mplFigureWidget2D.canvas.GridPar.ymax = dum*unit
            
        else:
            txt = self.PGridNYLedit.text()
            dum = str2int(txt, 'Ny')
            if dum==None: return False

            if (dum*2!=self.mplFigureWidget2D.canvas.GridPar.ny):
                self.mplFigureWidget2D.canvas.GridPar.ny = dum*2.
                self.mplFigureWidget2D.canvas.GridPar.gridChanged = True                

            txt = self.PGridYminLedit.text()
            dum = str2float(txt, 'Ymin')
            if dum==None: return False

            if (dum!=self.mplFigureWidget2D.canvas.GridPar.ymin/unit):
                self.mplFigureWidget2D.canvas.GridPar.ymin = dum*unit
                self.mplFigureWidget2D.canvas.GridPar.gridChanged = True

            txt = self.PGridYmaxLedit.text()
            dum = str2float(txt, 'Ymax')
            if dum==None: return False

            if (dum!=self.mplFigureWidget2D.canvas.GridPar.ymax/unit):
                self.mplFigureWidget2D.canvas.GridPar.ymax = dum*unit
                self.mplFigureWidget2D.canvas.GridPar.gridChanged = True
        
        if (self.mplFigureWidget2D.canvas.GridPar.CSectionPlane==0):
            self.mplFigureWidget2D.canvas.GridPar.ny = 1




        if (self.mplFigureWidget2D.canvas.GridPar.zcouple2x==True):
            
            txt = self.PGridNXLedit.text()
            dum = str2int(txt, 'Nx')
            if dum==None: return False
            self.mplFigureWidget2D.canvas.GridPar.nz = dum*2

            txt = self.PGridXminLedit.text()
            dum = str2float(txt, 'Ymin')
            if dum==None: return False
            self.mplFigureWidget2D.canvas.GridPar.zmin = dum*unit
            
            txt = self.PGridXmaxLedit.text()
            dum = str2float(txt, 'Ymax')
            if dum==None: return False
            self.mplFigureWidget2D.canvas.GridPar.zmax = dum*unit
            
        else:
            txt = self.PGridNZLedit.text()
            dum = str2int(txt, 'Nz')
            if dum==None: return False

            if (dum*2!=self.mplFigureWidget2D.canvas.GridPar.nz):
                self.mplFigureWidget2D.canvas.GridPar.nz = dum*2.
                self.mplFigureWidget2D.canvas.GridPar.gridChanged = True

            txt = self.PGridZminLedit.text()
            dum = str2float(txt, 'Zmin')
            if dum==None: return False
            if (dum!=self.mplFigureWidget2D.canvas.GridPar.zmin/unit):
                self.mplFigureWidget2D.canvas.GridPar.zmin = dum*unit
                self.mplFigureWidget2D.canvas.GridPar.gridChanged = True

            txt = self.PGridZmaxLedit.text()
            dum = str2float(txt, 'Ymax')
            if dum==None: return False
            if (dum!=self.mplFigureWidget2D.canvas.GridPar.zmax/unit):
                self.mplFigureWidget2D.canvas.GridPar.zmax = dum*unit
                self.mplFigureWidget2D.canvas.GridPar.gridChanged = True
                               
        if (self.mplFigureWidget2D.canvas.GridPar.CSectionPlane==2):
            self.mplFigureWidget2D.canvas.GridPar.nz = 1



        readContourLimitsOK = self.readOutContourLimits()
        if not readContourLimitsOK: 
            return False
        else: 
            return True

    def setPGridWidgetValues(self):
        
        unitIndex = self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID
        unit      = self.mplFigureWidget2D.canvas.CPlotPar.plotLimUnit[unitIndex]

        self.PGridNXLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.nx/2))
        self.PGridXminLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.xmin/unit))
        self.PGridXmaxLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.xmax/unit))

        self.PGridNYLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.ny/2))
        self.PGridYminLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.ymin/unit))
        self.PGridYmaxLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.ymax/unit))


        self.PGridNZLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.nz/2))
        self.PGridZminLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.zmin/unit))
        self.PGridZmaxLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.zmax/unit))

    def readOutCrossSectionValues(self):


        unitIndex = self.mplFigureWidget2D.canvas.CPlotPar.currentLimUnitID
        unit      = self.mplFigureWidget2D.canvas.CPlotPar.plotLimUnit[unitIndex]

        txt = self.CSection3rdAxisValueLedit.text()
        dum = str2float(txt, '3rd Axis Value (Plot 2D)')
        if dum==None: return False
        if (dum!=self.mplFigureWidget2D.canvas.GridPar.CSection3rdAxisValue/unit):
            self.mplFigureWidget2D.canvas.GridPar.gridChanged = True
            self.mplFigureWidget2D.canvas.GridPar.CSection3rdAxisValue = dum*unit

        if (self.CSectionXZRButton.isChecked()==True):
            self.mplFigureWidget2D.canvas.GridPar.CSectionPlane = 0
            self.mplFigureWidget2D.canvas.GridPar.xval = 0.0
            self.mplFigureWidget2D.canvas.GridPar.yval = self.mplFigureWidget2D.canvas.GridPar.CSection3rdAxisValue
            self.mplFigureWidget2D.canvas.GridPar.zval = 0.0
            if self.parent().showToolTipHelp:
                self.CSection3rdAxisValueLedit.setToolTip(str2ToolTipString('Set the value of the Y coordinate'))

        if (self.CSectionYZRButton.isChecked()==True):
            self.mplFigureWidget2D.canvas.GridPar.CSectionPlane = 1
            self.mplFigureWidget2D.canvas.GridPar.xval = self.mplFigureWidget2D.canvas.GridPar.CSection3rdAxisValue
            self.mplFigureWidget2D.canvas.GridPar.yval = 0.0
            self.mplFigureWidget2D.canvas.GridPar.zval = 0.0
            if self.parent().showToolTipHelp:
                self.CSection3rdAxisValueLedit.setToolTip(str2ToolTipString('Set the value of the X coordinate'))

        if (self.CSectionXYRButton.isChecked()==True):
            self.mplFigureWidget2D.canvas.GridPar.CSectionPlane = 2
            self.mplFigureWidget2D.canvas.GridPar.xval = 0.0
            self.mplFigureWidget2D.canvas.GridPar.yval = 0.0
            self.mplFigureWidget2D.canvas.GridPar.zval = self.mplFigureWidget2D.canvas.GridPar.CSection3rdAxisValue
            if self.parent().showToolTipHelp:
                self.CSection3rdAxisValueLedit.setToolTip(str2ToolTipString('Set the value of the Z coordinate'))

        return True
    def setCrossSectionValues(self):
        
        if (self.mplFigureWidget2D.canvas.GridPar.CSectionPlane == 0):
            self.CSectionXZRButton.setChecked(True)
        if (self.mplFigureWidget2D.canvas.GridPar.CSectionPlane == 1):
            self.CSectionYZRButton.setChecked(True)
        if (self.mplFigureWidget2D.canvas.GridPar.CSectionPlane == 2):
            self.CSectionXYRButton.setChecked(True)

        self.CSection3rdAxisValueLedit.setText(str(self.mplFigureWidget2D.canvas.GridPar.CSection3rdAxisValue))



    #def resetLContGridParams(self):

        #ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index('density')
        #txt = str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].min())+', '+ str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].max())
        #self.LContDensLedit.setText(txt)
        
        #ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index('temperature')
        #txt = str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].min())+', '+ str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].max())
        #self.LContTempLedit.setText(txt)

        #ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index('velocity_x')
        #txt = str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].min())+', '+ str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].max())
        #self.LContVxLedit.setText(txt)

        #ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index('velocity_y')
        #txt = str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].min())+', '+ str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].max())
        #self.LContVyLedit.setText(txt)

        #ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index('velocity_z')
        #txt = str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].min())+', '+ str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].max())
        #self.LContVzLedit.setText(txt)

        #ivar = self.mplFigureWidget2D.canvas.PlottedLContVarName.index('abundance')
        #txt = str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].min())+', '+ str(self.mplFigureWidget2D.canvas.var[:,:,:,ivar].max())
        #self.LContAbunLedit.setText(txt)



    #def setLContGridParams(self):
       
        #print '????'


        #txt = str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['density'].minval)+', '+\
                #str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['density'].maxval)
        #self.LContDensLedit.setText(txt)
        
        #txt = str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['temperature'].minval)+', '+\
                #str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['temperature'].maxval)
        #self.LContTempLedit.setText(txt)
        
        #txt = str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['velocity_x'].minval)+', '+\
                #str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['velocity_x'].maxval)
        #self.LContVxLedit.setText(txt)
        
        #txt = str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['velocity_y'].minval)+', '+\
                #str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['velocity_y'].maxval)
        #self.LContVyLedit.setText(txt)
        
        #txt = str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['velocity_z'].minval)+', '+\
                #str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['velocity_z'].maxval)
        #self.LContVzLedit.setText(txt)
        
        #txt = str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['abundance'].minval)+', '+\
                #str(self.mplFigureWidget2D.canvas.GridPar.LContGrid['abundance'].maxval)
        #self.LContAbunLedit.setText(txt)
        
    def readOutContourLimits(self):

        self.mplFigureWidget2D.canvas.GridPar.xylimfromimage = self.Plot2DGrabBoundFigureButton.isChecked()

        if (self.mplFigureWidget2D.canvas.GridPar.xylimfromimage==True):
            return True
        else:

            txt = self.Plot2DXrange.text().split(',')
            dum1 = str2float(txt[0], 'X Range') 
            if dum1==None: return False
            dum2 = str2float(txt[1], 'X Range') 
            if dum2==None: return False
            self.mplFigureWidget2D.canvas.CPlotPar.plotXlim = [dum1, dum2]

            txt = self.Plot2DYrange.text().split(',')
            dum1 = str2float(txt[0], 'Y Range') 
            if dum1==None: return False
            dum2 = str2float(txt[1], 'Y Range') 
            if dum2==None: return False
            self.mplFigureWidget2D.canvas.CPlotPar.plotYlim = [dum1, dum2]
            
            return True

    def drawFilledContours(self,modelParamID):
        
        self.mplFigureWidget2D.canvas.PlottedModelID = self.CurrentModelID
        txt = str(self.FContComboBox.currentText()).strip()
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        for ikey in labels.keys():
            if labels[ikey]==txt:
                self.mplFigureWidget2D.canvas.PlottedFContVarName = ikey
        
        self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas,UnitSystem=self.parent().configParams.unitSystem)
        self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)


        #if (modelParamID==0):
            #self.mplFigureWidget2D.canvas.PlottedFContVarName = 'density'

            #self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas,UnitSystem=self.parent().configParams.unitSystem)
            #self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)
        #if (modelParamID==1):
            #self.mplFigureWidget2D.canvas.PlottedFContVarName = 'temperature'

            #self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas,UnitSystem=self.parent().configParams.unitSystem)
            #self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)
        #if (modelParamID==2):
            #self.mplFigureWidget2D.canvas.PlottedFContVarName = 'velocity_x'

            #self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas,UnitSystem=self.parent().configParams.unitSystem)
            #self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)
        #if (modelParamID==3):
            #self.mplFigureWidget2D.canvas.PlottedFContVarName = 'velocity_y'

            #self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas,UnitSystem=self.parent().configParams.unitSystem)
            #self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)
        #if (modelParamID==4):
            #self.mplFigureWidget2D.canvas.PlottedFContVarName = 'velocity_z'

            #self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas,UnitSystem=self.parent().configParams.unitSystem)
            #self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)
        #if (modelParamID==5):
            #self.mplFigureWidget2D.canvas.PlottedFContVarName = 'abundance'

            #self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas,UnitSystem=self.parent().configParams.unitSystem)
            #self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)


    def drawContourLines(self):

        self.mplFigureWidget2D.canvas.PlottedModelID = self.CurrentModelID
        labels = self.mplFigureWidget2D.canvas.PlottedFContVarLabel
        checked_vars = []

        for ikey in labels.keys():
            self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = False


        if (self.LCont1Button.checkState()==2):
            txt = str(self.LCont1CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = True          
                    checked_vars.append(ikey)
        else:
            txt = str(self.LCont1CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = False
                    checked_vars.append(ikey)

        if (self.LCont2Button.checkState()==2):
            txt = str(self.LCont2CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = True           
                        checked_vars.append(ikey)
        else:
            txt = str(self.LCont2CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = False
                        checked_vars.append(ikey)
        

        if (self.LCont3Button.checkState()==2):
            txt = str(self.LCont3CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = True           
                        checked_vars.append(ikey)
        else:
            txt = str(self.LCont3CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = False
                        checked_vars.append(ikey)

        if (self.LCont4Button.checkState()==2):
            txt = str(self.LCont4CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = True           
                        checked_vars.append(ikey)
        else:
            txt = str(self.LCont4CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = False
                        checked_vars.append(ikey)

        if (self.LCont5Button.checkState()==2):
            txt = str(self.LCont5CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = True           
                        checked_vars.append(ikey)
        else:
            txt = str(self.LCont5CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = False
                        checked_vars.append(ikey)

        if (self.LCont6Button.checkState()==2):
            txt = str(self.LCont6CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = True           
                        checked_vars.append(ikey)
        else:
            txt = str(self.LCont6CBox.currentText()).strip()
            for ikey in labels.keys():
                if labels[ikey]==txt:
                    if not checked_vars.__contains__(ikey):
                        self.mplFigureWidget2D.canvas.PlottedLContVar[ikey] = False
                        checked_vars.append(ikey)

        #print self.mplFigureWidget2D.canvas.PlottedLContVar

        #if (self.LCont1Button.checkState()==2):
            #self.mplFigureWidget2D.canvas.PlottedLContVar[0] = True
        #else:
            #self.mplFigureWidget2D.canvas.PlottedLContVar[0] = False
        
        #if (self.LCont2Button.checkState()==2):
            #self.mplFigureWidget2D.canvas.PlottedLContVar[1] = True
        #else:
            #self.mplFigureWidget2D.canvas.PlottedLContVar[1] = False

        #if (self.LCont3Button.checkState()==2):
            #self.mplFigureWidget2D.canvas.PlottedLContVar[2] = True
        #else:
            #self.mplFigureWidget2D.canvas.PlottedLContVar[2] = False
            
        #if (self.LCont4Button.checkState()==2):
            #self.mplFigureWidget2D.canvas.PlottedLContVar[3] = True
        #else:
            #self.mplFigureWidget2D.canvas.PlottedLContVar[3] = False

        #if (self.LCont5Button.checkState()==2):
            #self.mplFigureWidget2D.canvas.PlottedLContVar[4] = True
        #else:
            #self.mplFigureWidget2D.canvas.PlottedLContVar[4] = False

        #if (self.LCont6Button.checkState()==2):
            #self.mplFigureWidget2D.canvas.PlottedLContVar[5] = True
        #else:
            #self.mplFigureWidget2D.canvas.PlottedLContVar[5] = False
        
        #self.resetLContGridParams()
        
        self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas, UnitSystem=self.parent().configParams.unitSystem)
        self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)

    
    def reDrawContoursVarUpdate(self):
        
        self.mplFigureWidget2D.canvas.updateModelVars(self,defaultPar=False)
        #self.resetLContGridParams()

    def forceUpdateAllPlots(self):
        
        self.reDrawContoursVarUpdate()
        self.reDrawContours()
        self.updateFigure1DNewYRange()


    def reDrawContours(self):

        readPGridValsOK = self.readOutPGridWidgetValues()
        if not readPGridValsOK: return

        xlim = self.mplFigureWidget2D.canvas.CPlotPar.plotXlim
        ylim = self.mplFigureWidget2D.canvas.CPlotPar.plotYlim


        if (self.PGridYCoupleToXCheckBox.isChecked()==True):
            self.PGridYmaxLedit.setText(str(xlim[1]))

        if (self.PGridZCoupleToXCheckBox.isChecked()==True):
            self.PGridZmaxLedit.setText(str(xlim[1]))

        self.readOutPGridWidgetValues()
        readCrossSectionValsOK = self.readOutCrossSectionValues()
        if not readCrossSectionValsOK: return

        if (self.mplFigureWidget2D.canvas.GridPar.gridChanged == True):
            self.mplFigureWidget2D.canvas.updateModelVars(self)
            #self.resetLContGridParams()

            self.mplFigureWidget2D.canvas.GridPar.gridChanged = False
             
        
        dum = self.mplFigureWidget2D.canvas.GridPar.xmax
        self.mplFigureWidget2D.canvas.CPlotPar.plotxlim = [dum*(-1.), dum]
        dum = self.mplFigureWidget2D.canvas.GridPar.zmax
        self.mplFigureWidget2D.canvas.CPlotPar.plotylim = [dum*(-1.), dum]

        self.mplFigureWidget2D.canvas.GridPar.xmax = self.mplFigureWidget2D.canvas.GridPar.xmax
        self.mplFigureWidget2D.canvas.GridPar.xmin = self.mplFigureWidget2D.canvas.GridPar.xmax*(-1.)
        self.mplFigureWidget2D.canvas.GridPar.zmax = self.mplFigureWidget2D.canvas.GridPar.zmax
        self.mplFigureWidget2D.canvas.GridPar.zmin = self.mplFigureWidget2D.canvas.GridPar.zmax*(-1.)

        readContourLimitsOK = self.readOutContourLimits()
        if not readContourLimitsOK: return

        self.mplFigureWidget2D.canvas.plotModelVariableFilledContours(self.mplFigureWidget2D.canvas, UnitSystem=self.parent().configParams.unitSystem)
        self.mplFigureWidget2D.canvas.plotModelVariableContourLines(self.mplFigureWidget2D.canvas)


class startupDialog(QtGui.QDialog, Ui_StartupDialog):

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self)
        self.setupUi(self)
        self.show()
        self.setFocus()
        self.raise_()
        self.connect(self.OpenModelButton, QtCore.SIGNAL("clicked()"), self.accept)
        self.connect(self.CreateModelButton, QtCore.SIGNAL("clicked()"), self.reject)
        self.connect(self.ExitButton, QtCore.SIGNAL("clicked()"), self.exitARTIST)

    def exitARTIST(self):
        sys.exit(0)


class DustOpacityPlottingWindow(QtGui.QMainWindow, Ui_mplFigurePixSpecMainWindow):

    def __init__(self, parent = None):
        super(DustOpacityPlottingWindow, self).__init__(parent)
        self.setupUi(self)

        self.resize(800,600)
        self.setWindowTitle('Dust opacity')
        self.connect(self.PixSpecCloseButton,QtCore.SIGNAL("clicked()"), self.closeWindow)


    def closeWindow(self):
        self.setVisible(False)



class MainWindow(QtGui.QMainWindow, Ui_MainWindow):

    EnumIndexChanged = QtCore.pyqtSignal(name='EnumIndexChanged')

    def __init__(self, parent = None, app=None):
        super(MainWindow, self).__init__(parent)
        self.setupUi(self)
   
        self.setWindowTitle('ARTIST GUI V1.1 Beta')

        # Turn off polarization (as it is not yet implemented)
        self.LRTpolarizationCBox.setVisible(False)


        self.showToolTipHelp = False
        self.nc  = natconst()
        self.par = lime.createInputPars()
        self.AU  = 1.496e11
        self.PC  = 3.0857200e+16 

        self.modelDir        = '.'

        self.par.radius     = 2000.*self.nc.au/100.
        self.par.minScale   = 1.0*self.nc.au/100.
        self.par.phi        = 0.0
        self.par.distance   = 140.*self.nc.pc/100
        self.par.tcmb       = 2.725
        self.par.pIntensity = 30
        self.par.sinkPoints = 30

        self.par.dust        = " "
        self.par.moldatfile	= self.modelDir+"/hco+.dat"
        self.par.outputfile	= self.modelDir+"/outfile.pop"
        self.par.gridfile 	= self.modelDir+"/grid.vtk"
        self.par.sampling	= 0


        self.images = []

    # Image 
        
        self.img = lime.createImage()	
        self.img.pxls       	= 101
        self.img.imgres     	= 0.1

        self.img.theta	    = 0. 
        self.img.distance	= 140 * self.nc.pc/100.
        self.img.unit       	= 0
        self.img.filename   	= self.modelDir+"/image0.fits"

        self.img.phi	       	= 0.0		
        self.img.source_vel	= 0.0
        self.img.nchan      	= 60
        self.img.velres     	= 500.
        self.img.trans      	= 2
            

        self.functionSetAbun     = False
        self.functionSetDens     = False
        self.functionSetDoppler  = False
        self.functionSetDustTemp = False
        self.functionSetMagfield = False 
        self.functionSetTemp     = False
        self.functionSetVelo     = False

        self.DustOpacityWindow = DustOpacityPlottingWindow(self)
        
        
        self.app=app

        self.RegionAbunParams = AbundanceModel()

        self.dialogDirectory = './'

        self.moldata=[]
        self.copyDataFilesToModelDir = True

        self.MImageParGrid = {}
        self.MImageParGrid['incl'] = Grid(minval=0., maxval=90., nval=10, gridstyle='lin') 
        self.MImageParGrid['imgres'] = Grid(minval=0.01, maxval=0.1, nval=10, gridstyle='lin') 
        self.MImageParGrid['velres'] = Grid(minval=100., maxval=1000., nval=10, gridstyle='lin') 

        self.MImagePar = {'incl': {'enabled':False, 'style':0,'aval':[], 'grid': copy.deepcopy(self.MImageParGrid['incl'])},\
                        'imgres': {'enabled':False, 'style':0,'aval':[], 'grid': copy.deepcopy(self.MImageParGrid['imgres'])},\
                        'velres': {'enabled':False, 'style':0,'aval':[], 'grid': copy.deepcopy(self.MImageParGrid['velres'])},\
                        'trans': {'enabled':False, 'style':0,'aval':[]}}

        dum = [("%f"%i) for i in self.MImageParGrid['incl'].val]
        self.ImGridinclRGridValCBox.clear()
        self.ImGridinclRGridValCBox.addItems(dum)

        dum = [("%f"%i) for i in self.MImageParGrid['imgres'].val]
        self.ImGridimgresRGridValCBox.clear()
        self.ImGridimgresRGridValCBox.addItems(dum)

        dum = [("%f"%i) for i in self.MImageParGrid['velres'].val]
        self.ImGridvelresRGridValCBox.clear()
        self.ImGridvelresRGridValCBox.addItems(dum)
        
       
        self.inOpeningModel = False
        self.doFinalizeConfig = True

        self.actionNew_Model.setShortcut('Ctrl+N')
        self.actionOpen_Model.setShortcut('Ctrl+O')
        self.actionSave_Model.setShortcut('Ctrl+S')
        self.actionSave_as.setShortcut('Shift+Ctrl+S')
        self.actionCustomize.setShortcut('Ctrl+P')
        self.actionQuitARTIST.setShortcut('Ctrl+Q')
        
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+right"), self, self.moveToNextTab)
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+left"), self, self.moveToPreviousTab)

        self.connect(self.action_About_ARTIST, QtCore.SIGNAL("triggered()"), self.openAboutDialog)
        self.connect(self.actionNew_Model, QtCore.SIGNAL("triggered()"), self.createNewModel)
        self.connect(self.actionOpen_Model, QtCore.SIGNAL("triggered()"), self.openModelDir)
        self.connect(self.actionSave_Model, QtCore.SIGNAL("triggered()"), self.saveModel)
        self.connect(self.actionSave_as, QtCore.SIGNAL("triggered()"), self.saveModelas)
        self.connect(self.actionCustomize, QtCore.SIGNAL("triggered(bool)"), self.openCustomizeDialog)
        self.connect(self.actionQuitARTIST, QtCore.SIGNAL("triggered(bool)"), sys.exit)
        self.connect(self.actionShow_tooltip_help, QtCore.SIGNAL("triggered(bool)"), self.setAllToolTips)
        self.connect(self.actionHide_tooltip_help, QtCore.SIGNAL("triggered(bool)"), self.hideAllToolTips)



        self.connect(self.AbunMoldatFileBrowseButton, QtCore.SIGNAL("clicked()"), self.openMoldatFile)
        self.connect(self.TdustDustopacFileBrowseButton, QtCore.SIGNAL("clicked()"), self.openDustpathFile)
        self.connect(self.TdustDustopacPlotButton,QtCore.SIGNAL("clicked()"), self.enableDustOpacityWindow)
        

        self.connect(self.ModellibBrowserButton, QtCore.SIGNAL("clicked()"), self.browseModellib)
        

        
        self.connect(self.ModelPar1Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar2Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar3Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar4Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar5Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar6Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar7Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar8Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar9Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.ModelPar10Ledit, QtCore.SIGNAL("returnPressed()"), self.changeParams)

        self.connect(self.ModelPar1CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar2CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar3CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar4CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar5CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar6CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar7CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar8CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar9CBox, QtCore.SIGNAL("activated()"), self.changeParams)
        self.connect(self.ModelPar10CBox, QtCore.SIGNAL("activated()"), self.changeParams)

        self.connect(self.DensInpTypeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setLabelsDensField)
        self.connect(self.TempInpTypeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setLabelsTempField)
        self.connect(self.AbunInpTypeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setLabelsAbunField)
        self.connect(self.VeloInpTypeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setLabelsVeloField)
        self.connect(self.DopplerInpTypeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setLabelsDopplerField)
        self.connect(self.MagfieldInpTypeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setLabelsMagfield)
        self.connect(self.TdustInpTypeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setLabelsDustTempField)

        self.connect(self.AbunRegionCustomizeButton, QtCore.SIGNAL("clicked()"), self.openAbundanceCustomDialog)

        self.connect(self.DensFNameLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.DensValueLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.TempFNameLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.TempValueLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.AbunFTempLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.AbunValueLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.VeloFNameLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.VeloValueLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.DopplerFNameLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.DopplerValueLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.MagfieldFNameLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.MagfieldValueLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.TdustFNameLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)
        self.connect(self.TdustValueLedit, QtCore.SIGNAL("returnPressed()"), self.changeParams)

        self.connect(self.AbunFNameBrowseButton, QtCore.SIGNAL("clicked()"), self.importAbunTable)
        self.connect(self.DensFNameBrowseButton, QtCore.SIGNAL("clicked()"), self.importDensTable)
        self.connect(self.DopplerFNameBrowseButton, QtCore.SIGNAL("clicked()"), self.importDopplerTable)
        self.connect(self.TdustFNameBrowseButton, QtCore.SIGNAL("clicked()"), self.importTdustTable)
        self.connect(self.MagfieldFNameBrowseButton, QtCore.SIGNAL("clicked()"), self.importMagfieldTable)
        self.connect(self.TempFNameBrowseButton, QtCore.SIGNAL("clicked()"), self.importTempTable)
        self.connect(self.VeloFNameBrowseButton, QtCore.SIGNAL("clicked()"), self.importVeloTable)


        self.connect(self.LRTBrowseMoldatfileSingleButton, QtCore.SIGNAL("clicked()"), self.openMoldatfileBrowserDialogSingle)
        self.connect(self.LRTBrowseMoldatfileMultiButton, QtCore.SIGNAL("clicked()"), self.openMoldatfileBrowserDialogMulti)
        self.connect(self.LRTimagemodeCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setContInpFields)
        
        self.connect(self.MultiImageParGroupBox, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)

        self.connect(self.ImGridinclCBox, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridinclArbitValRButton, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridinclRGridValRButton, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridinclRGridValGridParButton, QtCore.SIGNAL("clicked()"), self.openRGridDialogincl)
        
        self.connect(self.ImGridimgresCBox, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridimgresArbitValRButton, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridimgresRGridValRButton, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridimgresRGridValGridParButton, QtCore.SIGNAL("clicked()"), self.openRGridDialogimgres)

        self.connect(self.ImGridvelresCBox, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridvelresArbitValRButton, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridvelresRGridValRButton, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridvelresRGridValGridParButton, QtCore.SIGNAL("clicked()"), self.openRGridDialogvelres)
        
        self.connect(self.ImGridtransCBox, QtCore.SIGNAL("clicked()"), self.setMultiImageWidgetVisibility)
       
        self.connect(self.ImGridFNameTag1Ledit, QtCore.SIGNAL("returnPressed()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridFNameTag2Ledit, QtCore.SIGNAL("returnPressed()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridFNameTag3Ledit, QtCore.SIGNAL("returnPressed()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridFNameTag4Ledit, QtCore.SIGNAL("returnPressed()"), self.setMultiImageWidgetVisibility)
        self.connect(self.ImGridFNameSeparatorCoBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.setMultiImageWidgetVisibility)



        self.ImageViewerMainWindow = ImageViewerMainWindow(self)
        self.ImageViewerMainWindow.actionCustomize.setEnabled(False)
        self.ImageViewerMainWindow.actionCustomize.setVisible(False)
        self.ImageViewerMainWindow.actionQuitIvan.setEnabled(False)
        self.ImageViewerMainWindow.actionQuitIvan.setVisible(False)
        self.connect(self.ButtonViewImage, QtCore.SIGNAL("clicked(bool)"),
                self.openImageViewerMainWindow)

        log.debug('Initalizing Image viewer')

        self.MainControlTabWidget.setCurrentIndex(0)


        self.setTabOrder(self.ModelPar1Ledit, self.ModelPar1CBox)
        self.setTabOrder(self.ModelPar1CBox, self.ModelPar2Ledit)
        self.setTabOrder(self.ModelPar2Ledit, self.ModelPar2CBox)
        self.setTabOrder(self.ModelPar2CBox, self.ModelPar3Ledit)
        self.setTabOrder(self.ModelPar3Ledit, self.ModelPar3CBox)
        self.setTabOrder(self.ModelPar3CBox, self.ModelPar4Ledit)
        self.setTabOrder(self.ModelPar4Ledit, self.ModelPar4CBox)
        self.setTabOrder(self.ModelPar4CBox, self.ModelPar5Ledit)
        self.setTabOrder(self.ModelPar5Ledit, self.ModelPar5CBox)
        self.setTabOrder(self.ModelPar5CBox, self.ModelPar6Ledit)
        self.setTabOrder(self.ModelPar6Ledit, self.ModelPar6CBox)
        self.setTabOrder(self.ModelPar6CBox, self.ModelPar7Ledit)
        self.setTabOrder(self.ModelPar7Ledit, self.ModelPar7CBox)
        self.setTabOrder(self.ModelPar7CBox, self.ModelPar8Ledit)
        self.setTabOrder(self.ModelPar8Ledit, self.ModelPar8CBox)
        self.setTabOrder(self.ModelPar8CBox, self.ModelPar9Ledit)
        self.setTabOrder(self.ModelPar9Ledit, self.ModelPar9CBox)
        self.setTabOrder(self.ModelPar9CBox, self.ModelPar10Ledit)
        self.setTabOrder(self.ModelPar10Ledit, self.DensInpTypeCBox)
        
        self.setTabOrder(self.DensInpTypeCBox, self.DensValueLedit) 
        self.setTabOrder(self.DensValueLedit, self.DensFNameLedit) 
        self.setTabOrder(self.DensFNameLedit, self.DensValueLedit) 
        self.setTabOrder(self.DensValueLedit, self.DensFNameBrowseButton) 
        self.setTabOrder(self.DensFNameBrowseButton, self.TempInpTypeCBox) 
        

        self.setTabOrder(self.TempInpTypeCBox, self.TempValueLedit) 
        self.setTabOrder(self.TempValueLedit, self.TempFNameLedit) 
        self.setTabOrder(self.TempFNameLedit, self.TempValueLedit) 
        self.setTabOrder(self.TempValueLedit, self.TempFNameBrowseButton) 
        self.setTabOrder(self.TempFNameBrowseButton, self.TdustInpTypeCBox) 
        
        
        self.setTabOrder(self.TdustInpTypeCBox, self.TdustFNameLedit) 
        self.setTabOrder(self.TdustFNameLedit, self.TdustValueLedit) 
        self.setTabOrder(self.TdustValueLedit, self.TdustFNameBrowseButton) 
        self.setTabOrder(self.TdustFNameBrowseButton, self.TdustDustopacFileLedit) 
        self.setTabOrder(self.TdustDustopacFileLedit, self.TdustDustopacFileBrowseButton) 
        self.setTabOrder(self.TdustDustopacFileBrowseButton, self.VeloInpTypeCBox) 

        self.setTabOrder(self.VeloInpTypeCBox, self.VeloFNameLedit) 
        self.setTabOrder(self.VeloFNameLedit, self.VeloValueLedit) 
        self.setTabOrder(self.VeloValueLedit, self.VeloFNameBrowseButton) 
        self.setTabOrder(self.VeloFNameBrowseButton, self.DopplerInpTypeCBox)

        self.setTabOrder(self.DopplerInpTypeCBox, self.DopplerFNameLedit) 
        self.setTabOrder(self.DopplerFNameLedit, self.DopplerValueLedit) 
        self.setTabOrder(self.DopplerValueLedit, self.DopplerFNameBrowseButton) 
        self.setTabOrder(self.DopplerFNameBrowseButton, self.MagfieldInpTypeCBox)


        self.setTabOrder(self.MagfieldInpTypeCBox, self.MagfieldFNameLedit) 
        self.setTabOrder(self.MagfieldFNameLedit, self.MagfieldValueLedit) 
        self.setTabOrder(self.MagfieldValueLedit, self.MagfieldFNameBrowseButton) 
        self.setTabOrder(self.MagfieldFNameBrowseButton, self.AbunRegionCustomizeButton) 

        self.setTabOrder(self.AbunRegionCustomizeButton, self.AbunInpTypeCBox) 
        self.setTabOrder(self.AbunInpTypeCBox, self.AbunFTempLedit) 
        self.setTabOrder(self.AbunFTempLedit, self.AbunValueLedit) 
        self.setTabOrder(self.AbunValueLedit, self.AbunFNameBrowseButton) 
        self.setTabOrder(self.AbunFNameBrowseButton, self.AbunMoldatFileLedit) 
        self.setTabOrder(self.AbunMoldatFileLedit, self.AbunMoldatFileBrowseButton) 
        self.setTabOrder(self.AbunMoldatFileBrowseButton, self.ButtonVisualize1d2d) 


        self.setTabOrder(self.LRTRadiusLedit, self.LRTpIntensityLedit) 
        self.setTabOrder(self.LRTpIntensityLedit, self.LRTsamplingCBox) 
        self.setTabOrder(self.LRTsamplingCBox, self.LRTpopoutfileLedit) 
        self.setTabOrder(self.LRTpopoutfileLedit, self.LRTminScaleLedit) 
        self.setTabOrder(self.LRTminScaleLedit, self.LRTsinkPointsLedit) 
        self.setTabOrder(self.LRTsinkPointsLedit, self.LRTtcmbLedit) 
        self.setTabOrder(self.LRTtcmbLedit, self.LRTgridfileLedit)
        self.setTabOrder(self.LRTgridfileLedit, self.ButtonVisualize1d2d)

        self.setTabOrder(self.LRTpxlsLedit, self.LRTthetaLedit)
        self.setTabOrder(self.LRTthetaLedit, self.LRTphiLedit)
        self.setTabOrder(self.LRTphiLedit, self.LRTunitCBox)
        self.setTabOrder(self.LRTunitCBox, self.LRTimgresLedit)
        self.setTabOrder(self.LRTimgresLedit, self.LRTdistanceLedit)
        self.setTabOrder(self.LRTdistanceLedit, self.LRTfilenameLedit)
        self.setTabOrder(self.LRTfilenameLedit, self.LRTtransLedit)
        self.setTabOrder(self.LRTtransLedit, self.LRTBrowseMoldatfileSingleButton)
        self.setTabOrder(self.LRTBrowseMoldatfileSingleButton, self.LRTsourcevelLedit)
        self.setTabOrder(self.LRTsourcevelLedit, self.LRTvelresLedit)
        self.setTabOrder(self.LRTvelresLedit, self.LRTnchanLedit)
        self.setTabOrder(self.LRTnchanLedit, self.MultiImageParGroupBox)
        self.setTabOrder(self.MultiImageParGroupBox, self.ImGridinclCBox)
        self.setTabOrder(self.ImGridinclCBox, self.ImGridinclArbitValRButton)
        self.setTabOrder(self.ImGridinclArbitValRButton, self.ImGridinclArbitValLedit)
        self.setTabOrder(self.ImGridinclArbitValLedit, self.ImGridinclRGridValRButton)
        self.setTabOrder(self.ImGridinclArbitValLedit, self.ImGridinclRGridValRButton)
        self.setTabOrder(self.ImGridinclRGridValRButton, self.ImGridinclRGridValCBox)
        self.setTabOrder(self.ImGridinclRGridValCBox, self.ImGridinclRGridValGridParButton)
        self.setTabOrder(self.ImGridinclRGridValGridParButton, self.ImGridimgresCBox)
        
        self.setTabOrder(self.ImGridimgresCBox, self.ImGridimgresArbitValRButton)
        self.setTabOrder(self.ImGridimgresArbitValRButton, self.ImGridimgresArbitValLedit)
        self.setTabOrder(self.ImGridimgresArbitValLedit, self.ImGridimgresRGridValRButton)
        self.setTabOrder(self.ImGridimgresRGridValRButton, self.ImGridimgresRGridValCBox)
        self.setTabOrder(self.ImGridimgresRGridValCBox, self.ImGridimgresRGridValGridParButton)
        self.setTabOrder(self.ImGridimgresRGridValGridParButton, self.ImGridvelresCBox)

        self.setTabOrder(self.ImGridvelresCBox, self.ImGridvelresArbitValRButton)
        self.setTabOrder(self.ImGridvelresArbitValRButton, self.ImGridvelresArbitValLedit)
        self.setTabOrder(self.ImGridvelresArbitValLedit, self.ImGridvelresRGridValRButton)
        self.setTabOrder(self.ImGridvelresRGridValRButton, self.ImGridvelresRGridValCBox)
        self.setTabOrder(self.ImGridvelresRGridValCBox, self.ImGridvelresRGridValGridParButton)
        self.setTabOrder(self.ImGridvelresRGridValGridParButton, self.ImGridtransCBox)
        
        self.setTabOrder(self.ImGridtransCBox, self.ImGridtransArbitValLedit)
        self.setTabOrder(self.ImGridtransArbitValLedit, self.LRTBrowseMoldatfileMultiButton)
        self.setTabOrder(self.LRTBrowseMoldatfileMultiButton, self.ImGridFNameSeparatorCoBox)
        self.setTabOrder(self.ImGridFNameSeparatorCoBox, self.ImGridFNameTag1Ledit)
        self.setTabOrder(self.ImGridFNameTag1Ledit, self.ImGridFNameTag2Ledit)
        self.setTabOrder(self.ImGridFNameTag2Ledit, self.ImGridFNameTag3Ledit)
        self.setTabOrder(self.ImGridFNameTag3Ledit, self.ImGridFNameTag4Ledit)
        self.setTabOrder(self.ImGridFNameTag4Ledit, self.ButtonVisualize1d2d)

        self.setTabOrder(self.ButtonVisualize1d2d, self.ButtonRunParaView) 
        self.setTabOrder(self.ButtonRunParaView, self.ButtonRunLineRTCode) 
        self.setTabOrder(self.ButtonRunLineRTCode, self.ButtonViewImage) 
    

        self.configParams = ConfigurationParams() 

        self.VisualizeMainWindow = VisualizeMainWindow(self)
        self.connect(self.ButtonVisualize1d2d, QtCore.SIGNAL("clicked(bool)"),
                self.openVisualizeMainWindow)
        

        self.VisualizeMainWindow.CurrentModelID = 0
        modellib.setCurrentModel(modellib.getModelIDs()[self.VisualizeMainWindow.CurrentModelID])       

        self.connect(self.ButtonRunParaView, QtCore.SIGNAL("clicked(bool)"),
                self.startParaView)



        self.StatusDisplay = StatusDisplay(self)
        self.connect(self.ButtonRunLineRTCode, QtCore.SIGNAL("clicked()"), self.runLime)

        

        if os.path.exists(os.environ['HOME']+'/.artist.cfg'):
            self.configParamFileName = '.artist.cfg'
            self.loadDefaultConfigParams(True)
        else:
            self.configParamFileName = '.artist.cfg'
            self.loadDefaultConfigParams(False)


        self.setModelParamLabels(self.VisualizeMainWindow.CurrentModelID, True)
        self.setLRTParams(self.par,self.img)


        self.DensValueLedit.setText('1000.0')
        self.DensFNameLedit.setText('-1.5')
        self.TempValueLedit.setText('50.0')
        self.TempFNameLedit.setText('-0.5')
        self.TdustValueLedit.setText('50.0')
        self.AbunValueLedit.setText('1e-9')
        self.DopplerValueLedit.setText('100')
        self.MagfieldValueLedit.setText('0.0')

        self.TdustDustopacFileLedit.setText(' ')
        self.AbunMoldatFileLedit.setText(' ')
        modellib.setFunction("temperature", "scalarConst")
        modellib.setFunctionParamDouble("temperature", "val", 50.0)
        self.functionSetTemp = True
        


        self.setLabelsTempField(1,False)
        self.setLabelsDensField(3,False)
        self.setLabelsAbunField(3,False)
        self.setLabelsVeloField(3,False)
        self.setLabelsDopplerField(3,False)
        self.setLabelsMagfield(5,False)
        self.setLabelsDustTempField(4,False)


        
        if ('-model' in sys.argv):
            index = sys.argv.index('-model')+1
            try:
                self.modelDir = sys.argv[index]
            except:
                reply = QtGui.QMessageBox.critical(self, "Error",
                        'No model name is given.\n\nUsage:\n artist -model modelname', 
                        QtGui.QMessageBox.Ok)
                sys.exit(0) 
            try: 
                os.chdir(self.modelDir) 
            except:
                reply = QtGui.QMessageBox.critical(self, "Error",
                        'The directory \n'+self.modelDir+'\ndoes not exist. \nDo you want to create a new model with this name?',
                        QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
                if reply == QtGui.QMessageBox.Yes:
                    if (self.modelDir[-4:]!='.mdl'):
                        self.modelDir = self.modelDir+'.mdl'
                    try:
                        os.mkdir(self.modelDir)
                    except:
                        reply = QtGui.QMessageBox.critical(None,"Error", self.modelDir+'\n could not be created. Please check the '+\
                                'whether the path to the model is correct and whether you have writing permissions in the parent '+\
                                'directory.', QtGui.QMessageBox.Ok)
                        sys.exit(0)

                    os.chdir(self.modelDir)
                    self.modelSetupPar = modelSetupPar(self)
                    self.changeModel(self.VisualizeMainWindow.CurrentModelID)
                    self.saveModel()

                if reply == QtGui.QMessageBox.No:
                    sys.exit(0)

            dum = self.openModel() 
            if not dum:
                reply = QtGui.QMessageBox.critical(None,"Error","Model setup file either "+\
                        " does not exist or is corrupted.", QtGui.QMessageBox.Ok)
                sys.exit(0)
            self.readMoldatfile()
        else:
            self.modelSetupPar = modelSetupPar(self)
            self.startUp()

    def enableDustOpacityWindow(self):

        opacfname = str(self.TdustDustopacFileLedit.text()).strip()

        try:
            opac = np.genfromtxt(opacfname)
        except:
            log.exception('Dust opacity plotting: file cannot be read')
            reply = QtGui.QMessageBox.critical(None,"Error", "Dust opacity file "+opacfname+" cannot be read.", QtGui.QMessageBox.Ok)
            
        self.DustOpacityWindow.mplFigureWidgetPixSpec.canvas.plotData(dataX=opac[:,0], dataY=opac[:,1], xtitle=r'$\lambda$ [$\mu$m]', ytitle=r'$\kappa$ [cm$^2$/g]', logscale=True)  
        self.DustOpacityWindow.setVisible(True)

        

    def hideAllToolTips(self):

        self.showToolTipHelp = False
        self.ModellibBrowserButton.setToolTip('')
        
        self.DensInpFrameLabel.setToolTip('')
        self.DensFNameLedit.setToolTip('')
        self.DensValueLedit.setToolTip('')
        self.DensFNameBrowseButton.setToolTip('')

        self.TempInpFrameLabel.setToolTip('')
        self.TempFNameLedit.setToolTip('')
        self.TempValueLedit.setToolTip('')
        self.TempFNameBrowseButton.setToolTip('')

        self.TdustInpFrameLabel.setToolTip('')
        self.TdustFNameLedit.setToolTip('')
        self.TdustValueLedit.setToolTip('')
        self.TdustFNameBrowseButton.setToolTip('')

        self.VeloInpFrameLabel.setToolTip('')
        self.VeloFNameLedit.setToolTip('')
        self.VeloValueLedit.setToolTip('')
        self.VeloFNameBrowseButton.setToolTip('')

        self.DopplerInpFrameLabel.setToolTip('')
        self.DopplerFNameLedit.setToolTip('')
        self.DopplerValueLedit.setToolTip('')
        self.DopplerFNameBrowseButton.setToolTip('')
        
        self.MagfieldInpFrameLabel.setToolTip('')
        self.MagfieldFNameLedit.setToolTip('')
        self.MagfieldValueLedit.setToolTip('')
        self.MagfieldFNameBrowseButton.setToolTip('')

        self.AbunInpFrameLabel.setToolTip('')
        self.AbunFTempLedit.setToolTip('')
        self.AbunValueLedit.setToolTip('')
        self.AbunFNameBrowseButton.setToolTip('')
        
        
        self.TdustInpFrameLabel.setToolTip('')
        self.TdustInpTypeCBox.setToolTip('')
        self.TdustDustopacFileLedit.setToolTip('')
        self.TdustDustopacFileBrowseButton.setToolTip('')
        self.TdustDustopacPlotButton.setToolTip('')
        
        self.AbunInpFrameLabel.setToolTip('')
        self.AbunInpTypeCBox.setToolTip('')
        self.AbunMoldatFileLedit.setToolTip('')
        self.AbunMoldatFileBrowseButton.setToolTip('')


        self.LRTRadiusLabel.setToolTip('')
        self.LRTRadiusLedit.setToolTip('')
        self.LRTpIntensityLedit.setToolTip('')
        self.LRTpIntensityLabel.setToolTip('')
        self.LRTsamplingLabel.setToolTip('')
        self.LRTsamplingCBox.setToolTip('')
        self.LRTpopoutfileLabel.setToolTip('')
        self.LRTpopoutfileLedit.setToolTip('')
        self.LRTantialiasLedit.setToolTip('')
        self.LRTantialiasLabel.setToolTip('')
        self.LRTLTECBox.setToolTip('')
        
        self.LRTminScaleLabel.setToolTip('')
        self.LRTminScaleLedit.setToolTip('')
        self.LRTsinkPointsLabel.setToolTip('')
        self.LRTsinkPointsLedit.setToolTip('')
        self.LRTtcmbLabel.setToolTip('')
        self.LRTtcmbLedit.setToolTip('')
        self.LRTgridfileLabel.setToolTip('')
        self.LRTgridfileLedit.setToolTip('')
        self.LRTpolarizationCBox.setToolTip('')
        self.LRTblendCBox.setToolTip('')



        self.LRTpxlsLabel.setToolTip('')
        self.LRTpxlsLedit.setToolTip('')
        self.LRTthetaLabel.setToolTip('')
        self.LRTthetaLedit.setToolTip('')
        self.LRTphiLabel.setToolTip('')
        self.LRTphiLedit.setToolTip('')
        self.LRTfreqLabel.setToolTip('')
        self.LRTfreqLedit.setToolTip('')
        self.LRTunitLabel.setToolTip('')
        self.LRTunitCBox.setToolTip('')
        self.LRTimgresLabel.setToolTip('')
        self.LRTimgresLedit.setToolTip('')
        self.LRTdistanceLabel.setToolTip('')
        self.LRTdistanceLedit.setToolTip('')
        self.LRTfilenameLabel.setToolTip('')
        self.LRTfilenameLedit.setToolTip('')
        self.LRTbandwidthLabel.setToolTip('')
        self.LRTbandwidthLedit.setToolTip('')
        self.LRTtransLabel.setToolTip('')
        self.LRTtransLedit.setToolTip('')
        self.LRTsourcevelLabel.setToolTip('')
        self.LRTsourcevelLedit.setToolTip('')
        self.LRTvelresLabel.setToolTip('')
        self.LRTvelresLedit.setToolTip('')
        self.LRTnchanLabel.setToolTip('')
        self.LRTnchanLedit.setToolTip('')
        self.LRTimagemodeLabel.setToolTip('')
        self.LRTimagemodeCBox.setToolTip('')
        self.LRTBrowseMoldatfileSingleButton.setToolTip('')
        
        self.MultiImageParGroupBox.setToolTip('')
        self.ImGridinclCBox.setToolTip('')
        self.ImGridinclArbitValRButton.setToolTip('')
        self.ImGridinclArbitValLedit.setToolTip('')
        self.ImGridinclRGridValRButton.setToolTip('')
        self.ImGridinclRGridValCBox.setToolTip('')
        self.ImGridinclRGridValGridParButton.setToolTip('')


        self.LRTimagemodeLabel.setToolTip('')
        self.ImGridimgresCBox.setToolTip('')
        self.ImGridimgresArbitValRButton.setToolTip('')
        self.ImGridimgresArbitValLedit.setToolTip('')
        self.ImGridimgresRGridValRButton.setToolTip('')
        self.ImGridimgresRGridValCBox.setToolTip('')
        self.ImGridimgresRGridValGridParButton.setToolTip('')
        self.ImGridvelresCBox.setToolTip('')
        self.ImGridvelresArbitValRButton.setToolTip('')
        self.ImGridvelresArbitValLedit.setToolTip('')
        self.ImGridvelresRGridValRButton.setToolTip('')
        self.ImGridvelresRGridValCBox.setToolTip('')
        self.ImGridvelresRGridValGridParButton.setToolTip('')



        self.ImGridtransCBox.setToolTip('')
        self.ImGridtransArbitValLedit.setToolTip('')
        self.LRTBrowseMoldatfileMultiButton.setToolTip('')
        self.ImGridFNameSeparatorCoBox.setToolTip('')
        self.ImGridFNameTag1Ledit.setToolTip('')
        self.ImGridFNameTag2Ledit.setToolTip('')
        self.ImGridFNameTag3Ledit.setToolTip('')
        self.ImGridFNameTag4Ledit.setToolTip('')
        self.ImGridFNameTemplateLabel.setToolTip('')
        
        
        self.ButtonVisualize1d2d.setToolTip('')
        self.ButtonRunParaView.setToolTip('')
        self.ButtonRunLineRTCode.setToolTip('')
        self.ButtonViewImage.setToolTip('')

        self.VisualizeMainWindow.hideAllToolTips()
    
    def setAllToolTips(self):

        self.showToolTipHelp = True

        self.ModellibBrowserButton.setToolTip(str2ToolTipString('Open the modellib browser dialog'))
        
        self.DensInpFrameLabel.setToolTip(str2ToolTipString('Number density of the collisional partner (e.g., H2)'))
        self.DensInpTypeCBox.setToolTip(str2ToolTipString('Chooose how the density is given.'))
        InpType = self.DensInpTypeCBox.currentIndex()
        if InpType==0:
            self.DensFNameLedit.setToolTip('')
            self.DensValueLedit.setToolTip(str2ToolTipString('Constant density value '))
            self.DensFNameBrowseButton.setToolTip('')
        elif InpType==1:
            self.DensFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
            self.DensValueLedit.setToolTip(str2ToolTipString('Value of the density at Rin'))
            self.DensFNameBrowseButton.setToolTip('')
        elif InpType==2:
            self.DensFNameLedit.setToolTip('')
            self.DensValueLedit.setToolTip(str2ToolTipString('Name of the file containing the gas density in an ARTIST Table format (For the description of the file format see the documentation)'))
            self.DensFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC, RADMC-3D)'))
        elif InpType==3:
            self.DensFNameLedit.setToolTip('')
            self.DensValueLedit.setToolTip('')
            self.DensFNameBrowseButton.setToolTip('')
        
        
        self.TempInpFrameLabel.setToolTip(str2ToolTipString('Kinetik gas temperature'))
        self.TempInpTypeCBox.setToolTip(str2ToolTipString('Chooose how the gas temperature is given.'))
        InpType = self.TempInpTypeCBox.currentIndex()
        if InpType==0:
            self.TempFNameLedit.setToolTip('')
            self.TempValueLedit.setToolTip(str2ToolTipString('Constant kinetik temperature value '))
            self.TempFNameBrowseButton.setToolTip('')
        elif InpType==1:
            self.TempFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
            self.TempValueLedit.setToolTip(str2ToolTipString('Value of the gas temperature at Rin'))
            self.TempFNameBrowseButton.setToolTip('')
        elif InpType==2:
            self.TempFNameLedit.setToolTip('')
            self.TempValueLedit.setToolTip(str2ToolTipString('Name of the file containing the kinetik gas temperature in an ARTIST Table format (For the description of the '+\
                    'file format see the documentation)'))
            self.TempFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC-3D)'))
        elif InpType==3:
            self.TempFNameLedit.setToolTip('')
            self.TempValueLedit.setToolTip('')
            self.TempFNameBrowseButton.setToolTip('')
        elif InpType==4:
            self.TempFNameLedit.setToolTip('')
            self.TempValueLedit.setToolTip('')
            self.TempFNameBrowseButton.setToolTip('')


        self.TdustInpFrameLabel.setToolTip(str2ToolTipString('Dust temperature'))
        self.TdustInpTypeCBox.setToolTip(str2ToolTipString('Chooose how the dust temperature is given'))
        self.TdustDustopacFileLedit.setToolTip(str2ToolTipString('Dust opacity file includindg full path to the file. The file should contain the dust opacities in a two column format: first column - wavelength in micrometer, second column - mass absorption coefficient in cm^2/g'))
        self.TdustDustopacFileBrowseButton.setToolTip(str2ToolTipString('Opens a file browser to select dust opacity file'))
        self.TdustDustopacPlotButton.setToolTip(str2ToolTipString('Plots the dust opacity as a function of wavelength in a new window'))


        InpType = self.TdustInpTypeCBox.currentIndex()
        if InpType==0:
            self.TdustFNameLedit.setToolTip('')
            self.TdustValueLedit.setToolTip(str2ToolTipString('Constant dust temperature value '))
            self.TdustFNameBrowseButton.setToolTip('')
        elif InpType==1:
            self.TdustFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
            self.TdustValueLedit.setToolTip(str2ToolTipString('Value of the dust temperature at Rin'))
            self.TdustFNameBrowseButton.setToolTip('')
        elif InpType==2:
            self.TdustFNameLedit.setToolTip('')
            self.TdustValueLedit.setToolTip(str2ToolTipString('Name of the file containing the dust temperature in an ARTIST Table format (For the description of the '+\
                    'file format see the documentation)'))
            self.TdustFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC, RADMC-3D)'))
        elif InpType==3:
            self.TdustFNameLedit.setToolTip('')
            self.TdustValueLedit.setToolTip('')
            self.TdustFNameBrowseButton.setToolTip('')
        elif InpType==4:
            self.TdustFNameLedit.setToolTip('')
            self.TdustValueLedit.setToolTip('')
            self.TdustFNameBrowseButton.setToolTip('')


        self.VeloInpFrameLabel.setToolTip(str2ToolTipString('Velocity field'))
        self.VeloInpTypeCBox.setToolTip(str2ToolTipString('Chooose how the velocity is given.'))
        InpType = self.VeloInpTypeCBox.currentIndex()
        if InpType==0:
            self.VeloFNameLedit.setToolTip('')
            self.VeloValueLedit.setToolTip(str2ToolTipString('Constant velocity vector value. The velocity vector is a constant scalar times the (1,0,0) vector in spherical coordinates, i.e. the velocity vector has only radial component. The sign of the velocity is negative if the vector points towards the origin of the coordinate system.   '))
            self.VeloFNameBrowseButton.setToolTip('')
        elif InpType==1:
            self.VeloFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent. The velocity vector is given in spherical coordinates and only the radial component differs from zero. The sign of the velocity is negative if the vector points towards the origin of the coordinate system. '))
            self.VeloValueLedit.setToolTip(str2ToolTipString('Amplitude of the gas velocity (radial component in spherical coordinates) at Rin. The sign of the velocity is negative if the vector points towards the origin of the coordinate system.'))
            self.VeloFNameBrowseButton.setToolTip('')
        elif InpType==2:
            self.VeloFNameLedit.setToolTip('')
            self.VeloValueLedit.setToolTip(str2ToolTipString('Name of the file containing the kinetik velocity in an ARTIST Table format (For the description of the '+\
                    'file format see the documentation)'))
            self.VeloFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC-3D)'))
        elif InpType==3:
            self.VeloFNameLedit.setToolTip('')
            self.VeloValueLedit.setToolTip('')
            self.VeloFNameBrowseButton.setToolTip('')


        self.DopplerInpFrameLabel.setToolTip(str2ToolTipString('Doppler b parameter to set the turbulent line width.'))
        self.DopplerInpTypeCBox.setToolTip(str2ToolTipString('Chooose how the Doppler b parameter is given.'))
        InpType = self.DopplerInpTypeCBox.currentIndex()
        if InpType==0:
            self.DopplerFNameLedit.setToolTip('')
            self.DopplerValueLedit.setToolTip(str2ToolTipString('Constant Doppler b value '))
            self.DopplerFNameBrowseButton.setToolTip('')
        elif InpType==1:
            self.DopplerFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
            self.DopplerValueLedit.setToolTip(str2ToolTipString('Value of the Doppler b parameter at Rin'))
            self.DopplerFNameBrowseButton.setToolTip('')
        elif InpType==2:
            self.DopplerFNameLedit.setToolTip('')
            self.DopplerValueLedit.setToolTip(str2ToolTipString('Name of the file containing the Doppler b parameter in an ARTIST Table format (For the description of the '+\
                    'file format see the documentation)'))
            self.DopplerFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data (ARTIST Table format) file'))
        elif InpType==3:
            self.DopplerFNameLedit.setToolTip('')
            self.DopplerValueLedit.setToolTip('')
            self.DopplerFNameBrowseButton.setToolTip('')


        self.MagfieldInpFrameLabel.setToolTip(str2ToolTipString('Magnetic field strength'))
        self.MagfieldInpTypeCBox.setToolTip(str2ToolTipString('Chooose how the magnetic field strength is given.'))
        InpType = self.MagfieldInpTypeCBox.currentIndex()
        if InpType==0:
            self.MagfieldFNameLedit.setToolTip('')
            self.MagfieldValueLedit.setToolTip(str2ToolTipString('Constant magnetic field strength value in the Cartesian x coordinate (and zero along the other two coordinates)'))
            self.MagfieldFNameBrowseButton.setToolTip('')
        if InpType==1:
            self.MagfieldFNameLedit.setToolTip('')
            self.MagfieldValueLedit.setToolTip(str2ToolTipString('Constant magnetic field strength value in the Cartesian y coordinate (and zero along the other two coordinates)'))
            self.MagfieldFNameBrowseButton.setToolTip('')
        if InpType==2:
            self.MagfieldFNameLedit.setToolTip('')
            self.MagfieldValueLedit.setToolTip(str2ToolTipString('Constant magnetic field strength value in the Cartesian z coordinate (and zero along the other two coordinates)'))
            self.MagfieldFNameBrowseButton.setToolTip('')
        elif InpType==3:

            self.MagfieldFNameLedit.setToolTip(str2ToolTipString('Dipole field with dipole moment (mDipole): \n if r &gt; Rmin\n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / r**3\n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / r**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / r**3\n if r &lt; Rmin \n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / Rmin**3 \n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / Rmin**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / Rmin**3', nobreak=True))
            self.MagfieldValueLedit.setToolTip(str2ToolTipString('Dipole field with dipole moment (mDipole): \n if r &gt; Rmin\n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / r**3\n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / r**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / r**3\n if r &lt; Rmin \n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / Rmin**3 \n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / Rmin**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / Rmin**3', nobreak=True))

            self.MagfieldFNameBrowseButton.setToolTip('')
        elif InpType==4:
            self.MagfieldFNameLedit.setToolTip('')
            self.MagfieldValueLedit.setToolTip(str2ToolTipString('Name of the file containing the Magnetic field in an ARTIST Table format (For the description of the '+\
                    'file format see the documentation)'))
            self.MagfieldFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data (ARTIST Table format) file'))
        elif InpType==5:
            self.MagfieldFNameLedit.setToolTip('')
            self.MagfieldValueLedit.setToolTip('')
            self.MagfieldFNameBrowseButton.setToolTip('')
        

        self.AbunInpFrameLabel.setToolTip(str2ToolTipString('Molecular abundance'))
        self.AbunInpTypeCBox.setToolTip(str2ToolTipString('Chooose how the Abundance is given'))
        self.AbunMoldatFileLedit.setToolTip(str2ToolTipString('Molecular data file in LAMBDA format (with full path)'))
        self.AbunMoldatFileBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select molecular data file'))
        InpType = self.AbunInpTypeCBox.currentIndex()
        if InpType==0:
            self.AbunFTempLedit.setToolTip('')
            self.AbunValueLedit.setToolTip(str2ToolTipString('Constant Abundance value '))
            self.AbunFNameBrowseButton.setToolTip('')
        elif InpType==1:
            self.AbunFTempLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
            self.AbunValueLedit.setToolTip(str2ToolTipString('Value of the Abundance parameter at Rin'))
            self.AbunFNameBrowseButton.setToolTip('')
        elif InpType==2:
            self.AbunFTempLedit.setToolTip('')
            self.AbunValueLedit.setToolTip('')
            self.AbunFNameBrowseButton.setToolTip('')
        elif InpType==3:
            self.AbunFTempLedit.setToolTip('')
            self.AbunValueLedit.setToolTip(str2ToolTipString('Name of the file containing the Abundance in an ARTIST Table format (For the description of the '+\
                    'file format see the documentation)'))
            self.AbunFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data (ARTIST Table format) file'))
        elif InpType==4:
            self.AbunFTempLedit.setToolTip('')
            self.AbunValueLedit.setToolTip('')
            self.AbunFNameBrowseButton.setToolTip('')


        self.LRTRadiusLabel.setToolTip(str2ToolTipString('Largest scale in the model grid (outer radius)'))
        self.LRTRadiusLedit.setToolTip(str2ToolTipString('Largest scale in the model grid (outer radius)'))
        self.LRTpIntensityLedit.setToolTip(str2ToolTipString('Number of grid points in the modelspace'))
        self.LRTpIntensityLabel.setToolTip(str2ToolTipString('Number of grid points in the modelspace'))
        self.LRTsamplingLabel.setToolTip(str2ToolTipString('Sampling type of the grid points. "Spherical" sampling is logarithmic in the spherical R coordinate and linear in the angular coordinates. "Cartesian" is linear in all three cartesian coordinates'))
        self.LRTsamplingCBox.setToolTip(str2ToolTipString('Sampling type of the grid points. "Spherical" sampling is logarithmic in the spherical R coordiante and linear in the angular coordinates. "Cartesian" is linear in all three cartesian coordinates'))
        self.LRTpopoutfileLabel.setToolTip(str2ToolTipString('Output file that will contain the level population at each gridpoint'))
        self.LRTpopoutfileLedit.setToolTip(str2ToolTipString('Output file that will contain the level population at each gridpoint'))
        self.LRTantialiasLedit.setToolTip(str2ToolTipString('Number of rays shot from a single image pixel. The intensity at each pixel will be the average of the intensities calculated along the individual rays.'))
        self.LRTantialiasLabel.setToolTip(str2ToolTipString('Number of rays shot from a single image pixel. The intensity at each pixel will be the average of the intensities calculated along the individual rays.'))
        self.LRTLTECBox.setToolTip(str2ToolTipString('Enables LTE excitation calculation instead of the default full NLTE'))
        
        self.LRTminScaleLabel.setToolTip(str2ToolTipString('Minimum spatial scales in the model (inner radius of the grid)'))
        self.LRTminScaleLedit.setToolTip(str2ToolTipString('Minimum spatial scales in the model (inner radius of the grid)'))
        self.LRTsinkPointsLabel.setToolTip(str2ToolTipString('Number of sinkpoints at the outer grid boundary'))
        self.LRTsinkPointsLedit.setToolTip(str2ToolTipString('Number of sinkpoints at the outer grid boundary'))
        self.LRTtcmbLabel.setToolTip(str2ToolTipString('Effective temperature of the background radiation field'))
        self.LRTtcmbLedit.setToolTip(str2ToolTipString('Effective temperature of the background radiation field'))
        self.LRTgridfileLabel.setToolTip(str2ToolTipString('File containing the physical variables in the model in a VTK format that can be used for 3D visualization with e.g. ParaView'))
        self.LRTgridfileLedit.setToolTip(str2ToolTipString('File containing the physical variables in the model in a VTK format that can be used for 3D visualization with e.g. ParaView'))
        self.LRTpolarizationCBox.setToolTip(str2ToolTipString('Enables continuum polarization'))
        self.LRTblendCBox.setToolTip(str2ToolTipString('Enables line blending'))



        self.LRTpxlsLabel.setToolTip(str2ToolTipString('Number of spatial pixels along one side of the image (images are rectangular)'))
        self.LRTpxlsLedit.setToolTip(str2ToolTipString('Number of spatial pixels along one side of the image (images are rectangular)'))
        self.LRTthetaLabel.setToolTip(str2ToolTipString('Rotation angle around the x-axis (equal to the inclination if phi=0). It can be calculated from the inclination (i) and the position angle (PA) as theta = arcsin(cos(-PA) * sin(incl))'))
        self.LRTthetaLedit.setToolTip(str2ToolTipString('Rotation angle around the x-axis (equal to the inclination if phi=0). It can be calculated from the inclination (i) and the position angle (PA) as theta = arcsin(cos(-PA) * sin(incl))'))
        self.LRTphiLabel.setToolTip(str2ToolTipString('Rotation angle around the z-axis. From a given inclination angle (i) and position angle (PA) phi can be calculated as phi = arctan(sin(-PA)*sin(i) / cos(i))'))
        self.LRTphiLedit.setToolTip(str2ToolTipString('Rotation angle around the z-axis. From a given inclination angle (i) and position angle (PA) phi can be calculated as phi = arctan(sin(-PA)*sin(i) / cos(i))'))
        self.LRTfreqLabel.setToolTip(str2ToolTipString('Central frequency of the image'))
        self.LRTfreqLedit.setToolTip(str2ToolTipString('Central frequency of the image'))
        self.LRTunitLabel.setToolTip(str2ToolTipString('Unit of the pixel values'))
        self.LRTunitCBox.setToolTip(str2ToolTipString('Unit of the pixel values'))
        self.LRTimgresLabel.setToolTip(str2ToolTipString('Image resolution (i.e., pixelsize) in arcsecond'))
        self.LRTimgresLedit.setToolTip(str2ToolTipString('Image resolution (i.e., pixelsize) in arcsecond'))
        self.LRTdistanceLabel.setToolTip(str2ToolTipString('Distance of the source in pc'))
        self.LRTdistanceLedit.setToolTip(str2ToolTipString('Distance of the source in pc'))
        self.LRTfilenameLabel.setToolTip(str2ToolTipString('Name of the fits file into which the image should be written'))
        self.LRTfilenameLedit.setToolTip(str2ToolTipString('Name of the fits file into which the image should be written'))
        self.LRTbandwidthLabel.setToolTip(str2ToolTipString('Bandwidth of the image (i.e., the value of the CDELT fits header keyword)'))
        self.LRTbandwidthLedit.setToolTip(str2ToolTipString('Bandwidth of the image (i.e., the value of the CDELT fits header keyword)'))
        self.LRTtransLabel.setToolTip(str2ToolTipString('Line transition number in the molecular data file'))
        self.LRTtransLedit.setToolTip(str2ToolTipString('Line transition number in the molecular data file'))
        self.LRTsourcevelLabel.setToolTip(str2ToolTipString('Systematic source velocity (shifts the whole image in frequency/velocity)'))
        self.LRTsourcevelLedit.setToolTip(str2ToolTipString('Systematic source velocity (shifts the whole image in frequency/velocity)'))
        self.LRTvelresLabel.setToolTip(str2ToolTipString('Velocity resolution of the image'))
        self.LRTvelresLedit.setToolTip(str2ToolTipString('Velocity resolution of the image'))
        self.LRTnchanLabel.setToolTip(str2ToolTipString('Number of velocity channels to calculate'))
        self.LRTnchanLedit.setToolTip(str2ToolTipString('Number of velocity channels to calculate'))
        self.LRTimagemodeLabel.setToolTip(str2ToolTipString('Sets the way how the spectral axis of the image is defined. "Line image (velocity)" calculates an image with the spectral axis given in velocity, "Line image (frequency)" calculates an image with the spectral axis defined in frequency, while the "Continuum image only" calculates a continuum image with no line emission with the spectral axis defined in frequency.'))
        self.LRTimagemodeCBox.setToolTip(str2ToolTipString('Sets the way how the spectral axis of the image is defined. "Line image (velocity)" calculates an image with the spectral axis given in velocity, "Line image (frequency)" calculates an image with the spectral axis defined in frequency, while the "Continuum image only" calculates a continuum image with no line emission with the spectral axis defined in frequency.'))
        self.LRTBrowseMoldatfileSingleButton.setToolTip(str2ToolTipString( 'Opens moleculara data file browser window that lists all the available transitions in the selected molecular datafile to select the desired transition'))
        
        self.ImGridinclCBox.setToolTip(str2ToolTipString('Enables the inclination to be changed from image to image if multiple images are calculated'))
        self.ImGridinclArbitValRButton.setToolTip(str2ToolTipString('The inclination angles are given as arbitrary (irregulary spaced / ordered) numbers'))
        self.ImGridinclArbitValLedit.setToolTip(str2ToolTipString('Comma-separated list of inclination angles (arbitrary numbers in arbitrary order)'))
        self.ImGridinclRGridValRButton.setToolTip(str2ToolTipString('The inclination angles are given by a regular grid'))
        self.ImGridinclRGridValCBox.setToolTip(str2ToolTipString('List of inclination angles'))
        self.ImGridinclRGridValGridParButton.setToolTip(str2ToolTipString('Opens a dialog to edit the parameters of the grid (lower- and upper-boundaries, resolution, etc.)'))

        self.ImGridimgresCBox.setToolTip(str2ToolTipString('Enables the image resolution to be changed from image to image if multiple images are calculated'))
        self.ImGridimgresArbitValRButton.setToolTip(str2ToolTipString('The image resolution is given as arbitrary (irregulary spaced / ordered) numbers'))
        self.ImGridimgresArbitValLedit.setToolTip(str2ToolTipString('Comma-separated list of image resolutions (arbitrary numbers in arbitrary order)'))
        self.ImGridimgresRGridValRButton.setToolTip(str2ToolTipString('The image resolution is given by a regular grid'))
        self.ImGridimgresRGridValCBox.setToolTip(str2ToolTipString('List of image resolutions'))
        self.ImGridimgresRGridValGridParButton.setToolTip(str2ToolTipString('Opens a dialog to edit the parameters of the grid (lower- and upper-boundaries, stepsize, etc.)'))
        
        self.ImGridvelresCBox.setToolTip(str2ToolTipString('Enables the velocity resolution to be changed from image to image if multiple images are calculated'))
        self.ImGridvelresArbitValRButton.setToolTip(str2ToolTipString('The velocity resolution is given as arbitrary (irregulary spaced / ordered) numbers'))
        self.ImGridvelresArbitValLedit.setToolTip(str2ToolTipString('Comma-separated list of velocity resolutions (arbitrary numbers in arbitrary order)'))
        self.ImGridvelresRGridValRButton.setToolTip(str2ToolTipString('The velocity resolution is given by a regular grid'))
        self.ImGridvelresRGridValCBox.setToolTip(str2ToolTipString('List of velocity resolutions'))
        self.ImGridvelresRGridValGridParButton.setToolTip(str2ToolTipString('Opens a dialog to edit the parameters of the grid (lower- and upper-boundaries, stepsize etc.)'))

        self.ImGridtransCBox.setToolTip(str2ToolTipString('Enables the molecular transition to be changed from image to image if multiple images are calculated'))
        self.ImGridtransArbitValLedit.setToolTip(str2ToolTipString('Comma-separated list of transition numbers (arbitrary numbers in arbitrary order)'))
        self.LRTBrowseMoldatfileMultiButton.setToolTip(str2ToolTipString('Opens a dialog to select multiple transitions from all available transitions in the datafile)'))
        self.ImGridFNameSeparatorCoBox.setToolTip(str2ToolTipString('Separator tag in the image file name'))
        self.ImGridFNameTag1Ledit.setToolTip(str2ToolTipString('File name tag for the grid in theta (equal to the inclination angle if phi=0)'))
        self.ImGridFNameTag2Ledit.setToolTip(str2ToolTipString('File name tag for the grid in image resolution (pixel size)'))
        self.ImGridFNameTag3Ledit.setToolTip(str2ToolTipString('File name tag for the grid in velocity resolution'))
        self.ImGridFNameTag4Ledit.setToolTip(str2ToolTipString('File name tag for the grid in molecular transition'))
        self.ImGridFNameTemplateLabel.setToolTip(str2ToolTipString('Example file name with the given file name tags'))


        # Set the visualization window tooltips

        self.VisualizeMainWindow.setAllToolTips()


    def moveToPreviousTab(self):
        ind = self.MainControlTabWidget.currentIndex()
        if ind==0: ind = 3
        self.MainControlTabWidget.setCurrentIndex((ind-1)%3)

    def moveToNextTab(self):

        ind = self.MainControlTabWidget.currentIndex()
        self.MainControlTabWidget.setCurrentIndex((ind+1)%3)



    def createNewModel(self):
        self.modelDir = QtGui.QFileDialog.getSaveFileName(self, 'Create Model', 'ARTIST models (*.mdl)', options=QtGui.QFileDialog.DontUseNativeDialog)
        self.modelDir = str(self.modelDir)
        if self.modelDir=="":
            reply = QtGui.QMessageBox.critical(None,"File name error", 'No model name was selected \n New model cannot be created', QtGui.QMessageBox.Ok)
            if reply == QtGui.QMessageBox.Ok:
                return
        else:
            if (self.modelDir[-4:]!='.mdl'):
                self.modelDir = self.modelDir+'.mdl'
            os.mkdir(self.modelDir)
            os.chdir(self.modelDir)

    def readMoldatfile(self):

        self.moldata = readmoldatfile(self.par.moldatfile)
    
    def openMoldatfileBrowserDialogSingle(self):
        txt = str(self.LRTtransLedit.text()).split(',')
        transID = []
        for i in range(len(txt)):
            dum = str2int(txt[i], ' Transition ')
            transID.append(dum)
        
        if len(txt)==1:
            if txt[0].strip()=='-1':
                transID = [1]
       
        dum = str(self.AbunMoldatFileLedit.text())
        if not os.path.exists(dum):
            reply = QtGui.QMessageBox.critical(None,"Error", ' No molecular data file is selected' ,
                        QtGui.QMessageBox.Ok)
            return

        dum = MoldatfileBrowserDialog(self, self.moldata, selectedTrans=transID, SingleSelectMode=True, configParams=self.configParams)
        dum.returnSelectedTrans.connect(self.updateTransIDsSingle)
        dum.exec_()
        dum.returnSelectedTrans.disconnect(self.updateTransIDsSingle)

    def importTdustTable(self):

        outdir = os.getcwd()
        fname, filterName =  QtGui.QFileDialog.getOpenFileNamesAndFilter(self, 'Import Dust Temperature table', \
                filter='ARTIST table (*.ARTIST.tab);; RADMC table (dusttemp_final.dat);; RADMC-3D table (dust_temperature.dat)', options=QtGui.QFileDialog.DontUseNativeDialog)
        if fname:
            fname = str(fname[0])

            codeName = str(filterName).split()[0]
            
            if codeName == 'ARTIST':
                dum = fname.split('/')
                dum = dum[len(dum)-1]
                self.TdustValueLedit.setText(dum)
                sp.Popen(['cp -v '+fname+' ./'],  stdout=sp.PIPE, shell=True).wait()
            else:
                if codeName == 'RADMC':
                    self.TdustValueLedit.setText('temperature.tab')
                    dum = fname.split('/')
                    tdir = ''
                    for i in range(len(dum)-1):
                        tdir = tdir + '/' + dum[i]
                    tdir = tdir[1:]+'/'
                    dum = ArtistTable.importTable(tableDir=tdir,codeName='radmc',varName='tdust',outDir=outdir)  
                    if dum[0]>0:
                        if dum[0]==1: 
                            msg = ' radius.inp cannot be opened. Variables from RADMC models can only be imported with the corresponding spatial grid files (radius.inp, theta.inp).'
                        if dum[0]==2:
                            msg = ' theta.inp cannot be opened. Variables from RADMC models can only be imported with the corresponding spatial grid files (radius.inp, theta.inp).'
                        if dum[0]==3:
                            msg = ' The number of theta grid points in dusttemp_final.dat and theta.inp differ. Theta mirroring is different in the two files'

                        if dum[0]==4:
                            msg = ' The number of radial grid points in dusttemp_final.dat and radius.inp differ.'

                        if dum[0]==5:
                            msg = ' The number of theta grid points in dusttemp_final.dat and theta.inp differ.'
                        
                        if dum[0]==6:
                            msg = ' dusttemp_final.dat cannot be opened or it is corrupted.'
                            
                        reply = QtGui.QMessageBox.critical(None,"Error", ' RADMC table could not be imported. \n'+msg,\
                                    QtGui.QMessageBox.Ok)
                        return False
                    else:
                        self.TdustValueLedit.setText(dum[2])

                else:
                    if codeName == 'RADMC-3D':
                        dum = fname.split('/')
                        tdir = ''
                        for i in range(len(dum)-1):
                            tdir = tdir + '/' + dum[i]
                        tdir = tdir[1:]+'/'
                        dum = ArtistTable.importTable(tableDir=tdir,codeName='radmc3d',varName='tdust',outDir=outdir)  
                        if dum[0]>0:
                            if dum[0]==1:  
                                msg = ' amr_grid.inp cannot be opened. Variables from RADMC-3D models can only be imported with the corresponding spatial grid file ((amr_grid.inp).'
                            if dum[0]==2:
                                msg = ' amr_grid.inp file contains an AMR or Delanuay grid. These grids are not yet supported'

                            if dum[0]==3:
                                msg = ' amr_grid.inp file contains an unsupported coordinate system.'

                            if dum[0]==4:
                                msg = ' Number of grid points in amr_grid.inp and dust_temperature.dat differ.'

                            if dum[0]==5:
                                msg = ' dust_temperature.dat cannot be opened or it is corrupted.'
                            reply = QtGui.QMessageBox.critical(None,"Error", ' RADMC-3D table could not be imported. \n'+msg,\
                                        QtGui.QMessageBox.Ok)
                        else:
                            self.TdustValueLedit.setText(dum[2])
                    else:
                        log.debug('Unknown code name')
                        print 'This should not happen...'

    def importAbunTable(self):
        fname, filterName =  QtGui.QFileDialog.getOpenFileNamesAndFilter(self, 'Import Abundance table', \
                filter='ARTIST table (*.ARTIST.tab)', options=QtGui.QFileDialog.DontUseNativeDialog)
        if fname:
            fname = str(fname[0])

            codeName = str(filterName).split()[0]
            
            if codeName == 'ARTIST':
                rfile = open(fname, 'r')
                var   = rfile.readline().strip()
                rfile.close()
                if var!='abundance':

                    reply = QtGui.QMessageBox.critical(None,"Error", ' RADMC-3D table could not be imported. \n'+msg,\
                                QtGui.QMessageBox.Ok)
                else:
                    dum = fname.split('/')
                    dum = dum[len(dum)-1]
                    self.AbunValueLedit.setText(dum)
                    sp.Popen(['cp -v '+fname+' ./'],  stdout=sp.PIPE, shell=True).wait()
            else:
                log.exception('Unknown code name when importing Abundance table')
                log.exception('Codename : '+codeName)
                print 'This should not happen...'

    def importDopplerTable(self):
        fname, filterName =  QtGui.QFileDialog.getOpenFileNamesAndFilter(self, 'Import Doppler table', \
                filter='ARTIST table (*.ARTIST.tab)', options=QtGui.QFileDialog.DontUseNativeDialog)
        if fname:
            fname = str(fname[0])

            codeName = str(filterName).split()[0]
            
            if codeName == 'ARTIST':
                dum = fname.split('/')
                dum = dum[len(dum)-1]
                self.DopplerValueLedit.setText(dum)
                sp.Popen(['cp -v '+fname+' ./'],  stdout=sp.PIPE, shell=True).wait()
            else:
                log.exception('Unknown code name when importing Doppler table')
                log.exception('Codename : '+codeName)
                print 'This should not happen...'

    def importMagfieldTable(self):
        fname, filterName =  QtGui.QFileDialog.getOpenFileNamesAndFilter(self, 'Import Magnetic field table', \
                filter='ARTIST table (*.ARTIST.tab)', options=QtGui.QFileDialog.DontUseNativeDialog)
        if fname:
            fname = str(fname[0])

            codeName = str(filterName).split()[0]
            
            if codeName == 'ARTIST':
                dum = fname.split('/')
                dum = dum[len(dum)-1]
                self.MagfieldValueLedit.setText(dum)
                sp.Popen(['cp -v '+fname+' ./'],  stdout=sp.PIPE, shell=True).wait()
            else:
                log.exception('Unknown code name when importing Magnetic field table')
                log.exception('Codename : '+codeName)
                print 'This should not happen...'


    def importVeloTable(self):
        outdir = os.getcwd()
        fname, filterName =  QtGui.QFileDialog.getOpenFileNamesAndFilter(self, 'Import Velocity table', \
                filter='ARTIST table (*.ARTIST.tab);; RADMC-3D table (gas_velocity.inp)', options=QtGui.QFileDialog.DontUseNativeDialog)
        if fname:
            fname = str(fname[0])

            codeName = str(filterName).split()[0]
            
            if codeName == 'ARTIST':
                dum = fname.split('/')
                dum = dum[len(dum)-1]
                self.VeloValueLedit.setText(dum)
                sp.Popen(['cp -v '+fname+' ./'],  stdout=sp.PIPE, shell=True).wait()
            else:
                if codeName == 'RADMC-3D':
                    dum = fname.split('/')
                    tdir = ''
                    for i in range(len(dum)-1):
                        tdir = tdir + '/' + dum[i]
                    tdir = tdir[1:]+'/'
                    dum = ArtistTable.importTable(tableDir=tdir,codeName='radmc3d',varName='velocity',outDir=outdir)  
                    if dum[0]>0:
                        if dum[0]==1:  
                            msg = ' amr_grid.inp cannot be opened. Variables from RADMC-3D models can only be imported with the corresponding spatial grid file ((amr_grid.inp).'
                        if dum[0]==2:
                            msg = ' amr_grid.inp file contains an AMR or Delanuay grid. These grids are not yet supported'

                        if dum[0]==3:
                            msg = ' amr_grid.inp file contains an unsupported coordinate system.'

                        if dum[0]==4:
                            msg = ' Number of grid points in amr_grid.inp and gas_velocity.inp differ.'

                        if dum[0]==5:
                            msg = ' gas_velocity.inp cannot be opened or it is corrupted.'
                        reply = QtGui.QMessageBox.critical(None,"Error", ' RADMC-3D table could not be imported. \n'+msg,\
                                    QtGui.QMessageBox.Ok)
                    else:
                        self.VeloValueLedit.setText(dum[2])
                else:
                    log.exception('Unknown code name when importing velocity table')
                    log.exception('Codename : '+codeName)
                    print 'This should not happen...'


    def importTempTable(self):

        outdir = os.getcwd()
        fname, filterName =  QtGui.QFileDialog.getOpenFileNamesAndFilter(self, 'Import Temperature table', \
                filter='ARTIST table (*.ARTIST.tab);; RADMC-3D table (gas_temperature.inp)', options=QtGui.QFileDialog.DontUseNativeDialog)
        if fname:
            fname = str(fname[0])

            codeName = str(filterName).split()[0]
            
            if codeName == 'ARTIST':
                dum = fname.split('/')
                dum = dum[len(dum)-1]
                self.TempValueLedit.setText(dum)
                sp.Popen(['cp -v '+fname+' ./'],  stdout=sp.PIPE, shell=True).wait()
            else:
                if codeName == 'RADMC-3D':
                    dum = fname.split('/')
                    tdir = ''
                    for i in range(len(dum)-1):
                        tdir = tdir + '/' + dum[i]
                    tdir = tdir[1:]+'/'
                    dum = ArtistTable.importTable(tableDir=tdir,codeName='radmc3d',varName='temperature',outDir=outdir)  
                    if dum[0]>0:
                        if dum[0]==1:  
                            msg = ' amr_grid.inp cannot be opened. Variables from RADMC-3D models can only be imported with the corresponding spatial grid file ((amr_grid.inp).'
                        if dum[0]==2:
                            msg = ' amr_grid.inp file contains an AMR or Delanuay grid. These grids are not yet supported'

                        if dum[0]==3:
                            msg = ' amr_grid.inp file contains an unsupported coordinate system.'

                        if dum[0]==4:
                            msg = ' Number of grid points in amr_grid.inp and dust_temperature.dat differ.'

                        if dum[0]==5:
                            msg = ' dust_temperature.dat cannot be opened or it is corrupted.'
                        reply = QtGui.QMessageBox.critical(None,"Error", ' RADMC-3D table could not be imported. \n'+msg,\
                                    QtGui.QMessageBox.Ok)
                    else:
                        self.TempValueLedit.setText(dum[2])
                else:
                    log.debug('Unknown code name')
                    print 'This should not happen...'



    def importDensTable(self):
        
        outdir = os.getcwd()
        fname, filterName =  QtGui.QFileDialog.getOpenFileNamesAndFilter(self, 'Import Density table', \
                filter='ARTIST table (*.ARTIST.tab);; RADMC table (dustdens.inp);; RADMC-3D table (dust_density.inp)', options=QtGui.QFileDialog.DontUseNativeDialog)
        if fname:
            fname = str(fname[0])

            codeName = str(filterName).split()[0]
            
            if codeName == 'ARTIST':
                dum = fname.split('/')
                dum = dum[len(dum)-1]
                self.DensValueLedit.setText(dum)
                sp.Popen(['cp -v '+fname+' ./'],  stdout=sp.PIPE, shell=True).wait()
            else:
                if codeName == 'RADMC':
                    dum = fname.split('/')
                    tdir = ''
                    for i in range(len(dum)-1):
                        tdir = tdir + '/' + dum[i]
                    tdir = tdir[1:]+'/'
                    dum = ArtistTable.importTable(tableDir=tdir,codeName='radmc',varName='density',outDir=outdir)  
                    if dum[0]>0:
                        if dum[0]==1: 
                            msg = ' radius.inp cannot be opened. Variables from RADMC models can only be imported with the corresponding spatial grid files (radius.inp, theta.inp).'
                        if dum[0]==2:
                            msg = ' theta.inp cannot be opened. Variables from RADMC models can only be imported with the corresponding spatial grid files (radius.inp, theta.inp).'
                        if dum[0]==3:
                            msg = ' The number of theta grid points in dustdens.inp and theta.inp differ. Theta mirroring is different in the two files'

                        if dum[0]==4:
                            msg = ' The number of radial grid points in dustdens.inp and radius.inp differ.'

                        if dum[0]==5:
                            msg = ' The number of theta grid points in dustdens.inp and theta.inp differ.'
                            
                        if dum[0]==6:
                            msg = ' dustdens.inp cannot be opened or it is corrupted.'

                        reply = QtGui.QMessageBox.critical(None,"Error", ' RADMC table could not be imported. \n'+msg,\
                                    QtGui.QMessageBox.Ok)

                        return False
                    else:
                        self.DensValueLedit.setText(dum[2])
                    
                else:
                    if codeName == 'RADMC-3D':
                        dum = fname.split('/')
                        tdir = ''
                        for i in range(len(dum)-1):
                            tdir = tdir + '/' + dum[i]
                        tdir = tdir[1:]+'/'
                        dum = ArtistTable.importTable(tableDir=tdir,codeName='radmc3d',varName='density',outDir=outdir)  
                        if dum[0]>0:
                            if dum[0]==1:  
                                msg = ' amr_grid.inp cannot be opened. Variables from RADMC-3D models can only be imported with the corresponding spatial grid file ((amr_grid.inp).'
                            if dum[0]==2:
                                msg = ' amr_grid.inp file contains an AMR or Delanuay grid. These grids are not yet supported'

                            if dum[0]==3:
                                msg = ' amr_grid.inp file contains an unsupported coordinate system.'

                            if dum[0]==4:
                                msg = ' Number of grid points in amr_grid.inp and dust_density.inp differ.'

                            if dum[0]==5:
                                msg = ' dust_density.inp cannot be opened or it is corrupted.'
                            print dum
                            reply = QtGui.QMessageBox.critical(None,"Error", ' RADMC-3D table could not be imported. \n'+msg,\
                                        QtGui.QMessageBox.Ok)
                        else:
                            self.DensValueLedit.setText(dum[2])
                           # self.DensValueLedit.setText('density.tab')

                    else:
                        log.debug('Unknown code name')
                        print 'This should not happen...'



    def openAboutDialog(self):

        dum = AboutDialog()
        dum.exec_()

    def updateFigureWithNewAbundance(self, abunParams=None):

        self.changeParams()
        self.RegionAbunParams = abunParams

    def openAbundanceCustomDialog(self):

        dum = AbundanceCustomDialog(minScale=self.par.minScale, radius=self.par.radius, abunModel=self.RegionAbunParams, configParams=self.configParams)
        dum.applyCustomAbundance.connect(self.updateFigureWithNewAbundance)
        dum.exec_()
        dum.applyCustomAbundance.disconnect(self.updateFigureWithNewAbundance)

    def openMoldatfileBrowserDialogMulti(self):


        if self.moldata==None:
            if self.par.moldatfile.strip()=='':
                dum = str(self.AbunMoldatFileLedit.text())
                if dum.strip()=='':
                    reply = QtGui.QMessageBox.critical(None,"Error", 'No molecular data file is selected.',\
                            QtGui.QMessageBox.Ok)
                    return
                else:
                    self.readMoldatfile()
        txt = self.ImGridtransArbitValLedit.text()
        if str(txt).strip()!='':
            txt = txt.split(',')
            transID = [] 
            for i in range(len(txt)):
                dum = str2int(txt[i], ' Transition ')
                transID.append(dum)
        else:
            transID = None

        dum = MoldatfileBrowserDialog(self, self.moldata, selectedTrans=transID, configParams=self.configParams)
        dum.returnSelectedTrans.connect(self.updateTransIDsMulti)
        dum.exec_()
        dum.returnSelectedTrans.disconnect(self.updateTransIDsMulti)

    def startUp(self):

        dialogFinished = 0
        while dialogFinished==0:
            whatToDo = startupDialog()
            whatToDo.exec_()

            if (whatToDo.result()==1):
                print 'Open a model'
                self.modelDir = QtGui.QFileDialog.getExistingDirectory(self, 'Open Model',  options=QtGui.QFileDialog.DontUseNativeDialog)
                self.modelDir = str(self.modelDir)
                if self.modelDir!="":
                    os.chdir(self.modelDir)
                    dum = self.openModel()
                    if dum:
                        dialogFinished = 1

            if (whatToDo.result()==0):
                print 'Create a model'
                self.inOpeningModel = True
                self.modelDir = QtGui.QFileDialog.getSaveFileName(self, 'Create Model', 'ARTIST models (*.mdl)')
                self.modelDir = str(self.modelDir)
                if self.modelDir!="":
                    if (self.modelDir[-4:]!='.mdl'):
                        self.modelDir = self.modelDir+'.mdl'
                    os.mkdir(self.modelDir)
                    os.chdir(self.modelDir)
                    dialogFinished = 1

                self.changeModel(self.VisualizeMainWindow.CurrentModelID)
                self.inOpeningModel = False
                self.saveModel()

    def updateMainWindowCustomization(self, configParams, doUpdate):


        if self.configParams.unitSystem!=configParams.unitSystem:
            self.configParams = cp.deepcopy(configParams)
            self.VisualizeMainWindow.updateFigure1D()
            self.VisualizeMainWindow.mplFigureWidget2D.canvas.updateModelVars(self.VisualizeMainWindow)
            self.VisualizeMainWindow.reDrawContours()
        else: 
            self.configParams = cp.deepcopy(configParams)

        if doUpdate:
            self.StatusDisplay.setStyleSheet('font-family: '+self.configParams.windowParams['font_family']+\
                    '; font-size: '+self.configParams.windowParams['font_size']+'pt;')

            self.ImageViewerMainWindow.updateImageViewerCustomization(configParams, 1)
            self.setStyleSheet('font-family: '+self.configParams.windowParams['font_family']+\
                    '; font-size: '+self.configParams.windowParams['font_size']+'pt;')
            self.VisualizeMainWindow.setStyleSheet('font-family: '+self.configParams.windowParams['font_family']+\
                    '; font-size: '+self.configParams.windowParams['font_size']+'pt;')

            self.VisualizeMainWindow.mplFigureWidget1D.canvas.updateAxisParams(self.configParams.figureParams)
            self.VisualizeMainWindow.mplFigureWidget2D.canvas.updateAxisParams(self.configParams.figureParams)
    
    def loadDefaultConfigParams(self, readParFile=True):
        
        if readParFile:
            self.configParams.readConfigurationParamsFromFile(fName=self.configParamFileName)
        self.updateMainWindowCustomization(self.configParams, 1)

    def writeDefaultConfigParams(self, configParams=None):

        if configParams!=None:
            configParams.writeConfigurationParamsToFile(fName=self.configParamFileName)

    def openCustomizeDialog(self):

        dum = MainWindowCustomizeDialog(self,self.configParams, showToolTipHelp=self.showToolTipHelp)
        dum.updateCustomization.connect(self.updateMainWindowCustomization)
        dum.writeCustomization.connect(self.writeDefaultConfigParams)
        dum.exec_()
        dum.updateCustomization.disconnect(self.updateMainWindowCustomization)
        dum.writeCustomization.disconnect(self.writeDefaultConfigParams)
    

    def openModelDir(self):

        self.modelDir = QtGui.QFileDialog.getExistingDirectory(self, 'Open Model', options=QtGui.QFileDialog.DontUseNativeDialog)
        self.modelDir = str(self.modelDir)
        self.openModel()


    def openDustpathFile(self):  
        
        txt = QtGui.QFileDialog.getOpenFileName(self, 'Open file',  options=QtGui.QFileDialog.DontUseNativeDialog)
        txt = str(txt)
        if txt!='':
            if self.copyDataFilesToModelDir:
                sp.Popen(['cp '+txt+' .'],  stdout=sp.PIPE, shell=True).wait()
                filename = txt.split('/')
                filename = filename[len(filename)-1]
                dum = os.getcwd()
                self.TdustDustopacFileLedit.setText(dum+'/'+filename) 
            else:
                filename = txt.strip()
                self.TdustDustopacFileLedit.setText(filename)


    def openMoldatFile(self):  
        
        txt = QtGui.QFileDialog.getOpenFileName(self, 'Open file', options=QtGui.QFileDialog.DontUseNativeDialog)
        txt = str(txt)
        if txt!='':
            if self.copyDataFilesToModelDir:
                sp.Popen(['cp '+txt+' .'],  stdout=sp.PIPE, shell=True).wait()
                filename = txt.split('/')
                filename = filename[len(filename)-1]
                dum = os.getcwd()
                self.AbunMoldatFileLedit.setText(dum+'/'+filename) 
            else:
                filename = txt.strip()
                self.AbunMoldatFileLedit.setText(filename)
            self.par.moldatfile = filename
            self.readMoldatfile()
    
    def browseModellib(self):

        dum = ModellibBrowser(self, configParams=self.configParams)
        res = dum.exec_()
        if res==1:
            modelIndex = modellib.getModelIDs().index(modellib.getCurrentModelID())
            self.changeModel(modelIndex)
            

    def openModel(self):

        self.inOpeningModel = True
        self.modelSetupPar = modelSetupPar(self)
        dum = self.modelSetupPar.readModelSetupFile()
        if not dum:
            return False
        
        self.RegionAbunParams = cp.deepcopy(self.modelSetupPar.RegionAbunParams)

        modellib.setCurrentModel(self.modelSetupPar.modelID[0])

        for var in self.modelSetupPar.modelParams[0]:
            ptype = modellib.getParamType(self.modelSetupPar.modelID[0],var)
            if ptype=='int':
                modellib.setParamInt(var, self.modelSetupPar.modelParams[0][var])
            if ptype=='double':
                modellib.setParamDouble(var, self.modelSetupPar.modelParams[0][var])
            if ptype=='string':
                modellib.setParamString(var, self.modelSetupPar.modelParams[0][var])
            if ptype=='enum':
                modellib.setParamEnumIndex(var, int(self.modelSetupPar.modelParams[0][var]))


        if self.doFinalizeConfig:
            modellib.finalizeConfiguration()

        self.ModelNameLabel.setText(self.modelSetupPar.modelID[0])

        self.par            = lime.createInputPars()      
        self.par.radius     = self.modelSetupPar.par['radius']
        self.par.minScale   = self.modelSetupPar.par['minScale']
        self.par.pIntensity = int(self.modelSetupPar.par['pIntensity'])
        self.par.sinkPoints = int(self.modelSetupPar.par['sinkPoints'])
        self.par.dust       = self.modelSetupPar.par['dust']
        self.par.outputfile = self.modelSetupPar.par['outputfile']
        self.par.gridfile   = self.modelSetupPar.par['gridfile']
        self.par.moldatfile = self.modelSetupPar.par['moldatfile']
        self.par.lte_only   = int(self.modelSetupPar.par['lte_only'])
        self.par.tcmb       = self.modelSetupPar.par['tcmb']
        self.par.sampling   = int(self.modelSetupPar.par['sampling'])
        self.par.antialias  = int(self.modelSetupPar.par['antialias'])
        self.par.blend      = int(self.modelSetupPar.par['blend'])

        self.images     = []
        self.img = lime.createImage()

        self.img.pxls       	= int(self.modelSetupPar.img[0]['pxls'])
        self.img.imgres     	= self.modelSetupPar.img[0]['imgres']
        self.img.theta	        = self.modelSetupPar.img[0]['theta']
        self.img.distance   	= self.modelSetupPar.img[0]['distance']
        self.img.unit       	= int(self.modelSetupPar.img[0]['unit'])
        self.img.filename   	= self.modelSetupPar.img[0]['filename']

        self.img.phi	       	= self.modelSetupPar.img[0]['phi']
        self.img.source_vel	    = self.modelSetupPar.img[0]['source_vel']
        self.img.nchan      	= int(self.modelSetupPar.img[0]['nchan'])
        self.img.velres     	= self.modelSetupPar.img[0]['velres']
        self.img.trans      	= int(self.modelSetupPar.img[0]['trans'])
        self.img.freq       	= int(self.modelSetupPar.img[0]['freq'])
        self.img.bandwidth    	= int(self.modelSetupPar.img[0]['bandwidth'])

        self.setLRTParams(self.par, self.img)


        index = modellib.getModelIDs().index(self.modelSetupPar.modelID[0])
        self.VisualizeMainWindow.CurrentModelID = index       
        self.setModelParamLabels(index,False,False)
        self.VisualizeMainWindow.mplFigureWidget2D.canvas.PlottedModelID = index

        modelProvided = modellib.getModelResultIDs(self.modelSetupPar.modelID[0]).__contains__('abundance')
        if self.modelSetupPar.modelID[0]=='Tabdata': modelProvided=False
        if modelProvided:
            self.AbunInpFrameLabel.setStyleSheet('color:black')
            if self.AbunInpTypeCBox.count()==4:
                self.AbunInpTypeCBox.addItem('Model')
        else:
            self.AbunInpFrameLabel.setStyleSheet('color:red')
            if self.AbunInpTypeCBox.count()==5:
                self.AbunInpTypeCBox.removeItem(4)
        
        modelProvided = modellib.getModelResultIDs(self.modelSetupPar.modelID[0]).__contains__('density')
        if self.modelSetupPar.modelID[0]=='Tabdata': modelProvided=False
        if modelProvided:
            if self.DensInpTypeCBox.count()==3:
                self.DensInpTypeCBox.addItem('Model')
            self.DensInpFrameLabel.setStyleSheet('color:black')
        else:
            if self.DensInpTypeCBox.count()==4:
                self.DensInpTypeCBox.removeItem(3)
            self.DensInpFrameLabel.setStyleSheet('color:red')

        modelProvided = modellib.getModelResultIDs(self.modelSetupPar.modelID[0]).__contains__('doppler')
        if self.modelSetupPar.modelID[0]=='Tabdata': modelProvided=False
        if modelProvided:
            self.DopplerInpFrameLabel.setStyleSheet('color:black')
            if self.DopplerInpTypeCBox.count()==3:
                self.DopplerInpTypeCBox.addItem('Model')
        else:
            self.DopplerInpFrameLabel.setStyleSheet('color:red')
            if self.DopplerInpTypeCBox.count()==4:
                self.DopplerInpTypeCBox.removeItem(3)
        
        modelProvided = modellib.getModelResultIDs(self.modelSetupPar.modelID[0]).__contains__('tdust')
        if self.modelSetupPar.modelID[0]=='Tabdata': modelProvided=False
        if modelProvided:
            if self.TdustInpTypeCBox.count()==4:
                self.TdustInpTypeCBox.addItem('Model')
            self.TdustInpFrameLabel.setStyleSheet('color:black')
        else:
            self.TdustInpFrameLabel.setStyleSheet('color:red')
            if self.TdustInpTypeCBox.count()==5:
                self.TdustInpTypeCBox.removeItem(4)
        
        modelProvided = modellib.getModelResultIDs(self.modelSetupPar.modelID[0]).__contains__('bmag')
        if self.modelSetupPar.modelID[0]=='Tabdata': modelProvided=False
        if modelProvided:
            if self.MagfieldInpTypeCBox.count()==5:
                self.MagfieldInpTypeCBox.addItem('Model')
            self.MagfieldInpFrameLabel.setStyleSheet('color:black')
        else:
            if self.MagfieldInpTypeCBox.count()==6:
                self.MagfieldInpTypeCBox.removeItem(5)
            self.MagfieldInpFrameLabel.setStyleSheet('color:red')
        
        modelProvided = modellib.getModelResultIDs(self.modelSetupPar.modelID[0]).__contains__('temperature')
        if self.modelSetupPar.modelID[0]=='Tabdata': modelProvided=False
        if modelProvided:
            if self.TempInpTypeCBox.count()==4:
                self.TempInpTypeCBox.addItem('Model')
            self.TempInpFrameLabel.setStyleSheet('color:black')
        else:
            if self.TempInpTypeCBox.count()==5:
                self.TempInpTypeCBox.removeItem(4)
            self.TempInpFrameLabel.setStyleSheet('color:red')
        
        modelProvided = modellib.getModelResultIDs(self.modelSetupPar.modelID[0]).__contains__('velocity')
        if self.modelSetupPar.modelID[0]=='Tabdata': modelProvided=False
        if modelProvided:
            if self.VeloInpTypeCBox.count()==3:
                self.VeloInpTypeCBox.addItem('Model')
            self.VeloInpFrameLabel.setStyleSheet('color:black')
        else:
            if self.VeloInpTypeCBox.count()==4:
                self.VeloInpTypeCBox.removeItem(3)
            self.VeloInpFrameLabel.setStyleSheet('color:red')
        if self.modelSetupPar.variable['abundance']['isFunction']:
            if (self.modelSetupPar.variable['abundance']['functionID']=='scalarConst'):
                ii = self.modelSetupPar.variable['abundance']['functionParamName'].index('val')
                self.AbunValueLedit.setText(str(self.modelSetupPar.variable['abundance']['functionParamValue'][ii]))
                self.AbunInpTypeCBox.setCurrentIndex(0)
                self.setLabelsAbunField(0,False) 

            if (self.modelSetupPar.variable['abundance']['functionID']=='scalarPowerR'):
                ii = self.modelSetupPar.variable['abundance']['functionParamName'].index('exponent')
                self.AbunFTempLedit.setText(str(self.modelSetupPar.variable['abundance']['functionParamValue'][ii]))
                R0    = str2float(self.LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
                powex = self.modelSetupPar.variable['abundance']['functionParamValue'][ii]
                ii = self.modelSetupPar.variable['abundance']['functionParamName'].index('factor')
                self.AbunValueLedit.setText(str(self.modelSetupPar.variable['abundance']['functionParamValue'][ii]/(R0**(-powex))))
                self.AbunInpTypeCBox.setCurrentIndex(1)
                self.setLabelsAbunField(1,False) 

            if (self.modelSetupPar.variable['abundance']['functionID']=='scalarTabdata'):
                if self.modelSetupPar.abunRegion:
                    self.AbunInpTypeCBox.setCurrentIndex(2)
                    self.setLabelsAbunField(2,False)
                else:
                    self.AbunInpTypeCBox.setCurrentIndex(3)
                    self.setLabelsAbunField(3,False)
                ii = self.modelSetupPar.variable['abundance']['functionParamName'].index('filename')
                self.AbunValueLedit.setText(self.modelSetupPar.variable['abundance']['functionParamValue'][ii])
        else:
            if modellib.isCurrentResultFunction('abundance'):modellib.unsetFunction("abundance")
        if self.modelSetupPar.variable['density']['isFunction']:
            if (self.modelSetupPar.variable['density']['functionID']=='scalarConst'):
                self.DensValueLedit.setVisible(True) 
                self.DensFNameLedit.setVisible(False) 
                ii = self.modelSetupPar.variable['density']['functionParamName'].index('val')
                self.DensValueLedit.setText(str(self.modelSetupPar.variable['density']['functionParamValue'][ii]))
                self.DensInpTypeCBox.setCurrentIndex(0)
                self.setLabelsDensField(0,False)

            if (self.modelSetupPar.variable['density']['functionID']=='scalarPowerR'):
                ii = self.modelSetupPar.variable['density']['functionParamName'].index('exponent')
                self.DensFNameLedit.setText(str(self.modelSetupPar.variable['density']['functionParamValue'][ii]))
                R0    = str2float(self.LRTminScaleLedit.text(), 'minScale (Code parameters / Grid)') * 1.496e11
                powex = self.modelSetupPar.variable['density']['functionParamValue'][ii]
                ii = self.modelSetupPar.variable['density']['functionParamName'].index('factor')
                self.DensValueLedit.setText(str(self.modelSetupPar.variable['density']['functionParamValue'][ii]/(R0**(-powex))))
                self.DensInpTypeCBox.setCurrentIndex(1)
                self.setLabelsDensField(1,False)
            
            if (self.modelSetupPar.variable['density']['functionID']=='scalarTabdata'):
                ii = self.modelSetupPar.variable['density']['functionParamName'].index('filename')
                self.DensInpTypeCBox.setCurrentIndex(2)
                self.setLabelsDensField(2,False)
                self.DensValueLedit.setText(self.modelSetupPar.variable['density']['functionParamValue'][ii])

        else:
            if modellib.isCurrentResultFunction('density'):modellib.unsetFunction("density")
            self.DensInpTypeCBox.setCurrentIndex(3)
            self.setLabelsDensField(3,False)
        
        if self.modelSetupPar.variable['doppler']['isFunction']:
            if (self.modelSetupPar.variable['doppler']['functionID']=='scalarConst'):
                ii = self.modelSetupPar.variable['doppler']['functionParamName'].index('val')
                self.DopplerValueLedit.setText(str(self.modelSetupPar.variable['doppler']['functionParamValue'][ii]))
                self.DopplerInpTypeCBox.setCurrentIndex(0)
                self.setLabelsDopplerField(0)

            if (self.modelSetupPar.variable['doppler']['functionID']=='scalarPowerR'):
                ii = self.modelSetupPar.variable['doppler']['functionParamName'].index('exponent')
                self.DopplerFName.setText(str(self.modelSetupPar.variable['doppler']['functionParamValue'][ii]))
                R0    = str2float(self.LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
                powex = self.modelSetupPar.variable['doppler']['functionParamValue'][ii]
                ii = self.modelSetupPar.variable['doppler']['functionParamName'].index('factor')
                self.DopplerValueLedit.setText(str(self.modelSetupPar.variable['doppler']['functionParamValue'][ii]/(R0**(-powex))))
                self.DopplerInpTypeCBox.setCurrentIndex(1)
                self.setLabelsDopplerField(1)
            
            if (self.modelSetupPar.variable['doppler']['functionID']=='scalarTabdata'):
                self.DopplerInpTypeCBox.setCurrentIndex(2)
                self.setLabelsDopplerField(2,False)
                ii = self.modelSetupPar.variable['doppler']['functionParamName'].index('filename')
                self.DopplerValueLedit.setText(self.modelSetupPar.variable['doppler']['functionParamValue'][ii])
        
        else:
            if modellib.isCurrentResultFunction('doppler'):modellib.unsetFunction("doppler")

        if self.modelSetupPar.variable['tdust']['isFunction']:
            if (self.modelSetupPar.variable['tdust']['functionID']=='scalarConst'):
                ii = self.modelSetupPar.variable['tdust']['functionParamName'].index('val')
                self.TdustValueLedit.setText(str(self.modelSetupPar.variable['tdust']['functionParamValue'][ii]))
                self.TdustInpTypeCBox.setCurrentIndex(0)
                self.setLabelsDustTempField(0,False)

            if (self.modelSetupPar.variable['tdust']['functionID']=='scalarPowerR'):
                ii = self.modelSetupPar.variable['tdust']['functionParamName'].index('exponent')
                self.TdustFNameLedit.setText(str(self.modelSetupPar.variable['tdust']['functionParamValue'][ii]))
                R0    = str2float(self.LRTminScaleLedit.text(), 'minScale (Code Parameters / Grid)') * 1.496e11
                powex = self.modelSetupPar.variable['tdust']['functionParamValue'][ii]
                ii = self.modelSetupPar.variable['tdust']['functionParamName'].index('factor')
                self.TdustValueLedit.setText(str(self.modelSetupPar.variable['tdust']['functionParamValue'][ii]/(R0**(-powex))))
                self.TdustInpTypeCBox.setCurrentIndex(1)
                self.setLabelsDustTempField(1,False)
            
            if (self.modelSetupPar.variable['tdust']['functionID']=='scalarTabdata'):
                self.TdustInpTypeCBox.setCurrentIndex(2)
                self.setLabelsDustTempField(2,False)
                ii = self.modelSetupPar.variable['tdust']['functionParamName'].index('filename')
                self.TdustValueLedit.setText(self.modelSetupPar.variable['tdust']['functionParamValue'][ii])
        
            if (self.modelSetupPar.variable['tdust']['functionID']=='setTdustIdentTemp'):
                self.TdustInpTypeCBox.setCurrentIndex(3)
                self.setLabelsDustTempField(3,False)

        else:
            if modellib.isCurrentResultFunction('tdust'):modellib.unsetFunction("tdust")
            self.TdustInpTypeCBox.setCurrentIndex(4)
            self.setLabelsDustTempField(4,False)
        
        if self.modelSetupPar.variable['bmag']['isFunction']:
            if (self.modelSetupPar.variable['bmag']['functionID']=='vectorConstXYZ'):
                ii = self.modelSetupPar.variable['bmag']['functionParamName'].index('valX')
                if self.modelSetupPar.variable['bmag']['functionParamValue'][ii]!=0.:
                    self.MagfieldValueLedit.setText(str(self.modelSetupPar.variable['bmag']['functionParamValue'][ii]))
                    self.MagfieldInpTypeCBox.setCurrentIndex(0)
                    self.setLabelsMagfield(0,False) 
                else: 
                    ii = self.modelSetupPar.variable['bmag']['functionParamName'].index('valY')
                    if self.modelSetupPar.variable['bmag']['functionParamValue'][ii]!=0.:
                        self.MagfieldValueLedit.setText(str(self.modelSetupPar.variable['bmag']['functionParamValue'][ii]))
                        self.MagfieldInpTypeCBox.setCurrentIndex(1)
                        self.setLabelsMagfield(1,False) 
                    else:
                        ii = self.modelSetupPar.variable['bmag']['functionParamName'].index('valZ')
                        if self.modelSetupPar.variable['bmag']['functionParamValue'][ii]!=0.:
                            self.MagfieldValueLedit.setText(str(self.modelSetupPar.variable['bmag']['functionParamValue'][ii]))
                            self.MagfieldInpTypeCBox.setCurrentIndex(2)
                            self.setLabelsMagfield(2,False) 
                        else:
                            self.MagfieldValueLedit.setText(str(self.modelSetupPar.variable['bmag']['functionParamValue'][ii]))
                            self.MagfieldInpTypeCBox.setCurrentIndex(0)
                            self.setLabelsMagfield(0,False) 


            if (self.modelSetupPar.variable['bmag']['functionID']=='VectorDipole'):
                ii = self.modelSetupPar.variable['bmag']['functionParamName'].index('lowerR')
                self.MagfieldFNameLedit.setText(str(self.modelSetupPar.variable['bmag']['functionParamValue'][ii]))
                ii = self.modelSetupPar.variable['bmag']['functionParamName'].index('mDipole')
                self.MagfieldValueLedit.setText(str(self.modelSetupPar.variable['bmag']['functionParamValue'][ii]))
                self.MagfieldInpTypeCBox.setCurrentIndex(3)
                self.setLabelsMagfield(3,False) 
            
            if (self.modelSetupPar.variable['bmag']['functionID']=='VectorTabdata'):
                self.MagfieldInpTypeCBox.setCurrentIndex(4)
                self.setLabelsMagfield(4,False) 
                ii = self.modelSetupPar.variable['bmag']['functionParamName'].index('filename')
                self.MagfieldFNameLedit.setText(str(self.modelSetupPar.variable['bmag']['functionParamValue'][ii]))

        else:
            if modellib.isCurrentResultFunction('bmag'):modellib.unsetFunction("bmag")
            self.MagfieldInpTypeCBox.setCurrentIndex(5)
            self.setLabelsMagfield(5,False) 
        
        if self.modelSetupPar.variable['temperature']['isFunction']:
            if (self.modelSetupPar.variable['temperature']['functionID']=='scalarConst'):
                ii = self.modelSetupPar.variable['temperature']['functionParamName'].index('val')
                self.TempValueLedit.setText(str(self.modelSetupPar.variable['temperature']['functionParamValue'][ii]))
                self.TempInpTypeCBox.setCurrentIndex(0)
                self.setLabelsTempField(0,False)

            if (self.modelSetupPar.variable['temperature']['functionID']=='scalarPowerR'):
                ii = self.modelSetupPar.variable['temperature']['functionParamName'].index('exponent')
                self.TempFNameLedit.setText(str(self.modelSetupPar.variable['temperature']['functionParamValue'][ii]))
                R0    = str2float(self.LRTminScaleLedit.text(), 'minScale (Code parameters / Grid)') * 1.496e11
                powex = self.modelSetupPar.variable['temperature']['functionParamValue'][ii]
                ii = self.modelSetupPar.variable['temperature']['functionParamName'].index('factor')
                self.TempValueLedit.setText(str(self.modelSetupPar.variable['temperature']['functionParamValue'][ii]/(R0**(-powex))))
                self.TempInpTypeCBox.setCurrentIndex(1)
                self.setLabelsTempField(1,False)

            if (self.modelSetupPar.variable['temperature']['functionID']=='scalarTabdata'):
                self.TempInpTypeCBox.setCurrentIndex(2)
                self.setLabelsTempField(2,False)
                ii = self.modelSetupPar.variable['temperature']['functionParamName'].index('filename')
                self.TempValueLedit.setText(self.modelSetupPar.variable['temperature']['functionParamValue'][ii])
            
            if (self.modelSetupPar.variable['temperature']['functionID']=='setTempIdentTdust'):
                self.TempInpTypeCBox.setCurrentIndex(3)
                self.setLabelsTempField(3,False)

        else:
            if modellib.isCurrentResultFunction('temperature'):modellib.unsetFunction("temperature")
            self.TempInpTypeCBox.setCurrentIndex(4)
            self.setLabelsTempField(4,False)
        
        if self.modelSetupPar.variable['velocity']['isFunction']:
            if (self.modelSetupPar.variable['velocity']['functionID']=='vectorConstR'):
                self.VeloInpTypeCBox.setCurrentIndex(0)
                self.setLabelsVeloField(0,False) 
                ii = self.modelSetupPar.variable['velocity']['functionParamName'].index('val')
                self.VeloValueLedit.setText(str(self.modelSetupPar.variable['velocity']['functionParamValue'][ii]))
               

            if (self.modelSetupPar.variable['velocity']['functionID']=='vectorRadialPowerR'):
                self.VeloInpTypeCBox.setCurrentIndex(1)
                self.setLabelsVeloField(1,False)
                ii = self.modelSetupPar.variable['velocity']['functionParamName'].index('exponent')
                self.VeloFNameLedit.setText(str(self.modelSetupPar.variable['velocity']['functionParamValue'][ii]))
                powex = self.modelSetupPar.variable['velocity']['functionParamValue'][ii]
                R0    = str2float(self.LRTminScaleLedit.text(), 'minScale (Code parameters / Grid)') * 1.496e11
                ii = self.modelSetupPar.variable['velocity']['functionParamName'].index('factor')
                self.VeloValueLedit.setText(str(self.modelSetupPar.variable['velocity']['functionParamValue'][ii]/(R0**(-powex))))

            
            if (self.modelSetupPar.variable['velocity']['functionID']=='vectorTabdata'):
                self.VeloInpTypeCBox.setCurrentIndex(2)
                self.setLabelsVeloField(2,False) 
                ii = self.modelSetupPar.variable['velocity']['functionParamName'].index('filename')
                self.VeloValueLedit.setText(str(self.modelSetupPar.variable['velocity']['functionParamValue'][ii]))

        else:
            if modellib.isCurrentResultFunction('velocity'):modellib.unsetFunction("velocity")
            self.VeloInpTypeCBox.setCurrentIndex(3)
            self.setLabelsVeloField(3,False) 
        
       
        self.readMoldatfile()
        self.inOpeningModel = False

        return True 


    def saveModelas(self):
        """
        Function to save the current Model with a new name
        """

        doneWithDialogs=False

        while not doneWithDialogs:
            new_modelDir = QtGui.QFileDialog.getSaveFileName(None, 'Save Model as',  options=QtGui.QFileDialog.DontUseNativeDialog)
            new_modelDir = str(new_modelDir)
            if new_modelDir!="":
                if (new_modelDir[-4:]!='.mdl'):
                    new_modelDir = new_modelDir+'.mdl'

                if os.path.exists(new_modelDir):

                    reply = QtGui.QMessageBox.question(None, 'Save Model as', new_modelDir+' already exists. '+\
                            'Do you want to overwrite it?',
                            QtGui.QMessageBox.Yes | QtGui.QMessageBox.No | QtGui.QMessageBox.Cancel)
                    if reply == QtGui.QMessageBox.Yes:
                        try:
                            sp.Popen(['cp -rv ./* '+new_modelDir],  stdout=sp.PIPE, shell=True).wait()
                            os.chdir(new_modelDir)
                        except:
                            reply = QtGui.QMessageBox.critical(None,"I/O Error", 'Cannot write '+new_modelDir+'. Model cannot be saved.',\
                                    QtGui.QMessageBox.Ok)

                        doneWithDialogs=True

                    elif reply == QtGui.QMessageBox.No:
                        doneWithDialogs=False
                    else:
                        doneWithDialogs=True

                else:
                    try:
                        os.mkdir(new_modelDir)
                        sp.Popen(['cp -rv ./* '+new_modelDir],  stdout=sp.PIPE, shell=True).wait()
                        os.chdir(new_modelDir)
                    except:
                        reply = QtGui.QMessageBox.critical(None,"I/O Error", 'Cannot write '+new_modelDir+'. Model cannot be saved.',\
                                QtGui.QMessageBox.Ok)

                    doneWithDialogs=True
            else:
                doneWithDialogs=True

        self.modelDir = new_modelDir
        self.saveModel()


    def saveModel(self):

        self.modelSetupPar = modelSetupPar(par=self.par, img=self.img)
        
        readLRTParsOK = self.readOutLRTParams(toSaveModel=True)
        if not readLRTParsOK:
            return
        readParamLabelsOK = self.readOutModelParamLabels()
        if not readParamLabelsOK:
            return
        readModelVarOK = self.readOutModelVarValues()
        if not readModelVarOK:
            return

        self.modelSetupPar.writeModelSetupFile(abunModel=self.RegionAbunParams)
    
    def changeModel(self,ModelID):

        oldModelID = modellib.getCurrentModelID()
        newModelID = modellib.getModelIDs()[ModelID] 
                
        self.inOpeningModel = True
        opacfile_exists = True
        if (newModelID=='DDN01'):
            opacfname = str(self.TdustDustopacFileLedit.text()).strip()
            if not os.path.exists(opacfname):
                opacfile_exists = False
                self.doFinalizeConfig= False
                
        if (newModelID=='Tabdata'):
            self.doFinalizeConfig= False

        if modellib.isTdustIdentTemp():
            modellib.unsetTdustIdentTemp() 
        if modellib.isTempIdentTdust():
            modellib.unsetTempIdentTdust() 

        if (newModelID=='user'):
            print 'skipped'

        elif (newModelID=='Tabdata'):
            self.VisualizeMainWindow.CurrentModelID = ModelID               
            self.setModelParamLabels(ModelID, True)

            if self.AbunInpTypeCBox.count()==5:
                self.AbunInpTypeCBox.removeItem(4)
            self.AbunInpFrameLabel.setStyleSheet('color:red')
            self.AbunInpTypeCBox.setCurrentIndex(3)
            self.setLabelsAbunField(3, False)
            
            if self.DensInpTypeCBox.count()==4:
                self.DensInpTypeCBox.removeItem(3)
            self.DensInpFrameLabel.setStyleSheet('color:red')
            self.DensInpTypeCBox.setCurrentIndex(2)
            self.setLabelsDensField(2,False)
            
            if self.DopplerInpTypeCBox.count()==4:
                self.DopplerInpTypeCBox.removeItem(3)
            self.DopplerInpFrameLabel.setStyleSheet('color:red')
            self.DopplerInpTypeCBox.setCurrentIndex(2)
            self.setLabelsDopplerField(2,False)
           
            if self.TdustInpTypeCBox.count()==5:
                self.TdustInpTypeCBox.removeItem(4)
            self.TdustInpFrameLabel.setStyleSheet('color:red')
            self.TdustInpTypeCBox.setCurrentIndex(2)
            self.setLabelsDustTempField(2,False)
            
            if self.MagfieldInpTypeCBox.count()==6:
                self.MagfieldInpTypeCBox.removeItem(5)
            self.MagfieldInpFrameLabel.setStyleSheet('color:red')
            self.MagfieldInpTypeCBox.setCurrentIndex(4)
            self.setLabelsMagfield(4, False)
            
            if self.TempInpTypeCBox.count()==5:
                self.TempInpTypeCBox.removeItem(4)
            self.TempInpFrameLabel.setStyleSheet('color:red')
            self.TempInpTypeCBox.setCurrentIndex(2)
            self.setLabelsTempField(2,False)
            
            self.VeloInpFrameLabel.setStyleSheet('color:red')
            if self.VeloInpTypeCBox.count()==4:
                self.VeloInpTypeCBox.removeItem(3)
            self.VeloInpTypeCBox.setCurrentIndex(2)
            self.setLabelsVeloField(2,False)
            
            self.VisualizeMainWindow.mplFigureWidget2D.canvas.PlottedModelID = ModelID
                
            if self.inOpeningModel:
                if not opacfile_exists:
                    self.inOpeningModel=False
                    self.doFinalizeConfig= False

            else:
                #self.VisualizeMainWindow.resetLContGridParams()
                self.VisualizeMainWindow.reDrawContours()
                self.VisualizeMainWindow.updateFigure1DNewYRange()

        else:
            self.VisualizeMainWindow.CurrentModelID = ModelID               
            self.setModelParamLabels(ModelID, True)
            
            modelProvided = modellib.getModelResultIDs(newModelID).__contains__('abundance')
            if modelProvided:
                if self.AbunInpTypeCBox.count()==4:
                    self.AbunInpTypeCBox.addItem('Model')
                self.AbunInpFrameLabel.setStyleSheet('color:black')
                self.AbunInpTypeCBox.setCurrentIndex(4)
                self.setLabelsAbunField(4, False)
            else:
                if self.AbunInpTypeCBox.count()==5:
                    self.AbunInpTypeCBox.removeItem(4)
                self.AbunInpFrameLabel.setStyleSheet('color:red')
                self.AbunInpTypeCBox.setCurrentIndex(0)
                self.setLabelsAbunField(0, False)

            modelProvided = modellib.getModelResultIDs(newModelID).__contains__('density')
            if modelProvided:
                if self.DensInpTypeCBox.count()==3:
                    self.DensInpTypeCBox.addItem('Model')
                self.DensInpFrameLabel.setStyleSheet('color:black')
                self.DensInpTypeCBox.setCurrentIndex(3)
                self.setLabelsDensField(3,False)
            else:
                if self.DensInpTypeCBox.count()==4:
                    self.DensInpTypeCBox.removeItem(3)
                self.DensInpFrameLabel.setStyleSheet('color:red')
                self.DensInpTypeCBox.setCurrentIndex(0)
                self.setLabelsDensField(0,False)

            modelProvided = modellib.getModelResultIDs(newModelID).__contains__('doppler')
            if modelProvided:
                if self.DopplerInpTypeCBox.count()==3:
                    self.DopplerInpTypeCBox.addItem('Model')
                self.DopplerInpFrameLabel.setStyleSheet('color:black')
                self.DopplerInpTypeCBox.setCurrentIndex(3)
                self.setLabelsDopplerField(3,False)
            else:
                if self.DopplerInpTypeCBox.count()==4:
                    self.DopplerInpTypeCBox.removeItem(3)
                self.DopplerInpFrameLabel.setStyleSheet('color:red')
                self.DopplerInpTypeCBox.setCurrentIndex(0)
                self.setLabelsDopplerField(0,False)

            modelProvided = modellib.getModelResultIDs(newModelID).__contains__('tdust')
            if modelProvided:
                if self.TdustInpTypeCBox.count()==4:
                    self.TdustInpTypeCBox.addItem('Model')

                self.TdustInpFrameLabel.setStyleSheet('color:black')

                self.TdustInpTypeCBox.setCurrentIndex(4)
                self.setLabelsDustTempField(4,False)

            else:
                if self.TdustInpTypeCBox.count()==5:
                    self.TdustInpTypeCBox.removeItem(4)

                self.TdustInpFrameLabel.setStyleSheet('color:red')
                if modellib.getModelResultIDs(newModelID).__contains__('temperature'):
                    self.TdustInpTypeCBox.setCurrentIndex(3)
                    self.setLabelsDustTempField(3,False)
                else: 
                    self.TdustInpTypeCBox.setCurrentIndex(0)
                    self.setLabelsDustTempField(0,False)

            modelProvided = modellib.getModelResultIDs(newModelID).__contains__('bmag')
            if modelProvided:
                if self.MagfieldInpTypeCBox.count()==5:
                    self.MagfieldInpTypeCBox.addItem('Model')
                self.MagfieldInpFrameLabel.setStyleSheet('color:black')
                self.MagfieldInpTypeCBox.setCurrentIndex(5)
                self.setLabelsMagfield(5,False)
            else:
                if self.MagfieldInpTypeCBox.count()==6:
                    self.MagfieldInpTypeCBox.removeItem(5)
                self.MagfieldInpFrameLabel.setStyleSheet('color:red')
                self.MagfieldInpTypeCBox.setCurrentIndex(0)
                self.setLabelsMagfield(0, False)

            modelProvided = modellib.getModelResultIDs(newModelID).__contains__('temperature')
            if modelProvided:
                if self.TempInpTypeCBox.count()==4:
                    self.TempInpTypeCBox.addItem('Model')
                self.TempInpFrameLabel.setStyleSheet('color:black')
                self.TempInpTypeCBox.setCurrentIndex(4)
                self.setLabelsTempField(4,False)
            else:
                self.TempInpFrameLabel.setStyleSheet('color:red')
                if self.TempInpTypeCBox.count()==5:
                    self.TempInpTypeCBox.removeItem(4)
                if modellib.getModelResultIDs(newModelID).__contains__('tdust'):
                    self.TempInpTypeCBox.setCurrentIndex(3)
                    self.setLabelsTempField(3,False)
                else: 
                    self.TempInpTypeCBox.setCurrentIndex(0)
                    self.setLabelsTempField(0,False)
            
            modelProvided = modellib.getModelResultIDs(newModelID).__contains__('velocity')
            if modelProvided:
                if self.VeloInpTypeCBox.count()==3:
                    self.VeloInpTypeCBox.addItem('Model')
                self.VeloInpFrameLabel.setStyleSheet('color:black')
                self.VeloInpTypeCBox.setCurrentIndex(3)
                self.setLabelsVeloField(3,False)
            else:
                self.VeloInpFrameLabel.setStyleSheet('color:red')
                if self.VeloInpTypeCBox.count()==4:
                    self.VeloInpTypeCBox.removeItem(3)
                self.VeloInpTypeCBox.setCurrentIndex(0)
                self.setLabelsVeloField(0,False)

            readModelVarOK = self.readOutModelVarValues()
            if not readModelVarOK:
                return


            self.VisualizeMainWindow.mplFigureWidget2D.canvas.PlottedModelID = ModelID
            
            self.inOpeningModel=False
            if newModelID=='DDN01':
                if not opacfile_exists:
                    reply = QtGui.QMessageBox.critical(None,"Error", 'Dust opacity file does not exists. Please, select a valid dust opacity file.',\
                            QtGui.QMessageBox.Ok)

                    self.inOpeningModel=False
                    self.opacfile_exists = True
                    self.doFinalizeConfig= True

            else:

                #self.VisualizeMainWindow.resetLContGridParams()
                self.VisualizeMainWindow.reDrawContours()
                self.VisualizeMainWindow.updateFigure1DNewYRange()
               
        if (newModelID=='Tabdata'):
            self.doFinalizeConfig= True


    def changeParams(self, defaultPar=True):
   
        readParamLabelsOK = self.readOutModelParamLabels()
        if not readParamLabelsOK:
            return

        readModelVarOK = self.readOutModelVarValues()
        if not readModelVarOK:
            return

        if self.configParams.autoupdateFigure:
            self.VisualizeMainWindow.mplFigureWidget2D.canvas.updateModelVars(self.VisualizeMainWindow,defaultPar=defaultPar)
            #self.VisualizeMainWindow.resetLContGridParams()
            self.VisualizeMainWindow.reDrawContours()
            self.VisualizeMainWindow.updateFigure1DNewYRange()
        
    
    def readOutModelVarValues(self):


        if (self.AbunInpTypeCBox.currentIndex()==0):
            val0  = str2float(self.AbunValueLedit.text(), 'Abundance value')
            if val0==None: return False
            modellib.setFunction("abundance", "scalarConst")
            modellib.setFunctionParamDouble("abundance", "val", val0)
            self.modelSetupPar.abunRegion = False
            self.functionSetAbun = True
            log.debug('Abundance : scalarConst')
            log.debug('     value = '+str(val0))

        if (self.AbunInpTypeCBox.currentIndex()==1):
            powex = str2float(self.AbunFTempLedit.text(), 'Abundance power exponent') 
            if powex==None: return False
            val0  = str2float(self.AbunValueLedit.text(), 'Abundance value at Rin')
            if val0==None: return False
            R0    = str2float(self.LRTminScaleLedit.text(), 'minScale') * 1.496e11
            if R0==None: return False
            modellib.setFunction("abundance", "scalarPowerR")
            modellib.setFunctionParamDouble("abundance", "factor", val0*R0**(-powex))
            modellib.setFunctionParamDouble("abundance", "exponent", powex)
            modellib.setFunctionParamDouble("abundance", "offset", 0.)
            self.modelSetupPar.abunRegion = False
            self.functionSetAbun = True
            log.debug('Abundance : scalarPowerR')
            log.debug('     rin         = '+str(R0))
            log.debug('     value (rin) = '+str(val0))
            log.debug('     powex       = '+str(powex))
        
        if (self.AbunInpTypeCBox.currentIndex()==2):
            fname = str(self.AbunValueLedit.text()).strip()
            if fname=='':
                log.exception('Abundance tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Abundance table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Abundance tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Abundance table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False

            modellib.setFunction('abundance', 'scalarTabdata')
            modellib.setFunctionParamString('abundance', 'filename', fname)
            modellib.setFunctionParamDouble('abundance', 'defValue', 1e-10)
            self.modelSetupPar.abunRegion = True
            self.functionSetAbun = True
            log.debug('Abundance : scalarTabdata')
            log.debug('     filename    = '+fname)
            log.debug('     defValue    = 1e-10')


        if (self.AbunInpTypeCBox.currentIndex()==3):
            fname = str(self.AbunValueLedit.text()).strip()
            if fname=='':
                log.exception('Abundance tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Abundance table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Abundance tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Abundance table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False

            modellib.setFunction('abundance', 'scalarTabdata')
            modellib.setFunctionParamString('abundance', 'filename', fname)
            modellib.setFunctionParamDouble('abundance', 'defValue', 1e-10)
            self.modelSetupPar.abunRegion = False
            self.functionSetAbun = True
            log.debug('Abundance : scalarTabdata')
            log.debug('     filename    = '+fname)
            log.debug('     defValue    = 1e-10')


        if (self.DensInpTypeCBox.currentIndex()==0):
            val0 = str2float(self.DensValueLedit.text(), 'Density value')
            val0 = val0 * 1e6
            log.debug('Density : scalarConst')
            log.debug('     value = '+str(val0))
            if val0==None:
                return False
            modellib.setFunction("density", "scalarConst")
            modellib.setFunctionParamDouble("density", "val", val0)
            self.functionSetDens = True
            
        if (self.DensInpTypeCBox.currentIndex()==1):
            powex = str2float(self.DensFNameLedit.text(), 'Density power exponent') 
            if powex==None: return False
            val0  = str2float(self.DensValueLedit.text(), 'Density value at Rin')
            val0 = val0 * 1e6
            if val0==None: return False
            R0    = str2float(self.LRTminScaleLedit.text(), 'minScale') * 1.496e11
            if R0==None: return False

            modellib.setFunction("density", "scalarPowerR")
            modellib.setFunctionParamDouble("density", "factor", val0*R0**(-powex))
            modellib.setFunctionParamDouble("density", "exponent", powex)
            modellib.setFunctionParamDouble("density", "offset", 0.)
            log.debug('Density : scalarPowerR')
            log.debug('     rin         = '+str(R0))
            log.debug('     value (rin) = '+str(val0))
            log.debug('     powex       = '+str(powex))
            self.functionSetDens = True
        
        if (self.DensInpTypeCBox.currentIndex()==2):
            fname = str(self.DensValueLedit.text()).strip()
            if fname=='':
                log.exception('Density tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Density table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Density tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Density table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False
            modellib.setFunction('density', 'scalarTabdata')
            modellib.setFunctionParamString('density', 'filename', fname)
            modellib.setFunctionParamDouble('density', 'defValue', 1e-3)
            self.functionSetAbun = True
            log.debug('Density : scalarTabdata')
            log.debug('     filename    = '+fname)
            log.debug('     defValue    = 1e-3')
        
        if (self.DensInpTypeCBox.currentIndex()==3):
            log.debug('Density : model')
            if modellib.isCurrentResultFunction('density'):
                modellib.unsetFunction("density")
                self.functionSetDens = False

        
        if (self.DopplerInpTypeCBox.currentIndex()==0):
            val0  = str2float(self.DopplerValueLedit.text(), 'Doppler value')
            if val0==None: return False
            modellib.setFunction("doppler", "scalarConst")
            modellib.setFunctionParamDouble("doppler", "val", val0)
            self.functionSetDoppler = True
            log.debug('Doppler : scalarConst')
            log.debug('     value = '+str(val0))

        if (self.DopplerInpTypeCBox.currentIndex()==1):
            powex = str2float(self.DopplerFNameLedit.text(), 'Doppler power exponent') 
            if powex==None: return False
            val0  = str2float(self.DopplerValueLedit.text(), 'Doppler value at Rin')
            if val0==None: return False
            R0    = str2float(self.LRTminScaleLedit.text(), 'minScale') * 1.496e11
            if R0==None: return False
            modellib.setFunction("doppler", "scalarPowerR")
            modellib.setFunctionParamDouble("doppler", "factor", val0*R0**(-powex))
            modellib.setFunctionParamDouble("doppler", "exponent", powex)
            modellib.setFunctionParamDouble("doppler", "offset", 0.)
            self.functionSetDoppler = True
            log.debug('Doppler : scalarPowerR')
            log.debug('     rin         = '+str(R0))
            log.debug('     value (rin) = '+str(val0))
            log.debug('     powex       = '+str(powex))
        
        if (self.DopplerInpTypeCBox.currentIndex()==3):
            fname = str(self.DopplerValueLedit.text()).strip()
            if fname=='':
                log.exception('Doppler tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Doppler table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Doppler tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Doppler table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False
            modellib.setFunction('doppler', 'scalarTabdata')
            modellib.setFunctionParamString('doppler', 'filename', fname)
            modellib.setFunctionParamDouble('doppler', 'defValue', 1e-30)
            self.functionSetAbun = True
            log.debug('Doppler : scalarTabdata')
            log.debug('     filename    = '+fname)
            log.debug('     defValue    = 2.73')


        if (self.TdustInpTypeCBox.currentIndex()==0):
            if modellib.isTdustIdentTemp():
                modellib.unsetTdustIdentTemp()
            val0  = str2float(self.TdustValueLedit.text(), 'Dust temperature value')
            if val0==None: return False
            modellib.setFunction("tdust", "scalarConst")
            modellib.setFunctionParamDouble("tdust", "val", val0)
            self.functionSetDustTemp = True
            log.debug('Tdust : scalarConst')
            log.debug('     value = '+str(val0))

        if (self.TdustInpTypeCBox.currentIndex()==1):
            if modellib.isTdustIdentTemp():
                modellib.unsetTdustIdentTemp()
            powex = str2float(self.TdustFNameLedit.text(), 'Dust temperature power exponent') 
            if powex==None: return False
            val0  = str2float(self.TdustValueLedit.text(), 'Dust temperature value at Rin')
            if val0==None: return False
            R0    = str2float(self.LRTminScaleLedit.text(), 'minScale') * 1.496e11
            if R0==None: return False
            modellib.setFunction("tdust", "scalarPowerR")
            modellib.setFunctionParamDouble("tdust", "factor", val0*R0**(-powex))
            modellib.setFunctionParamDouble("tdust", "exponent", powex)
            modellib.setFunctionParamDouble("tdust", "offset", 0.)
            self.functionSetDustTemp = True
            log.debug('Tdust : scalarPowerR')
            log.debug('     rin         = '+str(R0))
            log.debug('     value (rin) = '+str(val0))
            log.debug('     powex       = '+str(powex))
        
        if (self.TdustInpTypeCBox.currentIndex()==2):
            fname = str(self.TdustValueLedit.text()).strip()
            if fname=='':
                log.exception('Dust temperature tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Dust temperature table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Dust temperature tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Dust temperature table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False

            if modellib.isTdustIdentTemp():
                modellib.unsetTdustIdentTemp()
            modellib.setFunction('tdust', 'scalarTabdata')
            modellib.setFunctionParamString('tdust', 'filename', fname)
            modellib.setFunctionParamDouble('tdust', 'defValue', 1e-30)
            self.functionSetAbun = True
            log.debug('Tdust : scalarTabdata')
            log.debug('     filename    = '+fname)
            log.debug('     defValue    = 2.73')

        if (self.TdustInpTypeCBox.currentIndex()==3):
            log.debug('Tdust : identTemp')
            if not modellib.isTdustIdentTemp():
                modellib.setTdustIdentTemp()
        
        if (self.TdustInpTypeCBox.currentIndex()==4):
            if modellib.isTdustIdentTemp():
                modellib.unsetTdustIdentTemp()
            if modellib.isCurrentResultFunction('tdust'): 
                modellib.unsetFunction("tdust")
            log.debug('Tdust : model')
                 


        if (self.MagfieldInpTypeCBox.currentIndex()==0):
            val0 = str2float(self.MagfieldValueLedit.text(), 'Magfield value')
            log.debug('Magfield : VectorConstXYZ (X)')
            log.debug('     value = '+str(val0))
            if val0==None:
                return False
            modellib.setFunction("bmag", "vectorConstXYZ")
            modellib.setFunctionParamDouble("bmag", "valX", val0)
            modellib.setFunctionParamDouble("bmag", "valY", 0.0)
            modellib.setFunctionParamDouble("bmag", "valZ", 0.0)
            self.functionSetMagfield = True
            

        if (self.MagfieldInpTypeCBox.currentIndex()==1):
            val0 = str2float(self.MagfieldValueLedit.text(), 'Magfield value')
            log.debug('Magfield : VectorConstXYZ (Y)')
            log.debug('     value = '+str(val0))
            if val0==None:
                return False
            modellib.setFunction("bmag", "vectorConstXYZ")
            modellib.setFunctionParamDouble("bmag", "valX", 0.0)
            modellib.setFunctionParamDouble("bmag", "valY", val0)
            modellib.setFunctionParamDouble("bmag", "valZ", 0.0)
            self.functionSetMagfield = True


        if (self.MagfieldInpTypeCBox.currentIndex()==2):
            val0 = str2float(self.MagfieldValueLedit.text(), 'Magfield value')
            log.debug('Magfield : VectorConstXYZ (Z)')
            log.debug('     value = '+str(val0))
            if val0==None:
                return False
            modellib.setFunction("bmag", "vectorConstXYZ")
            modellib.setFunctionParamDouble("bmag", "valX", 0.0)
            modellib.setFunctionParamDouble("bmag", "valY", 0.0)
            modellib.setFunctionParamDouble("bmag", "valZ", val0)
            self.functionSetMagfield = True

        if (self.MagfieldInpTypeCBox.currentIndex()==3):
            val0 = str2float(self.MagfieldValueLedit.text(), 'Dipole moment')
            val1 = str2float(self.MagfieldFNameLedit.text(), 'Rmin') 
            val1 = val1*1.496e11
            log.debug('Magfield : VectorDipole')
            log.debug('     mDipole = '+str(val0))
            log.debug('     lowerR = '+str(val1))
            if val0==None:
                return False
            modellib.setFunction("bmag", "vectorDipole")
            modellib.setFunctionParamDouble("bmag", "mDipole", val0)
            modellib.setFunctionParamDouble("bmag", "lowerR", val1)
            self.functionSetMagfield = True
        
        if (self.MagfieldInpTypeCBox.currentIndex()==4):
            fname = str(self.MagfieldValueLedit.text()).strip()
            if fname=='':
                log.exception('Magnetic field tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Magnetic field table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Magnetic field tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Magnetic field table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False
            modellib.setFunction('bmag', 'vectorTabdata')
            modellib.setFunctionParamString('bmag', 'filename', fname)
            self.functionSetAbun = True
            log.debug('Magfield : vectorTabdata')
            log.debug('     filename    = '+fname)
        
        if (self.MagfieldInpTypeCBox.currentIndex()==5):
            modelProvided = modellib.getModelResultIDs(modellib.getCurrentModelID()).__contains__('bmag')
            if modelProvided:
                log.debug('Magnetic Field : model')
                if modellib.isCurrentResultFunction('bmag'):
                    modellib.unsetFunction("bmag")
                    self.functionSetMagfield = False
            else:
                val0 = str2float(self.MagfieldValueLedit.text(), 'Magfield value')
                log.debug('Magfield : VectorConstXYZ (X)')
                log.debug('     value = '+str(val0))
                if val0==None:
                    return False
                modellib.setFunction("bmag", "vectorConstXYZ")
                modellib.setFunctionParamDouble("bmag", "valX", val0)
                modellib.setFunctionParamDouble("bmag", "valY", 0.0)
                modellib.setFunctionParamDouble("bmag", "valZ", 0.0)
                self.functionSetMagfield = True

        if (self.TempInpTypeCBox.currentIndex()==0):
            if modellib.isTempIdentTdust():
                modellib.unsetTempIdentTdust()
            
            val0 = str2float(self.TempValueLedit.text(), 'Temperature value')
            if val0==None:
                return False
            modellib.setFunction("temperature", "scalarConst")
            modellib.setFunctionParamDouble("temperature", "val", val0)
            log.debug('Temperature : scalarConst')
            log.debug('     value = '+str(val0))
            self.functionSetTemp = True
            
        if (self.TempInpTypeCBox.currentIndex()==1):
            if modellib.isTempIdentTdust():
                modellib.unsetTempIdentTdust()
            
            powex = str2float(self.TempFNameLedit.text(), 'Temperature power exponent') 
            if powex==None: return False
            val0  = str2float(self.TempValueLedit.text(), 'Temperature value at Rin')
            if val0==None: return False
            R0    = str2float(self.LRTminScaleLedit.text(), 'minScale') * 1.496e11
            if R0==None: return False

            modellib.setFunction("temperature", "scalarPowerR")
            modellib.setFunctionParamDouble("temperature", "factor", val0*R0**(-powex))
            modellib.setFunctionParamDouble("temperature", "exponent", powex)
            modellib.setFunctionParamDouble("temperature", "offset", 0.)
            
            log.debug('Temperature : scalarPowerR')
            log.debug('     rin         = '+str(R0))
            log.debug('     value (rin) = '+str(val0))
            log.debug('     powex       = '+str(powex))
            self.functionSetTemp = True

        if (self.TempInpTypeCBox.currentIndex()==2):
            fname = str(self.TempValueLedit.text()).strip()
            if fname=='':
                log.exception('Temperature tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Temperature table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Temperature tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Temperature table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False

            if modellib.isTempIdentTdust():
                modellib.unsetTempIdentTdust()
            
            modellib.setFunction('temperature', 'scalarTabdata')
            modellib.setFunctionParamString('temperature', 'filename', fname)
            modellib.setFunctionParamDouble('temperature', 'defValue', 2.73)
            self.functionSetTemp = True
            log.debug('Temperature : scalarTabdata')
            log.debug('     filename    = '+fname)
            log.debug('     defValue    = 2.73')
        
        if (self.TempInpTypeCBox.currentIndex()==3):
            log.debug('Temp : identTdust')
            if not modellib.isTempIdentTdust():
                modellib.setTempIdentTdust()

        if (self.TempInpTypeCBox.currentIndex()==4):
            if modellib.isTempIdentTdust():
                modellib.unsetTempIdentTdust()
            log.debug('Temperature : model')
            if modellib.isCurrentResultFunction('temperature'):
                modellib.unsetFunction("temperature")
                self.functionSetTemp = False



        if (self.VeloInpTypeCBox.currentIndex()==0):
            val0 = str2float(self.VeloValueLedit.text(), 'Velocity value')
            log.debug('Velo : VectorConstR')
            log.debug('     value = '+str(val0))
            if val0==None:
                return False
            modellib.setFunction("velocity", "vectorConstR")
            modellib.setFunctionParamDouble("velocity", "val", val0)
            self.functionSetVelo = True
            


        if (self.VeloInpTypeCBox.currentIndex()==1):
            powex = str2float(self.VeloFNameLedit.text(), 'Velocity power exp.') 
            if powex==None: return False
            val0  = str2float(self.VeloValueLedit.text(), 'Vr value at Rin')
            if val0==None: return False
            R0    = str2float(self.LRTminScaleLedit.text(), 'minScale') * 1.496e11
            if R0==None: return False

            modellib.setFunction("velocity", "vectorRadialPowerR")
            modellib.setFunctionParamDouble("velocity", "factor", val0*R0**(-powex))
            modellib.setFunctionParamDouble("velocity", "exponent", powex)
            modellib.setFunctionParamDouble("velocity", "offset", 0.)
            modellib.setFunctionParamDouble("velocity", "lowerR", R0)
            self.functionSetVelocity = True
            log.debug('Velocity : vectorRadialPowerR')
            log.debug('     rin         = '+str(R0))
            log.debug('     value (rin) = '+str(val0))
            log.debug('     powex       = '+str(powex))
            log.debug('     lowerR      = '+str(R0))

        
        if (self.VeloInpTypeCBox.currentIndex()==2):
            fname = str(self.VeloValueLedit.text()).strip()
            if fname=='':
                log.exception('Velocity tabel data file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Velocity table data file "'+fname+'" does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False
            else:
                if not os.path.exists(fname):
                    log.exception('Velocity tabel data file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Velocity table data file "'+fname+'" does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False

            modellib.setFunction('velocity', 'vectorTabdata')
            modellib.setFunctionParamString('velocity', 'filename', fname)
            self.functionSetVelocity = True
            log.debug('Velo : vectorTabdata')
            log.debug('     filename    = '+fname)
        
        if (self.VeloInpTypeCBox.currentIndex()==3):
            modelProvided = modellib.getModelResultIDs(modellib.getCurrentModelID()).__contains__('velocity')
            if modelProvided:
                log.debug('Velocity : model')
                if modellib.isCurrentResultFunction('velocity'):
                    modellib.unsetFunction("velocity")
                    self.functionSetVelo = False
            else:
                val0 = str2float(self.VeloValueLedit.text(), 'Velocity value')
                log.debug('Velo : VectorConstXYZ (X)')
                log.debug('     value = '+str(val0))
                if val0==None:
                    return False
                modellib.setFunction("velocity", "vectorConstXYZ")
                modellib.setFunctionParamDouble("velocity", "valX", val0)
                modellib.setFunctionParamDouble("velocity", "valY", 0.0)
                modellib.setFunctionParamDouble("velocity", "valZ", 0.0)
                self.functionSetVelo = True
        
        if self.doFinalizeConfig:
            modellib.finalizeConfiguration()
        return True
    def readOutModelParamLabels(self):


        ModelName = modellib.getCurrentModelID()
        ParamIDs = modellib.getParamIDs(ModelName)
        
        if ModelName=='Tabdata':
            ParamIDs=[]
        
        PID      = [i for i in ParamIDs]
        ParamIDs = []
        for i in range(len(PID)):
            if PID[i]!='dustopac_fname':
                ParamIDs.append(PID[i])

        log.debug('ModelName = '+str(ModelName))

        ModelParLabels = [self.ModelPar1Label,self.ModelPar2Label,self.ModelPar3Label,self.ModelPar4Label,
                          self.ModelPar5Label,self.ModelPar6Label,self.ModelPar7Label,self.ModelPar8Label,
                          self.ModelPar9Label,self.ModelPar10Label]

        ModelParLedits = [self.ModelPar1Ledit,self.ModelPar2Ledit,self.ModelPar3Ledit,self.ModelPar4Ledit,
                          self.ModelPar5Ledit,self.ModelPar6Ledit,self.ModelPar7Ledit,self.ModelPar8Ledit,
                          self.ModelPar9Ledit,self.ModelPar10Ledit]
        
        ModelParCBoxs = [self.ModelPar1CBox,self.ModelPar2CBox,self.ModelPar3CBox,self.ModelPar4CBox,
                          self.ModelPar5CBox,self.ModelPar6CBox,self.ModelPar7CBox,self.ModelPar8CBox,
                          self.ModelPar9CBox,self.ModelPar10CBox]


        for i in range(len(ParamIDs)):
            paramType = modellib.getParamType(ModelName, ParamIDs[i])
            if paramType=='enum':
                dum = ModelParCBoxs[i].currentIndex()
                modellib.setParamEnumIndex(ParamIDs[i], dum)
                log.debug(ParamIDs[i] + ' = '+str(ModelName)+' (EnumIndex) ')

            else:
                txt = str(ModelParLabels[i].text()).split('[')[0]
                log.debug(ParamIDs[i]+' = '+str(txt))

                dum = str2float(ModelParLedits[i].text(), txt)
                if dum!=None:
                    modellib.setParamDouble(ParamIDs[i], dum) 
                else:
                    return False
        
        if modellib.getParamIDs(ModelName).__contains__('dustopac_fname'):
            dum       = str(self.TdustDustopacFileLedit.text())
            dustopac_fname = dum.strip()
      
            log.debug('dustopac_fname = '+str(dustopac_fname))

            modellib.setParamString('dustopac_fname', dustopac_fname)
            if not os.path.exists(dustopac_fname):
              return False
        
        if self.doFinalizeConfig:
            modellib.finalizeConfiguration()
        return True 

    def setModelParamLabels(self, ModelID, default, setNewModel=True):
        
        ModelName = modellib.getModelIDs()[ModelID]
       
        if setNewModel:
            modellib.setCurrentModel(modellib.getModelIDs()[ModelID])
        
        self.ModelNameLabel.setText(ModelName)
        ParamIDs = modellib.getParamIDs(modellib.getModelIDs()[ModelID])
    
        if ModelName=='Tabdata':
            ParamIDs=[]

        PID      = [i for i in ParamIDs]
        ParamIDs = []
        for i in range(len(PID)):
            if PID[i]!='dustopac_fname':
                ParamIDs.append(PID[i])
        defparams = False
        if default:
            nc = natconst()
            modellib.setCurrentModel(ModelName)
            setDefaultParamValues(ModelName)
            if self.doFinalizeConfig:
                modellib.finalizeConfiguration()

        ModelParLabels = [self.ModelPar1Label,self.ModelPar2Label,self.ModelPar3Label,self.ModelPar4Label,
                          self.ModelPar5Label,self.ModelPar6Label,self.ModelPar7Label,self.ModelPar8Label,
                          self.ModelPar9Label,self.ModelPar10Label]

        ModelParLedits = [self.ModelPar1Ledit,self.ModelPar2Ledit,self.ModelPar3Ledit,self.ModelPar4Ledit,
                          self.ModelPar5Ledit,self.ModelPar6Ledit,self.ModelPar7Ledit,self.ModelPar8Ledit,
                          self.ModelPar9Ledit,self.ModelPar10Ledit]

        ModelParCBoxs = [self.ModelPar1CBox,self.ModelPar2CBox,self.ModelPar3CBox,self.ModelPar4CBox,
                          self.ModelPar5CBox,self.ModelPar6CBox,self.ModelPar7CBox,self.ModelPar8CBox,
                          self.ModelPar9CBox,self.ModelPar10CBox]
        
        ModelParFrames = [self.ModelPar1Frame,self.ModelPar2Frame,self.ModelPar3Frame,self.ModelPar4Frame,
                          self.ModelPar5Frame,self.ModelPar6Frame,self.ModelPar7Frame,self.ModelPar8Frame,
                          self.ModelPar9Frame,self.ModelPar10Frame]

        ModelInputField = []

        self.ModelPar2CBox.clear()

        for i in range(len(ParamIDs)):
            if ParamIDs[i]!='dustopac_fname':
                unit_txt = modellib.getParamUnit(ModelName, ParamIDs[i])
        
                if unit_txt.strip()!='':
                    ModelParLabels[i].setText(ParamIDs[i] + ' ['+unit_txt+']')
                else:
                    ModelParLabels[i].setText(ParamIDs[i])


                ModelParLabels[i].setToolTip(modellib.getParamDesc(ModelName,ParamIDs[i]))
                paramType = modellib.getParamType(ModelName, ParamIDs[i])
                if paramType=='enum':
                    
                    parval = modellib.getParamEnumValues(ModelName, ParamIDs[i])
                    txt    = [str(pval) for pval in parval]
                    ModelParCBoxs[i].clear()
                    ModelParCBoxs[i].addItems(txt)
                    ModelParCBoxs[i].setCurrentIndex(modellib.getParamEnumIndex(ParamIDs[i]))
                   
                    ModelParCBoxs[i].setVisible(True)
                    ModelParLedits[i].setVisible(False)
                    ModelParFrames[i].setVisible(True)


                    ModelInputField.append(ModelParCBoxs[i])
                else:
                    ModelParCBoxs[i].setVisible(False)
                    ModelParLedits[i].setVisible(True)
                    ModelParFrames[i].setVisible(True)
                    ModelParLedits[i].setText(str(modellib.getParamDouble(ParamIDs[i].strip()))) 
                    ModelInputField.append(ModelParLedits[i])

                

        nParams = len(ParamIDs)
        if ParamIDs.__contains__('dustopac_fname'):
            nParams = nParams-1

        if (nParams<10):
            for i in range(nParams,10):
                ModelParLabels[i].setText(' ')
                ModelParLabels[i].setToolTip('')       
                ModelParLedits[i].setVisible(False)
                ModelParCBoxs[i].setVisible(False)
                ModelParFrames[i].setVisible(False)
        
    def setLabelsDensField(self,index=None,updatePlots=True):

        if (index==0):
            self.DensFNameLabel.setVisible(False)
            self.DensFNameLedit.setVisible(False)            
            self.DensValueLabel.setVisible(True)
            self.DensValueLedit.setVisible(True)
            self.DensValueLedit.setText('1e4')
            self.DensValueLabel.setText('n [1/cm^3]:')
            self.DensFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.DensFNameLedit.setToolTip(str2ToolTipString(''))
                self.DensValueLedit.setToolTip(str2ToolTipString('Constant density value '))
                self.DensFNameBrowseButton.setToolTip(str2ToolTipString(''))
        if (index==1):
            self.DensFNameLabel.setVisible(True)
            self.DensFNameLedit.setVisible(True)
            self.DensValueLabel.setVisible(True)
            self.DensValueLedit.setVisible(True)
            self.DensFNameLedit.setText('-1.0')
            self.DensValueLedit.setText('1e4')
            self.DensFNameLabel.setText('Power Exp:')
            self.DensValueLabel.setText('n(Rin) [1/cm^3]:')
            self.DensFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.DensFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
                self.DensValueLedit.setToolTip(str2ToolTipString('Value of the density at Rin'))
                self.DensFNameBrowseButton.setToolTip(str2ToolTipString(''))
        if (index==2):
            self.DensFNameLabel.setVisible(False)
            self.DensFNameLedit.setVisible(False)
            self.DensValueLabel.setVisible(True)
            self.DensValueLedit.setVisible(True)
            self.DensValueLabel.setText('File:')
            self.DensValueLedit.setText('')
            self.DensFNameBrowseButton.setVisible(True)
            if self.showToolTipHelp:
                self.DensFNameLedit.setToolTip(str2ToolTipString(''))
                self.DensValueLedit.setToolTip(str2ToolTipString('Name of the file containing the gas density in an ARTIST Table format (For the description of the file format see the documentation)'))
                self.DensFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC, RADMC-3D)'))
        if (index==3):
            self.DensFNameLabel.setVisible(False)
            self.DensFNameLedit.setVisible(False)
            self.DensValueLabel.setVisible(False)
            self.DensValueLedit.setVisible(False)
            self.DensFNameBrowseButton.setVisible(False)
            self.DensFNameLedit.setToolTip(str2ToolTipString(''))
            self.DensValueLedit.setToolTip(str2ToolTipString(''))
            self.DensFNameBrowseButton.setToolTip(str2ToolTipString(''))
        
        if not self.inOpeningModel:
            if updatePlots:
                self.changeParams()
    def setLabelsTempField(self, index=None, updatePlots=True):

        if (index==0):
            self.TempFNameLabel.setVisible(False)
            self.TempFNameLedit.setVisible(False)            
            self.TempValueLabel.setVisible(True)
            self.TempValueLedit.setVisible(True)
            self.TempValueLabel.setText('T [K]:')
            self.TempValueLedit.setText('30.0')
            self.TempFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.TempFNameLedit.setToolTip('')
                self.TempValueLedit.setToolTip(str2ToolTipString('Constant kinetik temperature value '))
                self.TempFNameBrowseButton.setToolTip('')
        if (index==1):
            self.TempFNameLabel.setVisible(True)
            self.TempFNameLedit.setVisible(True)
            self.TempValueLabel.setVisible(True)
            self.TempValueLedit.setVisible(True)
            self.TempFNameLabel.setText('Power Exp:')
            self.TempValueLabel.setText('T(Rin) [K]:')
            self.TempValueLedit.setText('30.0')
            self.TempFNameLedit.setText('-1.')
            self.TempFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.TempFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
                self.TempValueLedit.setToolTip(str2ToolTipString('Value of the gas temperature at Rin'))
                self.TempFNameBrowseButton.setToolTip('')
        if (index==2):
            self.TempFNameLabel.setVisible(False)
            self.TempFNameLedit.setVisible(False)
            self.TempValueLabel.setVisible(True)
            self.TempValueLedit.setVisible(True)
            self.TempValueLabel.setText('File:')
            self.TempValueLedit.setText('')
            self.TempFNameBrowseButton.setVisible(True)
            if self.showToolTipHelp:
                self.TempFNameLedit.setToolTip('')
                self.TempValueLedit.setToolTip(str2ToolTipString('Name of the file containing the kinetik gas temperature in an ARTIST Table format (For the description of the file format see the documentation)'))
                self.TempFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC-3D)'))

        if (index==3):
            if self.TdustInpTypeCBox.currentIndex()==3:
                modellib.unsetTdustIdentTemp()
                self.setLabelsDustTempField(0,False)
                self.TdustInpTypeCBox.setCurrentIndex(0)
            self.TempFNameLabel.setVisible(False)
            self.TempFNameLedit.setVisible(False)
            self.TempValueLabel.setVisible(False)
            self.TempValueLedit.setVisible(False)
            self.TempFNameBrowseButton.setVisible(False)
            self.TempFNameLedit.setToolTip('')
            self.TempValueLedit.setToolTip('')
            self.TempFNameBrowseButton.setToolTip('')
        if (index==4):
            self.TempFNameLabel.setVisible(False)
            self.TempFNameLedit.setVisible(False)
            self.TempValueLabel.setVisible(False)
            self.TempValueLedit.setVisible(False)
            self.TempFNameBrowseButton.setVisible(False)
            self.TempFNameLedit.setToolTip('')
            self.TempValueLedit.setToolTip('')
            self.TempFNameBrowseButton.setToolTip('')
       
        if not self.inOpeningModel:
            if updatePlots:
                self.changeParams()

    def setLabelsAbunField(self,index=None, updatePlots=True):

        tabdataInput = False
        if (index==0):
            self.AbunFTempLabel.setVisible(False)
            self.AbunFTempLedit.setVisible(False)
            self.AbunValueLabel.setVisible(True)
            self.AbunValueLabel.setText('Value:') 
            self.AbunValueLedit.setVisible(True)
            self.AbunValueLedit.setText('1e-9')
            self.AbunRegionCustomizeButton.setVisible(False)
            self.AbunFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.AbunFTempLedit.setToolTip('')
                self.AbunValueLedit.setToolTip(str2ToolTipString('Constant Abundance value '))
                self.AbunFNameBrowseButton.setToolTip('')
        if (index==1):
            self.AbunFTempLabel.setVisible(True)
            self.AbunFTempLedit.setVisible(True)
            self.AbunValueLabel.setVisible(True)
            self.AbunValueLedit.setVisible(True)
            self.AbunValueLedit.setText('1e-9')
            self.AbunFTempLedit.setText('-1.0')

            self.AbunFTempLabel.setText('Power Exp:')
            self.AbunValueLabel.setText('Value(Rin):') 
            self.AbunRegionCustomizeButton.setVisible(False)
            self.AbunFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.AbunFTempLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
                self.AbunValueLedit.setToolTip(str2ToolTipString('Value of the Abundance parameter at Rin'))
                self.AbunFNameBrowseButton.setToolTip('')

        if (index==2):
            self.AbunFTempLabel.setVisible(False)
            self.AbunFTempLedit.setVisible(False)
            self.AbunValueLabel.setVisible(False)
            self.AbunValueLedit.setVisible(False)
            self.AbunValueLedit.setText('abundance.ARTIST.tab')
            self.AbunRegionCustomizeButton.setVisible(True)
            self.AbunFNameBrowseButton.setVisible(False)
            self.AbunFTempLedit.setToolTip('')
            self.AbunValueLedit.setToolTip('')
            self.AbunFNameBrowseButton.setToolTip('')

        if (index==3):
            tabdataInput = True
            self.AbunFTempLabel.setVisible(False)
            self.AbunFTempLedit.setVisible(False)
            self.AbunValueLabel.setVisible(True)
            self.AbunValueLabel.setText('File:')
            self.AbunValueLedit.setVisible(True)
            self.AbunValueLedit.setText('')
            self.AbunRegionCustomizeButton.setVisible(False)
            self.AbunFNameBrowseButton.setVisible(True)
            if tabdataInput:
                self.inOpeningModel=True
            if self.showToolTipHelp:
                self.AbunFTempLedit.setToolTip('')
                self.AbunValueLedit.setToolTip(str2ToolTipString('Name of the file containing the Abundance in an ARTIST Table format (For the description of the '+\
                        'file format see the documentation)'))
                self.AbunFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data (ARTIST Table format) file'))

        if index==4:
            self.AbunFTempLabel.setVisible(False)
            self.AbunFTempLedit.setVisible(False)
            self.AbunValueLabel.setVisible(False)
            self.AbunValueLedit.setVisible(False)
            self.AbunRegionCustomizeButton.setVisible(False)
            self.AbunFNameBrowseButton.setVisible(False)
            self.AbunFTempLedit.setToolTip('')
            self.AbunValueLedit.setToolTip('')
            self.AbunFNameBrowseButton.setToolTip('')

        if not self.inOpeningModel:
            if updatePlots:
                self.changeParams()
    def setLabelsVeloField(self,index=None, updatePlots=True):

        self.VeloValueLedit.setReadOnly(False)
        if (index==0):
            self.VeloFNameLabel.setVisible(False)
            self.VeloFNameLedit.setVisible(False)            
            self.VeloValueLabel.setVisible(True)
            self.VeloValueLedit.setVisible(True)
            self.VeloValueLabel.setText('Value [m/s]:')
            self.VeloValueLedit.setText('0.0')
            self.VeloFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.VeloFNameLedit.setToolTip('')
                self.VeloValueLedit.setToolTip(str2ToolTipString('Constant velocity vector value. The velocity vector is a constant scalar times the (1,0,0) vector in spherical coordinates, i.e. the velocity vector has only radial component. The sign of the velocity is negative if the vector points towards the origin of the coordinate system.   '))
                self.VeloFNameBrowseButton.setToolTip('')
        if (index==1):
            self.VeloFNameLabel.setVisible(True)
            self.VeloFNameLedit.setVisible(True)
            self.VeloValueLabel.setVisible(True)
            self.VeloValueLedit.setVisible(True)
            self.VeloFNameLabel.setText('Power Exp:')
            self.VeloValueLabel.setText('Value(Rin)[m/s]:')
            self.VeloValueLedit.setText('0.0')
            self.VeloFNameLedit.setText('0.0')
            self.VeloFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.VeloFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent. The velocity vector is given in spherical coordinates and only the radial component differs from zero. The sign of the velocity is negative if the vector points towards the origin of the coordinate system. '))
                self.VeloValueLedit.setToolTip(str2ToolTipString('Amplitude of the gas velocity (radial component in spherical coordinates) at Rin. The sign of the velocity is negative if the vector points towards the origin of the coordinate system.'))
                self.VeloFNameBrowseButton.setToolTip('')

        if (index==2):
            self.VeloFNameLabel.setVisible(False)
            self.VeloFNameLedit.setVisible(False)
            self.VeloValueLabel.setVisible(True)
            self.VeloValueLedit.setVisible(True)
            self.VeloValueLabel.setText('File:')
            self.VeloValueLedit.setText('')
            self.VeloFNameBrowseButton.setVisible(True)
            if self.showToolTipHelp:
                self.VeloFNameLedit.setToolTip('')
                self.VeloValueLedit.setToolTip(str2ToolTipString('Name of the file containing the kinetik velocity in an ARTIST Table format (For the description of the '+\
                        'file format see the documentation)'))
                self.VeloFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC-3D)'))
        if (index==3):
            self.VeloFNameLabel.setVisible(False)
            self.VeloFNameLedit.setVisible(False)
            self.VeloValueLabel.setVisible(False)
            self.VeloValueLedit.setVisible(False)
            self.VeloFNameBrowseButton.setVisible(False)
            self.VeloFNameLedit.setToolTip('')
            self.VeloValueLedit.setToolTip('')
            self.VeloFNameBrowseButton.setToolTip('')

        if not self.inOpeningModel:
            if updatePlots:
                self.changeParams()
    def setLabelsDopplerField(self,index=None, updatePlots=True):

        if (index==0):
            self.DopplerFNameLabel.setVisible(False)
            self.DopplerFNameLedit.setVisible(False)            
            self.DopplerValueLabel.setVisible(True)
            self.DopplerValueLedit.setVisible(True)
            self.DopplerValueLabel.setText('Value [m/s]:')
            self.DopplerValueLedit.setText('100.0')
            self.DopplerFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.DopplerFNameLedit.setToolTip('')
                self.DopplerValueLedit.setToolTip(str2ToolTipString('Constant Doppler b value '))
                self.DopplerFNameBrowseButton.setToolTip('')

        if (index==1):
            self.DopplerFNameLabel.setVisible(True)
            self.DopplerFNameLedit.setVisible(True)
            self.DopplerValueLabel.setVisible(True)
            self.DopplerValueLedit.setVisible(True)
            self.DopplerFNameLabel.setText('Power Exp:')
            self.DopplerValueLabel.setText('Value(Rin)[m/s]:')
            self.DopplerValueLedit.setText('100.0')
            self.DopplerFNameLedit.setText('-1.')
            self.DopplerFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.DopplerFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
                self.DopplerValueLedit.setToolTip(str2ToolTipString('Value of the Doppler b parameter at Rin'))
                self.DopplerFNameBrowseButton.setToolTip('')

        if (index==2):
            self.DopplerFNameLabel.setVisible(False)
            self.DopplerFNameLedit.setVisible(False)
            self.DopplerValueLabel.setVisible(True)
            self.DopplerValueLedit.setVisible(True)
            self.DopplerValueLabel.setText('File:')
            self.DopplerValueLedit.setText('')
            self.DopplerFNameBrowseButton.setVisible(True)
            if self.showToolTipHelp:
                self.DopplerFNameLedit.setToolTip('')
                self.DopplerValueLedit.setToolTip(str2ToolTipString('Name of the file containing the Doppler b parameter in an ARTIST Table format (For the description of the '+\
                        'file format see the documentation)'))
                self.DopplerFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data (ARTIST Table format) file'))

        if (index==3):
            self.DopplerFNameLabel.setVisible(False)
            self.DopplerFNameLedit.setVisible(False)
            self.DopplerValueLabel.setVisible(False)
            self.DopplerValueLedit.setVisible(False)
            self.DopplerFNameBrowseButton.setVisible(False)
            self.DopplerFNameLedit.setToolTip('')
            self.DopplerValueLedit.setToolTip('')
            self.DopplerFNameBrowseButton.setToolTip('')
            
        
        if not self.inOpeningModel:
            if updatePlots:
                self.changeParams()

    def setLabelsMagfield(self,index=None, updatePlots=True):


        self.MagfieldValueLedit.setReadOnly(False)
        if (index>=0)&(index<=2):
            self.MagfieldFNameLabel.setVisible(False)
            self.MagfieldFNameLedit.setVisible(False)            
            self.MagfieldValueLabel.setVisible(True)
            self.MagfieldValueLedit.setVisible(True)
            self.MagfieldValueLabel.setText('Value [G]:')
            self.MagfieldValueLedit.setText('0.0')
            self.MagfieldFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                if index==0:
                    self.MagfieldFNameLedit.setToolTip('')
                    self.MagfieldValueLedit.setToolTip(str2ToolTipString('Constant magnetic field strength value in the Cartesian x coordinate (and zero along the other two coordinates)'))
                    self.MagfieldFNameBrowseButton.setToolTip('')

                if index==1:
                    self.MagfieldFNameLedit.setToolTip('')
                    self.MagfieldValueLedit.setToolTip(str2ToolTipString('Constant magnetic field strength value in the Cartesian y coordinate (and zero along the other two coordinates)'))
                    self.MagfieldFNameBrowseButton.setToolTip('')

                if index==2:
                    self.MagfieldFNameLedit.setToolTip('')
                    self.MagfieldValueLedit.setToolTip(str2ToolTipString('Constant magnetic field strength value in the Cartesian z coordinate (and zero along the other two coordinates)'))
                    self.MagfieldFNameBrowseButton.setToolTip('')

        if (index==3):
            self.MagfieldFNameLabel.setVisible(True)
            self.MagfieldFNameLedit.setVisible(True)
            self.MagfieldValueLabel.setVisible(True)
            self.MagfieldValueLedit.setVisible(True)
            self.MagfieldFNameLabel.setText('Rmin [AU]:')
            self.MagfieldValueLabel.setText('Dipole moment:')
            self.MagfieldValueLedit.setText('0.0')
            self.MagfieldFNameLedit.setText('1.0')
            self.MagfieldFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.MagfieldFNameLedit.setToolTip(str2ToolTipString('Dipole field with dipole moment (mDipole): \n if r &gt; Rmin\n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / r**3\n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / r**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / r**3\n if r &lt; Rmin \n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / Rmin**3 \n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / Rmin**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / Rmin**3'))
                self.MagfieldValueLedit.setToolTip(str2ToolTipString('Dipole field with dipole moment (mDipole): \n if r &gt; Rmin\n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / r**3\n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / r**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / r**3\n if r &lt; Rmin \n valX = 3 * mDipole * sin(theta) * cos(theta) * cos(phi) / Rmin**3 \n valY = 3 * mDipole * sin(theta) * cos(theta) * sin(phi) / Rmin**3 \n valZ = 3 * mDipole * (cos(theta)**2 - 1/3) / Rmin**3'))
                self.MagfieldFNameBrowseButton.setToolTip('')
                
                
            

        if (index==4):
            self.MagfieldFNameLabel.setVisible(False)
            self.MagfieldFNameLedit.setVisible(False)
            self.MagfieldValueLabel.setVisible(True)
            self.MagfieldValueLedit.setVisible(True)
            self.MagfieldValueLabel.setText('File:')
            self.MagfieldValueLedit.setText('')
            self.MagfieldFNameBrowseButton.setVisible(True)
            if self.showToolTipHelp:
                self.MagfieldFNameLedit.setToolTip('')
                self.MagfieldValueLedit.setToolTip(str2ToolTipString('Name of the file containing the Magnetic field in an ARTIST Table format (For the description of the '+\
                        'file format see the documentation)'))
                self.MagfieldFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data (ARTIST Table format) file'))

        if (index==5):
            self.MagfieldFNameLabel.setVisible(False)
            self.MagfieldFNameLedit.setVisible(False)            
            self.MagfieldValueLabel.setVisible(False)
            self.MagfieldValueLedit.setVisible(False)
            self.MagfieldFNameBrowseButton.setVisible(False)
            self.MagfieldFNameLedit.setToolTip('')
            self.MagfieldValueLedit.setToolTip('')
            self.MagfieldFNameBrowseButton.setToolTip('')
 
        if not self.inOpeningModel:
            if updatePlots:
                self.changeParams()

    def setLabelsDustTempField(self,index=None, updatePlots=True):

        if (index==0):
            self.TdustFNameLabel.setVisible(False)
            self.TdustFNameLedit.setVisible(False)            
            self.TdustValueLabel.setVisible(True)
            self.TdustValueLedit.setVisible(True)
            self.TdustValueLabel.setText('Value [K]:')
            self.TdustValueLedit.setText('30.0')
            self.TdustFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.TdustFNameLedit.setToolTip('')
                self.TdustValueLedit.setToolTip(str2ToolTipString('Constant dust temperature value '))
                self.TdustFNameBrowseButton.setToolTip('')

        if (index==1):
            self.TdustFNameLabel.setVisible(True)
            self.TdustFNameLedit.setVisible(True)
            self.TdustValueLabel.setVisible(True)
            self.TdustValueLedit.setVisible(True)
            self.TdustFNameLabel.setText('Power Exp:')
            self.TdustValueLabel.setText('Value(Rin)[K]:')
            self.TdustValueLedit.setText('30.0')
            self.TdustFNameLedit.setText('-1.0')
            self.TdustFNameBrowseButton.setVisible(False)
            if self.showToolTipHelp:
                self.TdustFNameLedit.setToolTip(str2ToolTipString('Powerlaw (as a function of spherical r coordinate) exponent'))
                self.TdustValueLedit.setToolTip(str2ToolTipString('Value of the dust temperature at Rin'))
                self.TdustFNameBrowseButton.setToolTip('')

        if (index==2):
            self.TdustFNameLabel.setVisible(False)
            self.TdustFNameLedit.setVisible(False)
            self.TdustValueLabel.setVisible(True)
            self.TdustValueLedit.setVisible(True)
            self.TdustValueLabel.setText('File:')
            self.TdustValueLedit.setText('')
            self.TdustFNameBrowseButton.setVisible(True)
            if self.showToolTipHelp:
                self.TdustFNameLedit.setToolTip('')
                self.TdustValueLedit.setToolTip(str2ToolTipString('Name of the file containing the dust temperature in an ARTIST Table format (For the description of the '+\
                        'file format see the documentation)'))
                self.TdustFNameBrowseButton.setToolTip(str2ToolTipString('Open a file browser to select a data file (allowed formats: ARTIST Table, RADMC, RADMC-3D)'))
            
        if (index==3):
            if self.TempInpTypeCBox.currentIndex()==3:
                if modellib.isTempIdentTdust():
                    modellib.unsetTempIdentTdust()
                self.setLabelsTempField(0, False)
                self.TempInpTypeCBox.setCurrentIndex(0)
            self.TdustFNameLabel.setVisible(False)
            self.TdustFNameLedit.setVisible(False)            
            self.TdustValueLabel.setVisible(False)
            self.TdustValueLedit.setVisible(False)
            self.TdustFNameBrowseButton.setVisible(False)
            self.TdustFNameLedit.setToolTip('')
            self.TdustValueLedit.setToolTip('')
            self.TdustFNameBrowseButton.setToolTip('')
        
        if (index==4):
            self.TdustFNameLabel.setVisible(False)
            self.TdustFNameLedit.setVisible(False)            
            self.TdustValueLabel.setVisible(False)
            self.TdustValueLedit.setVisible(False)
            self.TdustFNameBrowseButton.setVisible(False)
            self.TdustFNameLedit.setToolTip('')
            self.TdustValueLedit.setToolTip('')
            self.TdustFNameBrowseButton.setToolTip('')

        if not self.inOpeningModel:
            if updatePlots:
                self.changeParams()


    def readOutLRTParams(self, toSaveModel=False):


        dum       = str(self.TdustDustopacFileLedit.text())

        if self.copyDataFilesToModelDir:
            dum = dum.split('/')
            self.par.dust = dum[len(dum)-1]
        else:
            self.par.dust = dum.strip()
        
        log.debug('dust = '+self.par.dust)

        if toSaveModel==False:
            if not os.path.isfile(self.par.dust):
                log.exception('dust opacity file does not exist')
                reply = QtGui.QMessageBox.critical(None,"Error", ' Dust opacity file \n'+self.par.dust+'\n does not exists',\
                                    QtGui.QMessageBox.Ok)
                return False

        
       
        if self.LRTimagemodeCBox.currentIndex()!=2:
            dum = str(self.AbunMoldatFileLedit.text())
            if self.copyDataFilesToModelDir:
                dum = dum.split('/')
                self.par.moldatfile = dum[len(dum)-1]
            else:
                self.par.moldatfile = dum.strip()
        else:
            self.par.moldatfile=""
        
        log.debug('moldatfile = '+self.par.moldatfile)


        if toSaveModel==False:
            if self.LRTimagemodeCBox.currentIndex()!=2:
                if not os.path.isfile(self.par.moldatfile):
                    log.exception('moldatfile file does not exist')
                    reply = QtGui.QMessageBox.critical(None,"Error", ' Molecule data file \n'+self.par.moldatfile+'\n does not exists',\
                                        QtGui.QMessageBox.Ok)
                    return False
            else:
                 self.par.moldatfile=''
        self.par.outputfile = str(self.LRTpopoutfileLedit.text())
        log.debug('outputfile = '+self.par.outputfile)
        self.par.sampling   = self.LRTsamplingCBox.currentIndex()
        log.debug('sampling = '+str(self.par.sampling))
        self.par.gridfile = str(self.LRTgridfileLedit.text())
        log.debug('gridfile = '+self.par.gridfile)
        
        dum = str2float(self.LRTRadiusLedit.text(), 'radius')
        if dum==None: return False
        self.par.radius = dum*self.nc.au/100.
        log.debug('radius = '+str(self.par.radius))

        dum = str2float(self.LRTtcmbLedit.text(), 'tcmb')
        if dum==None: return False
        self.par.tcmb = dum
        log.debug('tcmb = '+str(self.par.tcmb))
        
        dum  = str2float(self.LRTminScaleLedit.text(),'minScale')
        if dum==None: return False
        self.par.minScale = dum*self.nc.au/100.
        log.debug('minScale = '+str(self.par.minScale))
        
        dum = str2int(self.LRTsinkPointsLedit.text(), 'sinkPoints')
        if dum==None: return False
        self.par.sinkPoints = dum
        log.debug('sinkPoints = '+str(self.par.sinkPoints))
        
        dum = str2int(self.LRTpIntensityLedit.text(), 'pIntensity')
        if dum==None: return False
        self.par.pIntensity = dum
        log.debug('pIntensity = '+str(self.par.pIntensity))
        

        if self.LRTLTECBox.isChecked():
            self.par.lte_only = 1
        else:
            self.par.lte_only = 0
        log.debug('lte_only = '+str(self.par.lte_only))
        
        if self.LRTblendCBox.isChecked():
            self.par.blend= 1
        else:
            self.par.blend = 0
        log.debug('blend= '+str(self.par.blend))
        
        dum = str2int(self.LRTantialiasLedit.text(), 'antialias')
        if dum==None: return False
        if dum<=0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Antialias should be larger than 0', QtGui.QMessageBox.Ok)
                return False

        self.par.antialias = dum
        log.debug('antialias = '+str(self.par.antialias))

        if self.LRTpolarizationCBox.isChecked():
            if self.LRTimagemodeCBox.currentIndex()==2:
                self.par.polarization = 1
            else:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Line image mode and polarization are both selected. As of now polarization can only be calculated in the continuum.', QtGui.QMessageBox.Ok)
                return False
        else:
            self.par.polarization = 0
        log.debug('polarization= '+str(self.par.polarization))
         


        dum       = str2int(self.LRTpxlsLedit.text(), 'pxls')
        if dum==None: return False
        im_pxls = dum
        log.debug('pxls= '+str(im_pxls))

        dum  = str2float(self.LRTdistanceLedit.text(), 'distance')
        if dum==None: return False
        im_distance = dum*self.PC
        log.debug('distance= '+str(dum))

        im_unit       = self.LRTunitCBox.currentIndex()
        log.debug('image unit= '+str(im_unit))
        im_filename   = str(self.LRTfilenameLedit.text())
        log.debug('image filename= '+str(im_filename))

        dum        = str2float(self.LRTphiLedit.text(), 'phi')
        if dum==None: return False
        im_phi = dum
        log.debug('phi= '+str(dum))


        if self.LRTimagemodeCBox.currentIndex()==0:
            dum = str2float(self.LRTsourcevelLedit.text(), 'source_vel')
            if dum==None: return False
            im_source_vel = dum

            dum      = str2int(self.LRTnchanLedit.text(), 'nchan')
            if dum==None: return False
            im_nchan = dum
            if im_nchan==0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Line image cube calculation is selected, but the number of channels is 0', QtGui.QMessageBox.Ok)
                return False
            dum      = str2int(self.LRTtransLedit.text(), 'trans')
            if dum==None: return False
            im_trans = dum
            if im_trans<0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Line image cube calculation is selected, but no transition is selected', QtGui.QMessageBox.Ok)
                return False
        
            dum     = str2float(self.LRTvelresLedit.text(), 'velres')
            if dum==None: return False
            im_velres = dum
            if im_velres==0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Line image cube calculation is selected, but the velocity resolution is 0', QtGui.QMessageBox.Ok)
                return False

            im_bandwidth = -1.0
            im_freq = -1.0

        elif self.LRTimagemodeCBox.currentIndex()==1: 
            im_source_vel = 0.0

            dum      = str2int(self.LRTnchanLedit.text(), 'nchan')
            if dum==None: return False
            im_nchan = dum
            if im_nchan==0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Line image cube calculation is selected, but the number of channels is 0', QtGui.QMessageBox.Ok)
                return False

            im_trans = -1
            im_velres = -1.0
            
            dum = str2float(self.LRTbandwidthLedit.text(), 'bandwidth')
            if dum==None: return False
            im_bandwidth = dum
            if im_bandwidth==0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Line image cube calculation is selected, but the bandiwdth is 0', QtGui.QMessageBox.Ok)
                return False

            dum = str2float(self.LRTfreqLedit.text(), 'freq')
            if dum==None: return False
            im_freq = dum
            if im_bandwidth==0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Line image cube calculation is selected, but the central frequency is 0', QtGui.QMessageBox.Ok)
                return False

        elif self.LRTimagemodeCBox.currentIndex()==2: 
            im_source_vel = 0.0
            im_nchan = 0
            im_trans = -1
            im_velres = -1.0
            im_bandwidth = -1.0

            dum = str2float(self.LRTfreqLedit.text(), 'freq')
            if dum==None: return False
            im_freq = dum
            if im_freq<=0:
                reply = QtGui.QMessageBox.critical(None,"Error", 'Continuum image calculation is selected, but the central frequency is 0', QtGui.QMessageBox.Ok)
                return False

        log.debug('source_vel= '+str(im_source_vel))
        log.debug('nchan= '+str(im_nchan))
        log.debug('velres= '+str(im_velres))
        log.debug('bandwidth= '+str(im_bandwidth))
        log.debug('freq= '+str(im_freq))

        dum     = str2float(self.LRTimgresLedit.text(), 'imgres')
        if dum==None: return False
        im_imgres = dum
        log.debug('imgres= '+str(im_imgres))

        dum     = str2float(self.LRTthetaLedit.text(), 'theta')
        if dum==None: return False
        im_theta = dum/180.*np.pi
        log.debug('theta= '+str(im_theta))



        self.images = []
       
        singleImage = True
        if self.MultiImageParGroupBox.isChecked():
            if self.ImGridinclCBox.isChecked(): singleImage = False
            if self.ImGridimgresCBox.isChecked(): singleImage = False
            if self.ImGridvelresCBox.isChecked(): singleImage = False
            if self.ImGridtransCBox.isChecked(): singleImage = False

        if singleImage:
            self.img= lime.createImage()	
            self.img.pxls = im_pxls
            self.img.distance = im_distance
            self.img.unit = im_unit
            self.img.filename = im_filename
            self.img.phi = im_phi
            self.img.source_vel = im_source_vel
            self.img.nchan = im_nchan
            self.img.imgres = im_imgres
            self.img.theta = im_theta
            self.img.velres = im_velres
            self.img.trans = im_trans
            self.img.bandwidth = im_bandwidth
            self.img.freq = im_freq
        
       
            self.images.append(self.img)
        

        else:
            if self.ImGridinclCBox.isChecked(): 
                incl_val = []
                incl_nametag = str(self.ImGridFNameTag1Ledit.text())
                if self.ImGridinclArbitValRButton.isChecked():
                    dum = str(self.ImGridinclArbitValLedit.text()).split(',')
                    for ival in dum:
                        val = str2float(ival, 'inclination (multiple images)')
                        if dum==None:
                            return False
                        else:
                            incl_val.append(val)

                    self.MImagePar['incl']['aval'] = incl_val
                    self.MImagePar['incl']['style'] = 0
                    self.MImagePar['incl']['enabled'] = True

                else:
                    incl_val = [i for i in self.MImageParGrid['incl'].val]
                    self.MImagePar['incl']['enabled'] = True
                    self.MImagePar['incl']['grid'] = copy.deepcopy(self.MImageParGrid['incl'])
                    self.MImagePar['incl']['style'] = 1

            else:
                incl_val = None

            if self.ImGridimgresCBox.isChecked(): 
                imgres_val = []
                imgres_nametag = str(self.ImGridFNameTag2Ledit.text())
                if self.ImGridimgresArbitValRButton.isChecked():
                    dum = str(self.ImGridimgresArbitValLedit.text()).split(',')
                    for ival in dum:
                        val = str2float(ival, 'imgres (multiple images)')
                        if dum==None:
                            return False
                        else:
                            imgres_val.append(val)

                    self.MImagePar['imgres']['enabled'] = True
                    self.MImagePar['imgres']['aval'] = imgres_val
                    self.MImagePar['imgres']['style'] = 0
                else:
                    imgres_val = [i for i in self.MImageParGrid['imgres'].val]
                    self.MImagePar['imgres']['enabled'] = True
                    self.MImagePar['imgres']['grid'] = copy.deepcopy(self.MImageParGrid['incl'])
                    self.MImagePar['imgres']['style'] = 1

            else:
                imgres_val = None

            if self.ImGridvelresCBox.isChecked(): 
                velres_val = []
                velres_nametag = str(self.ImGridFNameTag3Ledit.text())
                if self.ImGridvelresArbitValRButton.isChecked():
                    dum = str(self.ImGridvelresArbitValLedit.text()).split(',')
                    for ival in dum:
                        val = str2float(ival, 'velocity res. (multiple images)')
                        if dum==None:
                            return False
                        else:
                            velres_val.append(val)
                    self.MImagePar['velres']['aval'] = velres_val
                    self.MImagePar['velres']['style'] = 0
                    self.MImagePar['velres']['enabled'] = True
                else:
                    velres_val = [i for i in self.MImageParGrid['velres'].val]
                    self.MImagePar['velres']['enabled'] = True
                    self.MImagePar['velres']['grid'] = copy.deepcopy(self.MImageParGrid['velres'])
                    self.MImagePar['velres']['style'] = 1
            else:
                velres_val = None

            if self.ImGridtransCBox.isChecked(): 
                trans_val = []
                trans_nametag = str(self.ImGridFNameTag4Ledit.text())
                dum = str(self.ImGridtransArbitValLedit.text()).split(',')
                for ival in dum:
                    val = str2int(ival, 'transitions (multiple images)')
                    if dum==None:
                        return False
                    else:
                        trans_val.append(val)

                self.MImagePar['trans']['enabled'] = True
                self.MImagePar['trans']['aval'] = incl_val
                self.MImagePar['trans']['style'] = 0

            else:
                trans_val = None


            NImages = -1
            if incl_val!=None:
                NImages = len(incl_val)

            if imgres_val!=None:
                if NImages>=0:
                    if NImages!=len(imgres_val):
                        if self.configParams.mImagePolicy=='Series':
                            reply = QtGui.QMessageBox.critical(None,"Error", 'If multiple images are calculated all parameters enabled to vary '+\
                                    'in different images should have equal number of elements in the input fields', QtGui.QMessageBox.Ok)
                            return False
                else:
                    NImages = len(imgres_val)
            
            if velres_val!=None:
                if NImages>=0:
                    if NImages!=len(velres_val):
                        if self.configParams.mImagePolicy=='Series':
                            reply = QtGui.QMessageBox.critical(None,"Error", 'If multiple images are calculated all parameters enabled to vary '+\
                                    'in different images should have equal number of elements in the input fields', QtGui.QMessageBox.Ok)
                            return False 
                else:
                    NImages = len(velres_val)
            
            if trans_val!=None:
                if NImages>=0:
                    if NImages!=len(trans_val):
                        if self.configParams.mImagePolicy=='Series':
                            reply = QtGui.QMessageBox.critical(None,"Error", 'If multiple images are calculated all parameters enabled to vary '+\
                                    'in different images should have equal number of elements in the input fields', QtGui.QMessageBox.Ok)
                            return False 
                else:
                    NImages = len(trans_val)

            if self.configParams.mImagePolicy=='Series':
                separator = str(self.ImGridFNameSeparatorCoBox.currentText())
                for iimag in range(NImages):
                    self.img= lime.createImage()	
                    self.img.pxls = im_pxls
                    self.img.distance = im_distance
                    self.img.unit = im_unit
                    self.img.phi = im_phi
                    self.img.source_vel = im_source_vel
                    self.img.nchan = im_nchan
                  
                    im_fname = 'image'
                    if incl_val!=None:
                        self.img.theta = incl_val[iimag] / 180.*np.pi
                        im_fname = im_fname+separator+incl_nametag+("%d"%iimag)
                    else:
                        self.img.theta =im_theta

                    if imgres_val!=None:
                        self.img.imgres = imgres_val[iimag] 
                        im_fname = im_fname+separator+imgres_nametag+("%d"%iimag)
                    else:
                        self.img.imgres = im_imgres

                    if velres_val!=None:
                        self.img.velres = velres_val[iimag]
                        im_fname = im_fname+separator+velres_nametag+("%d"%iimag)
                    else:
                        self.img.velres = im_velres

                    if trans_val!=None:
                        self.img.trans = trans_val[iimag]
                        im_fname = im_fname+separator+trans_nametag+("%d"%iimag)
                    else:
                        self.img.trans = im_trans
                    
                    if im_fname=='image': 
                        self.img.filename = im_filename
                    else:
                        self.img.filename = im_fname+'.fits'

                    self.images.append(self.img)
            else:
                separator = str(self.ImGridFNameSeparatorCoBox.currentText())
                if incl_val!=None:
                    Nincl = len(incl_val)
                else:
                    Nincl = 1

                if imgres_val!=None:
                    Nimgres = len(imgres_val)
                else:
                    Nimgres = 1

                if velres_val!=None:
                    Nvelres = len(velres_val)
                else:
                    Nvelres = 1

                if trans_val!=None:
                    Ntrans = len(trans_val)
                else:
                    Ntrans = 1

                for ip1 in range(Nincl):
                    for ip2 in range(Nimgres):
                        for ip3 in range(Nvelres):
                            for ip4 in range(Ntrans):
                                self.img= lime.createImage()	
                                self.img.pxls = im_pxls
                                self.img.distance = im_distance
                                self.img.unit = im_unit
                                self.img.phi = im_phi
                                self.img.source_vel = im_source_vel
                                self.img.nchan = im_nchan
                              
                                im_fname = 'image'
                                if incl_val!=None:
                                    self.img.theta = incl_val[ip1] / 180.*np.pi
                                    im_fname = im_fname+separator+incl_nametag+("%d"%ip1)
                                else:
                                    self.img.theta =im_theta

                                if imgres_val!=None:
                                    self.img.imgres = imgres_val[ip2] 
                                    im_fname = im_fname+separator+imgres_nametag+("%d"%ip2)
                                else:
                                    self.img.imgres = im_imgres

                                if velres_val!=None:
                                    self.img.velres = velres_val[ip3]
                                    im_fname = im_fname+separator+velres_nametag+("%d"%ip3)
                                else:
                                    self.img.velres = im_velres

                                if trans_val!=None:
                                    self.img.trans = trans_val[ip4]
                                    im_fname = im_fname+separator+trans_nametag+("%d"%ip4)
                                else:
                                    self.img.trans = im_trans
                                
                                if im_fname=='image': 
                                    self.img.filename = im_filename
                                else:
                                    self.img.filename = im_fname+'.fits'

                                self.images.append(self.img)
            
        return True
    def updateTransIDsMulti(self, trans=None):

        txt = ''
        for itr in trans: txt = txt + ',' + str(itr)
        self.ImGridtransArbitValLedit.setText(txt[1:])
    def updateTransIDsSingle(self, trans=None):

        txt = ''
        for itr in trans: txt = txt + ',' + str(itr)
        self.LRTtransLedit.setText(txt[1:])

    def setContInpFields(self):
        
        if self.LRTimagemodeCBox.currentIndex()==0:
            self.LRTtransLedit.setEnabled(True)
            self.LRTtransLabel.setEnabled(True)
            
            self.LRTbandwidthLedit.setEnabled(False)
            self.LRTbandwidthLabel.setEnabled(False)
            
            self.LRTfreqLedit.setEnabled(False)
            self.LRTfreqLabel.setEnabled(False)

            self.LRTsourcevelLabel.setEnabled(True)
            self.LRTsourcevelLedit.setEnabled(True)

            self.LRTvelresLabel.setEnabled(True)
            self.LRTvelresLedit.setEnabled(True)

            self.LRTnchanLabel.setEnabled(True)
            self.LRTnchanLedit.setEnabled(True)
            
            self.AbunMoldatFileLedit.setEnabled(True)
            self.AbunMoldatFileLabel.setEnabled(True)

        if self.LRTimagemodeCBox.currentIndex()==1:
            self.LRTtransLedit.setEnabled(False)
            self.LRTtransLabel.setEnabled(False)
            
            self.LRTbandwidthLedit.setEnabled(True)
            self.LRTbandwidthLabel.setEnabled(True)
            
            self.LRTfreqLedit.setEnabled(True)
            self.LRTfreqLabel.setEnabled(True)

            self.LRTsourcevelLabel.setEnabled(False)
            self.LRTsourcevelLedit.setEnabled(False)

            self.LRTvelresLabel.setEnabled(False)
            self.LRTvelresLedit.setEnabled(False)

            self.LRTnchanLabel.setEnabled(True)
            self.LRTnchanLedit.setEnabled(True)
            
            self.AbunMoldatFileLedit.setEnabled(True)
            self.AbunMoldatFileLabel.setEnabled(True)

        if self.LRTimagemodeCBox.currentIndex()==2:
            self.LRTtransLedit.setEnabled(False)
            self.LRTtransLabel.setEnabled(False)
            
            self.LRTbandwidthLedit.setEnabled(False)
            self.LRTbandwidthLabel.setEnabled(False)
            
            self.LRTfreqLedit.setEnabled(True)
            self.LRTfreqLabel.setEnabled(True)

            self.LRTsourcevelLabel.setEnabled(False)
            self.LRTsourcevelLedit.setEnabled(False)

            self.LRTvelresLabel.setEnabled(False)
            self.LRTvelresLedit.setEnabled(False)

            self.LRTnchanLabel.setEnabled(False)
            self.LRTnchanLedit.setEnabled(False)

            self.AbunMoldatFileLedit.setEnabled(False)
            self.AbunMoldatFileLabel.setEnabled(False)
    def setLRTParams(self,par,img):

        self.LRTRadiusLedit.setText(str(par.radius/self.nc.au*100.))
        self.LRTtcmbLedit.setText(str(par.tcmb))
        if len(par.dust.split('/'))==1:
            txt = os.getcwd()+'/'+str(par.dust)
        else:
            txt = str(par.dust)
        
        if self.inOpeningModel:
            if str(par.dust).strip()=='':
                txt = ''
        self.TdustDustopacFileLedit.setText(txt)
        self.LRTminScaleLedit.setText(str(par.minScale/self.nc.au*100.))
        if par.moldatfile.strip()!='':
            if len(par.moldatfile.split('/'))==1:
                txt = os.getcwd()+'/'+str(par.moldatfile)
            else:
                txt = str(par.moldatfile)
        else:
            txt = ''
        self.AbunMoldatFileLedit.setText(txt)
        self.LRTpopoutfileLedit.setText(str(par.outputfile))
        self.LRTsamplingCBox.setCurrentIndex(par.sampling)
        self.LRTsinkPointsLedit.setText(str(par.sinkPoints))
        self.LRTpIntensityLedit.setText(str(par.pIntensity))
        self.LRTgridfileLedit.setText(str(par.gridfile))

        if (par.lte_only==1):
            self.LRTLTECBox.setChecked(True)
        else:
            self.LRTLTECBox.setChecked(False)

        if (par.blend==1):
            self.LRTblendCBox.setChecked(True)
        else:
            self.LRTblendCBox.setChecked(False)

        self.LRTantialiasLedit.setText(str(par.antialias))

        if (par.polarization==1):
            self.LRTpolarizationCBox.setChecked(True)
        else:
            self.LRTpolarizationCBox.setChecked(False)

        self.LRTdistanceLedit.setText(str(img.distance/self.PC))
        self.LRTphiLedit.setText(str(img.phi))
        self.LRTthetaLedit.setText(str(img.theta/np.pi*180.))
                
        if img.nchan>0:
            if img.velres>0.0:
                self.LRTimagemodeCBox.setCurrentIndex(0)
            else:
                self.LRTimagemodeCBox.setCurrentIndex(1)
        else:
            self.LRTimagemodeCBox.setCurrentIndex(2)

        self.LRTnchanLedit.setText(str(img.nchan))
        self.LRTvelresLedit.setText(str(img.velres))
        self.LRTsourcevelLedit.setText(str(img.source_vel))
        self.LRTbandwidthLedit.setText(str(img.bandwidth))
        self.LRTfreqLedit.setText(str(img.freq))
        self.LRTtransLedit.setText(str(img.trans))
        
        self.LRTpxlsLedit.setText(str(img.pxls))
        self.LRTimgresLedit.setText(str(img.imgres))
        self.LRTunitCBox.setCurrentIndex(img.unit)
        self.LRTfilenameLedit.setText(str(img.filename))

    def setMultiImageWidgetVisibility(self):

        singleFileName = True
        image_tags = ''
        sep = str(self.ImGridFNameSeparatorCoBox.currentText())

        if self.MultiImageParGroupBox.isChecked():
            if self.ImGridinclCBox.isChecked():
                singleFileName = False
                self.ImGridinclArbitValRButton.setEnabled(True)
                self.ImGridinclRGridValRButton.setEnabled(True)
                self.LRTthetaLedit.setEnabled(False)
                self.LRTthetaLabel.setEnabled(False)
                self.ImGridFNameTag1Ledit.setEnabled(True)
                self.ImGridFNameTag1Label.setEnabled(True)

                dum = str(self.ImGridFNameTag1Ledit.text())
                image_tags = image_tags + sep + dum+'0'

                if self.ImGridinclArbitValRButton.isChecked():
                    self.ImGridinclArbitValLedit.setEnabled(True)
                    self.ImGridinclRGridValCBox.setEnabled(False)
                    self.ImGridinclRGridValGridParButton.setEnabled(False)
                    self.ImGridinclRGridValLabel.setEnabled(False)
                else:
                    self.ImGridinclArbitValLedit.setEnabled(False)
                    self.ImGridinclRGridValCBox.setEnabled(True)
                    self.ImGridinclRGridValGridParButton.setEnabled(True)
                    self.ImGridinclRGridValLabel.setEnabled(True)
            else:
                self.LRTthetaLedit.setEnabled(True)
                self.LRTthetaLabel.setEnabled(True)
                self.LRTimgresLabel.setEnabled(True)
                self.ImGridFNameTag1Ledit.setEnabled(False)
                self.ImGridFNameTag1Label.setEnabled(False)
                self.ImGridinclArbitValLedit.setEnabled(False)
                self.ImGridinclRGridValCBox.setEnabled(False)
                self.ImGridinclRGridValGridParButton.setEnabled(False)
                self.ImGridinclRGridValLabel.setEnabled(False)
                self.ImGridinclArbitValRButton.setEnabled(False)
                self.ImGridinclRGridValRButton.setEnabled(False)


            if self.ImGridimgresCBox.isChecked():
                singleFileName = False
                self.ImGridimgresArbitValRButton.setEnabled(True)
                self.ImGridimgresRGridValRButton.setEnabled(True)
                self.LRTimgresLedit.setEnabled(False)
                self.LRTimgresLabel.setEnabled(False)
                self.ImGridFNameTag2Ledit.setEnabled(True)
                self.ImGridFNameTag2Label.setEnabled(True)

                dum = str(self.ImGridFNameTag2Ledit.text())
                image_tags = image_tags + sep + dum+'0'

                if self.ImGridimgresArbitValRButton.isChecked():
                    self.ImGridimgresArbitValLedit.setEnabled(True)
                    self.ImGridimgresRGridValCBox.setEnabled(False)
                    self.ImGridimgresRGridValGridParButton.setEnabled(False)
                    self.ImGridimgresRGridValLabel.setEnabled(False)
                else:
                    self.ImGridimgresArbitValLedit.setEnabled(False)
                    self.ImGridimgresRGridValCBox.setEnabled(True)
                    self.ImGridimgresRGridValGridParButton.setEnabled(True)
                    self.ImGridimgresRGridValLabel.setEnabled(True)
            else:
                self.LRTimgresLedit.setEnabled(True)
                self.LRTimgresLabel.setEnabled(True)
                self.ImGridFNameTag2Ledit.setEnabled(False)
                self.ImGridFNameTag2Label.setEnabled(False)
                self.ImGridimgresArbitValLedit.setEnabled(False)
                self.ImGridimgresRGridValCBox.setEnabled(False)
                self.ImGridimgresRGridValGridParButton.setEnabled(False)
                self.ImGridimgresRGridValLabel.setEnabled(False)
                self.ImGridimgresArbitValRButton.setEnabled(False)
                self.ImGridimgresRGridValRButton.setEnabled(False)

            if self.ImGridvelresCBox.isChecked():
                singleFileName = False
                self.ImGridvelresArbitValRButton.setEnabled(True)
                self.ImGridvelresRGridValRButton.setEnabled(True)
                self.LRTvelresLedit.setEnabled(False)
                self.LRTvelresLabel.setEnabled(False)
                self.ImGridFNameTag3Ledit.setEnabled(True)
                self.ImGridFNameTag3Label.setEnabled(True)
                
                dum = str(self.ImGridFNameTag3Ledit.text())
                image_tags = image_tags + sep + dum+'0'

                if self.ImGridvelresArbitValRButton.isChecked():
                    self.ImGridvelresArbitValLedit.setEnabled(True)
                    self.ImGridvelresRGridValCBox.setEnabled(False)
                    self.ImGridvelresRGridValGridParButton.setEnabled(False)
                    self.ImGridvelresRGridValLabel.setEnabled(False)
                else:
                    self.ImGridvelresArbitValLedit.setEnabled(False)
                    self.ImGridvelresRGridValCBox.setEnabled(True)
                    self.ImGridvelresRGridValGridParButton.setEnabled(True)
                    self.ImGridvelresRGridValLabel.setEnabled(True)

            else:
                self.LRTvelresLedit.setEnabled(True)
                self.LRTvelresLabel.setEnabled(True)
                self.ImGridFNameTag3Ledit.setEnabled(False)
                self.ImGridFNameTag3Label.setEnabled(False)
                self.ImGridvelresArbitValLedit.setEnabled(False)
                self.ImGridvelresRGridValCBox.setEnabled(False)
                self.ImGridvelresRGridValGridParButton.setEnabled(False)
                self.ImGridvelresRGridValLabel.setEnabled(False)
                self.ImGridvelresArbitValRButton.setEnabled(False)
                self.ImGridvelresRGridValRButton.setEnabled(False)
            
            if self.ImGridtransCBox.isChecked():
                singleFileName = False
                self.LRTtransLedit.setEnabled(False)
                self.LRTtransLabel.setEnabled(False)
                self.ImGridFNameTag4Ledit.setEnabled(True)
                self.ImGridFNameTag4Label.setEnabled(True)
                self.LRTBrowseMoldatfileSingleButton.setEnabled(False)
                self.ImGridtransArbitValLedit.setEnabled(True)
                self.LRTBrowseMoldatfileMultiButton.setEnabled(True)
                self.ImGridtransArbitValLabel.setEnabled(True)
                
                dum = str(self.ImGridFNameTag4Ledit.text())
                image_tags = image_tags + sep + dum+'0'

            else:
                self.ImGridFNameTag4Ledit.setEnabled(False)
                self.ImGridFNameTag4Label.setEnabled(False)
                self.LRTtransLedit.setEnabled(True)
                self.LRTtransLabel.setEnabled(True)
                self.LRTBrowseMoldatfileSingleButton.setEnabled(True)
                self.ImGridtransArbitValLedit.setEnabled(False)
                self.LRTBrowseMoldatfileMultiButton.setEnabled(False)
                self.ImGridtransArbitValLabel.setEnabled(False)
                
            if singleFileName:
                self.LRTfilenameLedit.setEnabled(True)
                self.LRTfilenameLabel.setEnabled(True)
            else:
                self.LRTfilenameLedit.setEnabled(False)
                self.LRTfilenameLabel.setEnabled(False)

            if image_tags!='':
                self.ImGridFNameTemplateLabel.setText('image'+image_tags+'.fits')
            else:
                self.ImGridFNameTemplateLabel.setText('image.fits')
        else:
            self.LRTthetaLedit.setEnabled(True)
            self.LRTthetaLabel.setEnabled(True)
            
            self.LRTimgresLedit.setEnabled(True)
            self.LRTimgresLabel.setEnabled(True)
            
            self.LRTfilenameLedit.setEnabled(True)
            self.LRTfilenameLabel.setEnabled(True)
            
            self.LRTtransLedit.setEnabled(True)
            self.LRTtransLabel.setEnabled(True)
            self.LRTBrowseMoldatfileSingleButton.setEnabled(True)

            self.LRTvelresLedit.setEnabled(True)
            self.LRTvelresLabel.setEnabled(True)
            
            


    def openRGridDialogincl(self):

        dum = GridDialog(self, grid=self.MImageParGrid['incl'], varName='inclination [deg]', windowstyle=2)
        dum.ApplyGridOnlyGrid.connect(self.updateRGridincl)
        dum.exec_()
        dum.ApplyGridOnlyGrid.disconnect(self.updateRGridincl)

    def updateRGridincl(self, grid=None):
        
        self.MImageParGrid['incl'] = copy.deepcopy(grid)
        dum = [("%f"%i) for i in self.MImageParGrid['incl'].val]
        self.ImGridinclRGridValCBox.clear()
        self.ImGridinclRGridValCBox.addItems(dum)

    def openRGridDialogimgres(self):

        dum = GridDialog(self, grid=self.MImageParGrid['imgres'], varName='imgres [arcsec]', windowstyle=2)
        dum.ApplyGridOnlyGrid.connect(self.updateRGridimgres)
        dum.exec_()
        dum.ApplyGridOnlyGrid.disconnect(self.updateRGridimgres)

    def updateRGridimgres(self, grid=None):
        
        self.MImageParGrid['imgres'] = copy.deepcopy(grid)
        dum = [("%f"%i) for i in self.MImageParGrid['imgres'].val]
        self.ImGridimgresRGridValCBox.clear()
        self.ImGridimgresRGridValCBox.addItems(dum)
    def openRGridDialogvelres(self):

        dum = GridDialog(self, grid=self.MImageParGrid['velres'], varName='velres [m/s]', windowstyle=2)
        dum.ApplyGridOnlyGrid.connect(self.updateRGridvelres)
        dum.exec_()
        dum.ApplyGridOnlyGrid.disconnect(self.updateRGridvelres)

    def updateRGridvelres(self, grid=None):
        
        self.MImageParGrid['velres'] = copy.deepcopy(grid)
        dum = [("%f"%i) for i in self.MImageParGrid['velres'].val]
        self.ImGridvelresRGridValCBox.clear()
        self.ImGridvelresRGridValCBox.addItems(dum)
        
    def openImageViewerMainWindow(self):
        self.ImageViewerMainWindow.setVisible(True)
        self.ImageViewerMainWindow.setFocus()
        self.ImageViewerMainWindow.raise_()

    def openVisualizeMainWindow(self):
        
        readModelVarOK = self.readOutModelVarValues()
        if not readModelVarOK:
            reply = QtGui.QMessageBox.critical(None,"Value error", 'Visualization window cannot be opened'+\
                    ' due to value error in some of the input fields', QtGui.QMessageBox.Abort)
            if reply == QtGui.QMessageBox.Abort:
                value = None
            return 0
        self.VisualizeMainWindow.mplFigureWidget2D.canvas.PlottedModelID = self.VisualizeMainWindow.CurrentModelID
        #self.VisualizeMainWindow.resetLContGridParams()
        self.VisualizeMainWindow.reDrawContours()
        self.VisualizeMainWindow.updateFigure1DNewYRange()
        
        self.VisualizeMainWindow.replotYRangeFigure1D()
        
        self.VisualizeMainWindow.setVisible(True)
        self.VisualizeMainWindow.PGridNYLedit.setVisible(False)
        self.VisualizeMainWindow.reDrawContours()
    def startParaView(self):

        gridFileName = str(self.LRTgridfileLedit.text())

        if not os.path.isfile(gridFileName):
            reply = QtGui.QMessageBox.critical(None,"Error", ' VTK file '+gridFileName+' does not exists. '+\
                    ' Have you run the Line RT code already? VTK files are created by the Line RT code after the '+\
                    ' grid building. ', QtGui.QMessageBox.Ok)
            return 

        try:
            self.paraView = sp.Popen([self.configParams.paraViewCommand, '--data='+gridFileName])
        except Exception, e:
            reply = QtGui.QMessageBox.critical(None,"Paraview error", 'Cannot execute paraview with the command\n '+\
                    self.configParams.paraViewCommand+' --data='+gridFileName, QtGui.QMessageBox.Ok)
            if reply == QtGui.QMessageBox.Ok:
                return

    def runLime(self):

        log.debug('********************************************************************* ')
        log.debug('Preparing to run Lime ')
        log.debug('********************************************************************* ')
        readLRTParsOK = self.readOutLRTParams()
        if not readLRTParsOK:
            return

        readParamLabelsOK = self.readOutModelParamLabels()
        if not readParamLabelsOK:
            return

        readModelVarOK = self.readOutModelVarValues()
        if not readModelVarOK:
            return
        
        AU = 1.49598e11    
        PC = 3.08568025e16 
        
        log.debug('Saving model file')
        self.saveModel()

        log.debug('Starting the Lime Thread')
        self.StatusDisplay.setVisible(True)
        self.StatusDisplay._initalize()
        self.StatusDisplay._runLime(self.par, self.images, app=self.app)

        MID = modellib.getCurrentModelID()
        PID = modellib.getParamIDs(MID)
        try:
            print '*****************************************************'
            print '             DONE!!!!!!!'
            print '*****************************************************'
            log.debug('********************************************************************* ')
            log.debug('DONE')
            log.debug('********************************************************************* ')

            self.ImageViewerMainWindow.imageData.imageFName = self.img.filename
            self.ImageViewerMainWindow.ImFitsNameLedit.setText(self.ImageViewerMainWindow.imageData.imageFName) 
            self.ImageViewerMainWindow.readImage()
            self.ImageViewerMainWindow.updateChannelMap()
            if len(self.ImageViewerMainWindow.imageData.image['data'].shape)>2:
                if self.ImageViewerMainWindow.imageData.image['data'].shape[1]>1:
                    self.ImageViewerMainWindow.imageData.makeMomentMap()
                    for i in range(2):
                        self.ImageViewerMainWindow.imageData.mmap['clipMin'][i] = self.ImageViewerMainWindow.imageData.mmap['data'][:,:,i].min()
                        self.ImageViewerMainWindow.imageData.mmap['clipMax'][i] = self.ImageViewerMainWindow.imageData.mmap['data'][:,:,i].max()
                    self.ImageViewerMainWindow.imageData.mmap['clipMin'][2] = np.sqrt(self.ImageViewerMainWindow.imageData.mmap['data'][:,:,2].min())
                    self.ImageViewerMainWindow.imageData.mmap['clipMax'][2] = np.sqrt(self.ImageViewerMainWindow.imageData.mmap['data'][:,:,2].max())
                    self.ImageViewerMainWindow.ImMMapClipMaxLedit.setText(str(self.ImageViewerMainWindow.imageData.mmap['clipMax'][0]))
                    self.ImageViewerMainWindow.ImMMapClipMinLedit.setText(str(self.ImageViewerMainWindow.imageData.mmap['clipMin'][0]))
                    self.ImageViewerMainWindow.updateMomentMap()

                    self.ImageViewerMainWindow.inOpeningImage=True
                    self.ImageViewerMainWindow.updatePVDiagram()
                    self.ImageViewerMainWindow.inOpeningImage=False
                    self.ImageViewerMainWindow.imageData.pvdiag['clipMin']  = self.ImageViewerMainWindow.imageData.pvdiag['data'].min()
                    self.ImageViewerMainWindow.imageData.pvdiag['clipMax']  = self.ImageViewerMainWindow.imageData.pvdiag['data'].max()
                    self.ImageViewerMainWindow.PVDiagClipMinLedit.setText(str(self.ImageViewerMainWindow.imageData.pvdiag['clipMin']))
                    self.ImageViewerMainWindow.PVDiagClipMaxLedit.setText(str(self.ImageViewerMainWindow.imageData.pvdiag['clipMax']))

            self.openImageViewerMainWindow()
        except Exception, e:
            print "Exception:", e

    
if ('-help' in sys.argv):
    print '   '
    print ' ARTIST (Adaptable Radiative Transfer Innovations for Submillimeter Telescopes)'
    print '  '
    print ' Usage: '
    print ' artist -OPTION'
    print ' '
    print ' OPTIONS:'
    print " model modelname   : open a model with name 'modelname'"
    print ' help              : display this help'
    print ' '

    sys.exit(0)

